/* ****************************************************************************
 *
 * Copyright (c) Microsoft Corporation. 
 *
 * This source code is subject to terms and conditions of the Microsoft Permissive License. A 
 * copy of the license can be found in the License.html file at the root of this distribution. If 
 * you cannot locate the  Microsoft Permissive License, please send an email to 
 * ironruby@microsoft.com. By using this source code in any fashion, you are agreeing to be bound 
 * by the terms of the Microsoft Permissive License.
 *
 * You must not remove this notice, or any other, from this software.
 *
 *
 * ***************************************************************************/

using System;
using System.IO;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using System.Security;
using System.Text;
using Microsoft.Scripting;
using Ruby;
using Ruby.Compiler;
using Ruby.Hosting;
using Ruby.Runtime;

namespace Ruby.Builtins {

    // Exceptions that doesn't map to BCL exceptions.
    
    [RubyClass, Serializable]
    public class ScriptError : Exception {
        public ScriptError() { }
        public ScriptError(string message) : base(message) { }
        public ScriptError(string message, Exception inner) : base(message, inner) { }
        public ScriptError(MutableString message) : base((message != null) ? message.ToString() : null) { ExceptionOps.SetMessage(this, message); }

#if !SILVERLIGHT
        protected ScriptError(System.Runtime.Serialization.SerializationInfo info, System.Runtime.Serialization.StreamingContext context) 
            : base(info, context) { }
#endif
    }

    [RubyClass, Serializable]
    public class LoadError : ScriptError {
        public LoadError() { }
        public LoadError(string message) : base(message) { }
        public LoadError(string message, Exception inner) : base(message, inner) { }
        public LoadError(MutableString message) : base((message != null) ? message.ToString() : null) { ExceptionOps.SetMessage(this, message); }

#if !SILVERLIGHT
        protected LoadError(System.Runtime.Serialization.SerializationInfo info, System.Runtime.Serialization.StreamingContext context)
            : base(info, context) { }
#endif
    }

    
    [RubyClass, Serializable]
    public class NotImplementedError : ScriptError {
        public NotImplementedError() { }
        public NotImplementedError(string message) : base(message) { }
        public NotImplementedError(string message, Exception inner) : base(message, inner) { }
        public NotImplementedError(MutableString message) : base((message != null) ? message.ToString() : null) { ExceptionOps.SetMessage(this, message); }

#if !SILVERLIGHT
        protected NotImplementedError(System.Runtime.Serialization.SerializationInfo info, System.Runtime.Serialization.StreamingContext context) 
            : base(info, context) { }
#endif
    }

    [RubyClass, Serializable]
    public class SyntaxError : ScriptError {
        public SyntaxError() { }
        public SyntaxError(string message) : base(message) { }
        public SyntaxError(string message, Exception inner) : base(message, inner) { }
        public SyntaxError(MutableString message) : base((message != null) ? message.ToString() : null) { ExceptionOps.SetMessage(this, message); }

#if !SILVERLIGHT
        protected SyntaxError(System.Runtime.Serialization.SerializationInfo info, System.Runtime.Serialization.StreamingContext context) 
            : base(info, context) { }
#endif
    }

    [RubyClass, Serializable]
    public class NoMemoryError : Exception {
        public NoMemoryError() { }
        public NoMemoryError(string message) : base(message) { }
        public NoMemoryError(string message, Exception inner) : base(message, inner) { }
        public NoMemoryError(MutableString message) : base((message != null) ? message.ToString() : null) { ExceptionOps.SetMessage(this, message); }

#if !SILVERLIGHT
        protected NoMemoryError(System.Runtime.Serialization.SerializationInfo info, System.Runtime.Serialization.StreamingContext context) 
            : base(info, context) { }
#endif
    }

    [RubyClass, Serializable]
    public class SystemStackError : Exception {
        public SystemStackError() { }
        public SystemStackError(string message) : base(message) { }
        public SystemStackError(string message, Exception inner) : base(message, inner) { }
        public SystemStackError(MutableString message) : base((message != null) ? message.ToString() : null) { ExceptionOps.SetMessage(this, message); }

#if !SILVERLIGHT
        protected SystemStackError(System.Runtime.Serialization.SerializationInfo info, System.Runtime.Serialization.StreamingContext context) 
            : base(info, context) { }
#endif
    }

    [RubyClass, Serializable]
    public class EOFError : IOException {
        public EOFError() { }
        public EOFError(string message) : base(message) { }
        public EOFError(string message, Exception inner) : base(message, inner) { }
        public EOFError(MutableString message) : base((message != null) ? message.ToString() : null) { ExceptionOps.SetMessage(this, message); }

#if !SILVERLIGHT
        protected EOFError(System.Runtime.Serialization.SerializationInfo info, System.Runtime.Serialization.StreamingContext context) 
            : base(info, context) { }
#endif
    }

    [RubyClass, Serializable]
    public class FloatDomainError : ArgumentOutOfRangeException {
        public FloatDomainError() { }
        public FloatDomainError(string message) : base(message) { }
        public FloatDomainError(MutableString message) : base((message != null) ? message.ToString() : null) { ExceptionOps.SetMessage(this, message); }

#if !SILVERLIGHT
        public FloatDomainError(string message, Exception inner) : base(message, inner) { }

        protected FloatDomainError(System.Runtime.Serialization.SerializationInfo info, System.Runtime.Serialization.StreamingContext context) 
            : base(info, context) { }
#endif
    }

    [RubyClass, Serializable]
    public class LocalJumpError : SystemException {
        public LocalJumpError() { }
        public LocalJumpError(string message) : base(message) { }
        public LocalJumpError(string message, Exception inner) : base(message, inner) { }
        public LocalJumpError(MutableString message) : base((message != null) ? message.ToString() : null) { ExceptionOps.SetMessage(this, message); }

#if !SILVERLIGHT
        protected LocalJumpError(System.Runtime.Serialization.SerializationInfo info, System.Runtime.Serialization.StreamingContext context) 
            : base(info, context) { }
#endif
    }

    [RubyClass, Serializable]
    public class RegexpError : SystemException {
        public RegexpError() { }
        public RegexpError(string message) : base(message) { }
        public RegexpError(string message, Exception inner) : base(message, inner) { }
        public RegexpError(MutableString message) : base((message != null) ? message.ToString() : null) { ExceptionOps.SetMessage(this, message); }

#if !SILVERLIGHT
        protected RegexpError(System.Runtime.Serialization.SerializationInfo info, System.Runtime.Serialization.StreamingContext context) 
            : base(info, context) { }
#endif
    }

    [RubyClass, Serializable]
    public class RuntimeError : SystemException {
        public RuntimeError() { }
        public RuntimeError(string message) : base(message) { }
        public RuntimeError(string message, Exception inner) : base(message, inner) { }
        public RuntimeError(MutableString message) : base((message != null) ? message.ToString() : null) { ExceptionOps.SetMessage(this, message); }

#if !SILVERLIGHT
        protected RuntimeError(System.Runtime.Serialization.SerializationInfo info, System.Runtime.Serialization.StreamingContext context) 
            : base(info, context) { }
#endif
    }

    [RubyClass, Serializable]
    public class ThreadError : SystemException {
        public ThreadError() { }
        public ThreadError(string message) : base(message) { }
        public ThreadError(string message, Exception inner) : base(message, inner) { }
        public ThreadError(MutableString message) : base((message != null) ? message.ToString() : null) { ExceptionOps.SetMessage(this, message); }

#if !SILVERLIGHT
        protected ThreadError(System.Runtime.Serialization.SerializationInfo info, System.Runtime.Serialization.StreamingContext context) 
            : base(info, context) { }
#endif
    }

    // The following types need factory, otherwise CLR ctors are used and the mutable message is not set if "initialize" is undefined.

    [RubyExtensionClass("StandardError", typeof(SystemException), Inherits = typeof(Exception))]
    public static class SystemExceptionOps {
        [RubyConstructor]
        public static SystemException/*!*/ Factory([Optional]MutableString message) {
            SystemException result = new IOException((message != null) ? message.ToString() : null);
            ExceptionOps.SetMessage(result, message);
            return result;
        }
    }

    [RubyExtensionClass("ArgumentError", typeof(ArgumentException), Inherits = typeof(SystemException))]
    public static class ArgumentErrorOps {
        [RubyConstructor]
        public static ArgumentException/*!*/ Factory([Optional]MutableString message) {
            ArgumentException result = new ArgumentException((message != null) ? message.ToString() : null);
            ExceptionOps.SetMessage(result, message);
            return result;
        }
    }

    [RubyExtensionClass("IOError", typeof(IOException), Inherits = typeof(SystemException))]
    public static class IOErrorOps {
        [RubyConstructor]
        public static IOException/*!*/ Factory([Optional]MutableString message) { 
            IOException result = new IOException((message != null) ? message.ToString() : null);
            ExceptionOps.SetMessage(result, message); 
            return result;
        }
    }

    [RubyExtensionClass("IndexError", typeof(IndexOutOfRangeException), Inherits = typeof(SystemException))]
    public static class IndexErrorOps {
        [RubyConstructor]
        public static IndexOutOfRangeException/*!*/ Factory([Optional]MutableString message) {
            IndexOutOfRangeException result = new IndexOutOfRangeException((message != null) ? message.ToString() : null);
            ExceptionOps.SetMessage(result, message);
            return result;
        }
    }

    [RubyExtensionClass("RangeError", typeof(ArgumentOutOfRangeException), Inherits = typeof(SystemException))]
    public static class RangeErrorOps {
        [RubyConstructor]
        public static ArgumentOutOfRangeException/*!*/ Factory([Optional]MutableString message) {
            ArgumentOutOfRangeException result = new ArgumentOutOfRangeException((message != null) ? message.ToString() : null);
            ExceptionOps.SetMessage(result, message);
            return result;
        }
    }

    [RubyExtensionClass("NameError", typeof(MemberAccessException), Inherits = typeof(SystemException))]
    public static class NameErrorOps {
        [RubyConstructor]
        public static MemberAccessException/*!*/ Factory([Optional]MutableString message) {
            MemberAccessException result = new MemberAccessException((message != null) ? message.ToString() : null);
            ExceptionOps.SetMessage(result, message);
            return result;
        }
    }

    [RubyExtensionClass("NoMethodError", typeof(MissingMethodException), Inherits = typeof(MemberAccessException))]
    public static class NoMethodErrorOps {
        [RubyConstructor]
        public static MissingMethodException/*!*/ Factory([Optional]MutableString message) {
            MissingMethodException result = new MissingMethodException((message != null) ? message.ToString() : null);
            ExceptionOps.SetMessage(result, message);
            return result;
        }
    }

    [RubyExtensionClass("SecurityError", typeof(SecurityException), Inherits = typeof(SystemException))]
    public static class SecurityErrorOps {
        [RubyConstructor]
        public static SecurityException/*!*/ Factory([Optional]MutableString message) {
            SecurityException result = new SecurityException((message != null) ? message.ToString() : null);
            ExceptionOps.SetMessage(result, message);
            return result;
        }
    }

    [RubyExtensionClass("SystemCallError", typeof(ExternalException), Inherits = typeof(SystemException))]
    public static class SystemCallErrorOps {
        [RubyConstructor]
        public static ExternalException/*!*/ Factory([Optional]MutableString message) {
            ExternalException result = new ExternalException((message != null) ? message.ToString() : null);
            ExceptionOps.SetMessage(result, message);
            return result;
        }
    }

    [RubyExtensionClass("TypeError", typeof(InvalidOperationException), Inherits = typeof(SystemException))]
    public static class TypeErrorOps {
        [RubyConstructor]
        public static InvalidOperationException/*!*/ Factory([Optional]MutableString message) {
            InvalidOperationException result = new InvalidOperationException((message != null) ? message.ToString() : null);
            ExceptionOps.SetMessage(result, message);
            return result;
        }
    }

    [RubyExtensionClass("ZeroDivisionError", typeof(DivideByZeroException), Inherits = typeof(SystemException))]
    public static class ZeroDivisionErrorOps {
        [RubyConstructor]
        public static DivideByZeroException/*!*/ Factory([Optional]MutableString message) {
            DivideByZeroException result = new DivideByZeroException((message != null) ? message.ToString() : null);
            ExceptionOps.SetMessage(result, message);
            return result;
        }
    }
}
