/* ****************************************************************************
 *
 * Copyright (c) Microsoft Corporation. 
 *
 * This source code is subject to terms and conditions of the Microsoft Permissive License. A 
 * copy of the license can be found in the License.html file at the root of this distribution. If 
 * you cannot locate the  Microsoft Permissive License, please send an email to 
 * ironruby@microsoft.com. By using this source code in any fashion, you are agreeing to be bound 
 * by the terms of the Microsoft Permissive License.
 *
 * You must not remove this notice, or any other, from this software.
 *
 *
 * ***************************************************************************/

using System;
using System.Collections.Generic;
using System.Text;
using Ruby.Runtime;
using Microsoft.Scripting.Math;
using Microsoft.Scripting;

namespace Ruby.Builtins {

    [RubyExtensionClass("Fixnum", typeof(int), Inherits = typeof(Integer))]
    public static class FixnumOps {

        #region Public Instance Methods

        [RubyMethodAttribute("+", RubyMethodAttributes.PublicInstance)]
        public static object Add(int self, int value) {
            // TODO: overflow
            return self + value;
        }

        [RubyMethodAttribute("+", RubyMethodAttributes.PublicInstance)]
        public static BigInteger Add(int self, BigInteger value) {
            // TODO:
            return null;
        }

        [RubyMethodAttribute("-", RubyMethodAttributes.PublicInstance)]
        public static object Subtract(int self, int value) {
            return self - value;
        }

        [RubyMethodAttribute("*", RubyMethodAttributes.PublicInstance)]
        public static object Multiply(int self, int value) {
            return self * value;
        }

        [RubyMethodAttribute("/", RubyMethodAttributes.PublicInstance)]
        public static object Divide(int self, int value) {
            return self / value;
        }

        [RubyMethodAttribute(">", RubyMethodAttributes.PublicInstance)]
        public static bool GraterThan(int self, int value) {
            return self > value;
        }

        [RubyMethodAttribute(">=", RubyMethodAttributes.PublicInstance)]
        public static bool GreaterThanOrEqual(int self, int value) {
            return self >= value;
        }

        [RubyMethodAttribute("<", RubyMethodAttributes.PublicInstance)]
        public static bool LessThan(int self, int value) {
            return self < value;
        }

        [RubyMethodAttribute("<=", RubyMethodAttributes.PublicInstance)]
        public static bool LessThanOrEqual(int self, int value) {
            return self <= value;
        }

        // TODO: in all heterogeneous type comparisons we need to handle 
        // a) implicit conversions that are legal and
        // b) return false for cases where the other type is not convertible

        //%
        //&
        //**
        //-@
        //<
        //<<
        //<=>
        //>>
        //[]
        //^
        //abs
        //div
        //divmod
        //id2name
        //modulo
        //power!
        //quo
        //rdiv
        //rpower
        //size
        //to_f
        //to_s
        //to_sym
        //zero?
        //|
        //~

        [RubyMethodAttribute("==", RubyMethodAttributes.PublicInstance)]
        public static bool Equal(int lhs, int rhs) {
            return lhs == rhs;
        }

        [RubyMethodAttribute("==", RubyMethodAttributes.PublicInstance)]
        public static bool Equal(int lhs, object rhs) {
            return false;
        }

        [RubyMethodAttribute("<=>", RubyMethodAttributes.PublicInstance)]
        public static int Compare(int lhs, int rhs) {
            return lhs.CompareTo(rhs);
        }

        [RubyMethodAttribute("inspect", RubyMethodAttributes.PublicInstance)]
        public static MutableString Inspect(int self) {
            return new MutableString(self.ToString());
        }

        [RubyMethodAttribute("to_i", RubyMethodAttributes.PublicInstance)]
        public static object ToInteger(object self) {
            return self;
        }

        [RubyMethodAttribute("to_s", RubyMethodAttributes.PublicInstance)]
        public static object ToString(object self) {
            return new MutableString(self.ToString());
        }

        #endregion
    }
}
