/* ****************************************************************************
 *
 * Copyright (c) Microsoft Corporation. 
 *
 * This source code is subject to terms and conditions of the Microsoft Permissive License. A 
 * copy of the license can be found in the License.html file at the root of this distribution. If 
 * you cannot locate the  Microsoft Permissive License, please send an email to 
 * ironruby@microsoft.com. By using this source code in any fashion, you are agreeing to be bound 
 * by the terms of the Microsoft Permissive License.
 *
 * You must not remove this notice, or any other, from this software.
 *
 *
 * ***************************************************************************/

using System;
using System.Collections.Generic;
using System.Text;
using Ruby.Runtime;
using System.Collections;
using System.Diagnostics;
using Microsoft.Scripting;

namespace Ruby.Builtins {

    /// <summary>
    /// Dictionary inherits from Object, mixes in Enumerable.
    /// Ruby hash is Dictionary{object, object}.
    /// </summary>
    [RubyExtensionClass("Hash", typeof(Dictionary<object, object>)), Includes(typeof(Enumerable))]
    public static class HashOps {

        #region Singleton Methods

        [RubyMethodAttribute("[]", RubyMethodAttributes.PublicSingleton)]
        public static Dictionary<object, object>/*!*/ CreateHash([NotNull]params object[] items) {
            // arg0: self
            // arg1: hash or first elem
            // argk: k-th elem

            int itemCount = items.Length - 1;

            if (itemCount % 2 != 0) {
                Dictionary<object, object> hash;
                if (items.Length == 2 && (hash = items[1] as Dictionary<object, object>) != null) {
                    return hash;
                }

                throw new ArgumentException("odd number of arguments for Hash");
            }

            return RubyOps.MakeHash(Utils.Array.RemoveFirst(items));
        }

        #endregion

        #region Singleton Methods

        // TODO:
        //[RubyMethodAttribute("[]", MethodAttributes.PublicInstance)]
        //public static object GetElement(object self, object key) {
        //    List<object> list = (List<object>)self;
        //    int ikey = (int)key;

        //    if (ikey < 0) return list[list.Count + ikey];
        //    if (ikey < list.Count) return list[ikey];
        //    return null;
        //}

        //[RubyMethodAttribute("[]=", MethodAttributes.PublicInstance)]
        //public static object SetElement(object self, object key, object value) {
        //    List<object> list = (List<object>)self;
        //    int ikey = (int)key;

        //    if (ikey < 0) ikey = list.Count + ikey;
        //    else if (ikey >= list.Count) list.Capacity = ikey + 1;

        //    list[ikey] = value;
        //    return value;
        //}

        #endregion


        [RubyMethodAttribute("inspect", RubyMethodAttributes.PublicInstance)]
        public static MutableString Inspect(Dictionary<object, object> self) {
            throw new NotImplementedError();
        }

        // TODO: handle recursion correctly
        [RubyMethodAttribute("to_s", RubyMethodAttributes.PublicInstance)]
        public static MutableString ToS(Dictionary<object, object> self) {
            MutableString result = new MutableString();
            foreach (KeyValuePair<object, object> item in self) {
                result.Append(RubySites.ToS(item.Key));
                result.Append(RubySites.ToS(item.Value));
            }
            return result;
        }
    }
}
