/* ****************************************************************************
 *
 * Copyright (c) Microsoft Corporation. 
 *
 * This source code is subject to terms and conditions of the Microsoft Permissive License. A 
 * copy of the license can be found in the License.html file at the root of this distribution. If 
 * you cannot locate the  Microsoft Permissive License, please send an email to 
 * ironruby@microsoft.com. By using this source code in any fashion, you are agreeing to be bound 
 * by the terms of the Microsoft Permissive License.
 *
 * You must not remove this notice, or any other, from this software.
 *
 *
 * ***************************************************************************/

using System;
using System.Collections.Generic;
using System.Text;
using Ruby.Runtime;
using Microsoft.Scripting;
using Microsoft.Scripting.Math;
using Microsoft.Scripting.Actions;

namespace Ruby.Builtins {

    [RubyClass]
    public abstract class Integer : Numeric {

        // ancestors: Integer,Precision,Numeric,Comparable,Object,Kernel,        

        #region Public Instance Methods

        //ceil
        //chr
        //downto
        //floor
        //integer?
        //next
        //round
        //succ

        [RubyMethodAttribute("succ", RubyMethodAttributes.PublicInstance)]
        public static int Succ(int/*!*/ instance) {
            return ((int)instance) + 1;
        }

        private static DynamicSite<Proc, BlockReturnReasonHolder, int, object> TimesSharedSite =
            DynamicSite<Proc, BlockReturnReasonHolder, int, object>.Create(CallAction.Simple);
 
        [RubyMethodAttribute("times", RubyMethodAttributes.PublicInstance)]
        public static object Times(CodeContext/*!*/ context, int self, Proc/*!*/ block) {
            if (context == null) throw new ArgumentNullException("context");
            if (block == null) throw new ArgumentNullException("block");
            
            BlockReturnReasonHolder returnReason = new BlockReturnReasonHolder();
            object result;
            int i = 0;
            while (i < self) {
                try {
                    result = TimesSharedSite.Invoke(context, block, returnReason, i);
                } catch (StackUnwinder.BlockReturn u) {
                    result = u.ReturnValue;
                    returnReason.Value = u.Reason;
                }

                switch(returnReason.Value) {
                    case BlockReturnReason.Undefined:
                        break;

                    case BlockReturnReason.Next:
                        returnReason.Value = BlockReturnReason.Undefined;
                        break;

                    case BlockReturnReason.BreakOrRetry:
                        return result;

                    case BlockReturnReason.Redo:
                        returnReason.Value = BlockReturnReason.Undefined;
                        continue;
                }

                i++;
            }

            return self;
        }

        [RubyMethodAttribute("times", RubyMethodAttributes.PublicInstance)]
        public static object Times(CodeContext/*!*/ context, BigInteger/*!*/ instance, Proc/*!*/ block) {
            throw new NotImplementedException();
        }

        //to_i
        //to_int
        //truncate
        //upto

        #endregion

        #region Singleton Methods

        // induced_from

        #endregion
    }
}
