/* ****************************************************************************
 *
 * Copyright (c) Microsoft Corporation. 
 *
 * This source code is subject to terms and conditions of the Microsoft Permissive License. A 
 * copy of the license can be found in the License.html file at the root of this distribution. If 
 * you cannot locate the  Microsoft Permissive License, please send an email to 
 * ironruby@microsoft.com. By using this source code in any fashion, you are agreeing to be bound 
 * by the terms of the Microsoft Permissive License.
 *
 * You must not remove this notice, or any other, from this software.
 *
 *
 * ***************************************************************************/

using System;
using System.Collections.Generic;
using System.Text;

using Ruby.Runtime;
using Microsoft.Scripting;
using System.Diagnostics;
using Ruby.Hosting;
using Ruby.Compiler;
using System.Collections;
using System.IO;
using System.Runtime.InteropServices;

namespace Ruby.Builtins {
    
    [RubyModuleAttribute]
    public static class Kernel {

        // obsolete:
        //[RubyMethodAttribute("ruby_case_set", RubyMethodAttributes.PublicSingleton)]
        //public static object RubyCaseSet(object self, object value) {
        //    RubyEngine.CurrentEngine.RubyCaseMemberLists = (bool)value;
        //    return null;
        //}

        #region Private Instance Methods

        //active_gem_with_options
        //gem_original_require
        //initialize_copy
        //location_of_caller
        //remove_instance_variable
        //singleton_method_added
        //singleton_method_removed
        //singleton_method_undefined
        //y
      
        #endregion

        #region Private Instance & Singleton Methods

        //Array
        //Float
        //Integer
        //String
        //URI
        //`
        //abort
        //at_exit
        //autoload
        //autoload?
        //binding
        //block_given?
        //callcc
        //caller
        //catch
        //chomp
        //chomp!
        //chop
        //chop!

        // TODO:
        //[RubyMethodAttribute("eval", RubyMethodAttributes.PrivateInstance)]
        //[RubyMethodAttribute("eval", RubyMethodAttributes.PublicSingleton)]
        //public static object Evaluate(CodeContext/*!*/ context, object self, object/*!*/ code) {
        //    Debug.Assert(context != null);

        //    SourceCodeUnit sourceUnit = new SourceCodeUnit(RubyEngine.CurrentEngine, (string)code);

        //    ScriptCode compiledCode = ScriptCode.FromCompiledCode(sourceUnit.Compile(
        //        context.LanguageContext.GetCompilerOptions(), // flows the current lexical scope if available
        //        RubyEngine.CurrentEngine.RuntimeErrorSink)
        //    );

        //    return compiledCode.Run(context.Scope, context.ModuleContext);
        //}
        
        //exec

        [RubyMethodAttribute("exit", RubyMethodAttributes.PrivateInstance)]
        [RubyMethodAttribute("exit", RubyMethodAttributes.PublicSingleton)]
        public static void Exit(object self) {
            Exit(self, 1);
        }

        [RubyMethodAttribute("exit", RubyMethodAttributes.PrivateInstance)]
        [RubyMethodAttribute("exit", RubyMethodAttributes.PublicSingleton)]
        public static void Exit(object self, bool isSuccessful) {
            Exit(self, isSuccessful ? 0 : 1);
        }

        [RubyMethodAttribute("exit", RubyMethodAttributes.PrivateInstance)]
        [RubyMethodAttribute("exit", RubyMethodAttributes.PublicSingleton)]
        public static void Exit(object self, int exitCode) {
            ScriptDomainManager.CurrentManager.PAL.TerminateScriptExecution(exitCode);
            // should throw SystemExit exception
        }

        [RubyMethodAttribute("exit!", RubyMethodAttributes.PrivateInstance)]
        [RubyMethodAttribute("exit!", RubyMethodAttributes.PublicSingleton)]
        public static void TerminateExecution(object self) {
            Exit(self, 1);
        }

        [RubyMethodAttribute("exit!", RubyMethodAttributes.PrivateInstance)]
        [RubyMethodAttribute("exit!", RubyMethodAttributes.PublicSingleton)]
        public static void TerminateExecution(object self, bool isSuccessful) {
            Exit(self, isSuccessful ? 0 : 1);
        }

        [RubyMethodAttribute("exit!", RubyMethodAttributes.PrivateInstance)]
        [RubyMethodAttribute("exit!", RubyMethodAttributes.PublicSingleton)]
        public static void TerminateExecution(object self, int exitCode) {
            ScriptDomainManager.CurrentManager.PAL.TerminateScriptExecution(exitCode);
        }

        //fork
        //format
        //getc
        //gets
        //global_variables
        //gsub
        //gsub!
        //iterator?
        //lambda

        [RubyMethodAttribute("load", RubyMethodAttributes.PrivateInstance)]
        [RubyMethodAttribute("load", RubyMethodAttributes.PublicSingleton)]
        public static object Load(CodeContext/*!*/ context, object self, object/*!*/ libraryName) {
            return RubyEngine.CurrentEngine.Loader.LoadFile(context, self, libraryName.ToString(), LoadFlags.None);
        }

        [RubyMethodAttribute("load", RubyMethodAttributes.PrivateInstance)]
        [RubyMethodAttribute("load", RubyMethodAttributes.PublicSingleton)]
        public static object Load(CodeContext/*!*/ context, object self, object/*!*/ libraryName, object wrap) {
            return RubyEngine.CurrentEngine.Loader.LoadFile(context, self, libraryName.ToString(), 
                ((bool)wrap) ? LoadFlags.LoadIsolated : LoadFlags.None);
        }

        //local_variables
        //loop

        [RubyMethodAttribute("method_missing", RubyMethodAttributes.PrivateInstance)]
        [RubyMethodAttribute("method_missing", RubyMethodAttributes.PublicSingleton)]
        public static object MethodMissing(CodeContext/*!*/ context, object self, SymbolId name, [NotNull]params object[]/*!*/ args) {
            return MethodMissing(context, self, null, name, args);
        }

        [RubyMethodAttribute("method_missing", RubyMethodAttributes.PrivateInstance)]
        [RubyMethodAttribute("method_missing", RubyMethodAttributes.PublicSingleton)]
        public static object MethodMissing(CodeContext/*!*/ context, object self, Proc proc, SymbolId name, [NotNull]params object[]/*!*/ args) {
            // TODO: is this correct?
            RubyExecutionContext ec = RubyOps.GetExecutionContext(context);
            throw new MissingMethodException(String.Format("undefined local variable or method `{0}' for {1}:{2}", 
                SymbolTable.IdToString(name), RubySites.ToS(self), ec.GetClassOf(self)));
        }

        //open
        //p

        [RubyMethodAttribute("print", RubyMethodAttributes.PrivateInstance)]
        [RubyMethodAttribute("print", RubyMethodAttributes.PublicSingleton)]
        public static void Print(CodeContext/*!*/ context, object self, [NotNull]params object[]/*!*/ args) {
            if (args == null) args = RubyOps.NullArray;
            
            for (int i = 0; i < args.Length; i++) {
                // TODO: common output redirectable in Silverlight?
                Console.Write(ToString(context, args[i]));
                //IronPython.Runtime.Operations.PythonOps.Write(IronPython.Runtime.SystemState.Instance.stdout, ToString(context, args[i]));
            }
        }

        //printf
        //proc
        //putc
        
        [RubyMethodAttribute("puts", RubyMethodAttributes.PrivateInstance)]
        [RubyMethodAttribute("puts", RubyMethodAttributes.PublicSingleton)]
        public static void PutString(object self, MutableString str) {
            Console.Write(str.ToString() + "\n");
        }
        
        [RubyMethodAttribute("puts", RubyMethodAttributes.PrivateInstance)]
        [RubyMethodAttribute("puts", RubyMethodAttributes.PublicSingleton)]
        public static void PutString(CodeContext/*!*/ context, object self, [NotNull]params object[] args) {
            if (args == null) args = RubyOps.NullArray;
            
            for (int i = 0; i < args.Length; i++) {
                // TODO: common output redirectable in Silverlight?
                Console.Write(ToString(context, args[i]) + "\n");
                //IronPython.Runtime.Operations.PythonOps.Write(IronPython.Runtime.SystemState.Instance.stdout, ToString(context, args[i]) + "\n");
            }
        }

        public static string/*!*/ ToString(CodeContext/*!*/ context, object obj) {
            MutableString str = new MutableString();
            
            List<object> list = obj as List<object>;
            Dictionary<object, object> hash;
            if (list != null) {
                ArrayOps.RecursiveJoin(list, "\n", str, new Dictionary<object, bool>());
            } else if ((hash = obj as Dictionary<object, object>) != null) {
                
                foreach (KeyValuePair<object, object> entry in hash) {
                    str.Append(ToString(context, entry.Key));
                    str.Append(ToString(context, entry.Value));
                }
            } else if (obj == null) {
                str.Append("nil");
            } else if (obj is bool) {
                str.Append((bool)obj ? "true" : "false");
            } else if (obj is double) {
                // TODO:
#if SILVERLIGHT
                str.Append(obj);
#else
                if (System.Math.Truncate((double)obj) == (double)obj) {
                    str.Append(obj);
                    str.Append(".0");
                } else {
                    str.Append(obj);
                }
#endif
            } else {
                str.Append(obj);
            }
            return str.ToString();
        }

        //public static string/*!*/ ToRepr(object obj) {
        //    StringBuilder sb = new StringBuilder();

        //    List<object> list = obj as List<object>;
        //    string str = obj as string;

        //    if (list != null) {
        //        sb.Append('[');
        //        bool first = true;
        //        foreach (object item in list) {
        //            if (!first) sb.Append(',');
        //            sb.Append(GetRepresentation(item));
        //            first = false;
        //        }
        //        sb.Append(']');
        //    } else {
        //        if (obj is string) {
                    
        //        }
        //    }
        //}

        [RubyMethodAttribute("raise", RubyMethodAttributes.PrivateInstance)]
        [RubyMethodAttribute("raise", RubyMethodAttributes.PublicSingleton)]
        [RubyMethodAttribute("fail", RubyMethodAttributes.PrivateInstance)]
        [RubyMethodAttribute("fail", RubyMethodAttributes.PublicSingleton)]
        public static void RaiseException(CodeContext/*!*/ context, object self) {
            RubyExecutionContext ec = RubyOps.GetExecutionContext(context);

            Exception exceptionClass = ec.CurrentException;
            if (exceptionClass != null) {
                throw ExceptionOps.CreateException(context, ec.GetClassOf(exceptionClass));
            }

            throw new RuntimeError();
        }

        [RubyMethodAttribute("raise", RubyMethodAttributes.PrivateInstance)]
        [RubyMethodAttribute("raise", RubyMethodAttributes.PublicSingleton)]
        [RubyMethodAttribute("fail", RubyMethodAttributes.PrivateInstance)]
        [RubyMethodAttribute("fail", RubyMethodAttributes.PublicSingleton)]
        public static void RaiseException(object self, [NotNull]MutableString/*!*/ message) {
            if (message == null) throw new ArgumentNullException("message");
            throw new RuntimeError(message);
        }

        [RubyMethodAttribute("raise", RubyMethodAttributes.PrivateInstance)]
        [RubyMethodAttribute("raise", RubyMethodAttributes.PublicSingleton)]
        [RubyMethodAttribute("fail", RubyMethodAttributes.PrivateInstance)]
        [RubyMethodAttribute("fail", RubyMethodAttributes.PublicSingleton)]
        public static void RaiseException(CodeContext/*!*/ context, object self, [NotNull]RubyClass/*!*/ exceptionClass, [Optional]MutableString message, [Optional]List<object> stackTrace) {
            throw ExceptionOps.MakeException(context, exceptionClass, message, stackTrace);
        }

        [RubyMethodAttribute("raise", RubyMethodAttributes.PrivateInstance)]
        [RubyMethodAttribute("raise", RubyMethodAttributes.PublicSingleton)]
        [RubyMethodAttribute("fail", RubyMethodAttributes.PrivateInstance)]
        [RubyMethodAttribute("fail", RubyMethodAttributes.PublicSingleton)]
        public static void RaiseException(CodeContext/*!*/ context, object self, object/*!*/ exceptionObject, [Optional]MutableString message, [Optional]List<object> stackTrace) {
            // exceptionObject <# { exception: void -> Exception }

            if (RubySites.RespondTo(exceptionObject, "exception")) {
                Exception e = RubySites.Exception(exceptionObject);
                if (e != null) {
                    throw e;
                }
            } 

            throw RubyExceptions.CreateTypeError("exception class/object expected");
        }

        //rand
        //readline
        //readlines
        //scan
        //select
        //set_trace_func
        //sleep
        //split
        //sprintf
        //srand
        //sub
        //sub!
        //syscall
        //system
        //test
        //throw
        //trace_var
        //trap
        //untrace_var
        //warn
        
        #endregion

        #region Public Instance Methods

        //==
        //===
        //=~
        //__id__
        //__send__
        //class
        //clone
        //display
        //dup
        //eql?
        //equal?
        //extend
        //freeze
        //frozen?
        //gem
        //hash
        //id
        //inspect
        //instance_eval
        //instance_of?
        //instance_variable_get
        //instance_variable_set
        //instance_variables
        //is_a?
        //kind_of?
        //method
        //methods
        //nil?
        //object_id
        //private_methods
        //protected_methods
        //public_methods
        //respond_to?
        //send
        //singleton_methods
        //taint
        //tainted?
        //to_a
        //to_s
        //type
        //untaint

        #endregion

        #region Public Instance & Singleton Methods

        [RubyMethodAttribute("require", RubyMethodAttributes.PublicInstance)]
        [RubyMethodAttribute("require", RubyMethodAttributes.PublicSingleton)]
        public static bool Require(CodeContext/*!*/ context, object self, string/*!*/ libraryName) {
            // TODO: Remove Demo hack
            if (libraryName == "System.Windows.Forms")
                libraryName = "System.Windows.Forms, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089";
            else if (libraryName == "System.Drawing")
                libraryName = "System.Drawing, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a";

            return RubyEngine.CurrentEngine.Loader.LoadFile(context, self, libraryName, LoadFlags.LoadOnce);
        }

        [RubyMethodAttribute("require", RubyMethodAttributes.PublicInstance)]
        [RubyMethodAttribute("require", RubyMethodAttributes.PublicSingleton)]
        public static bool Require(CodeContext/*!*/ context, object self, MutableString/*!*/ libraryName) {
            return Require(context, self, libraryName.ToString());
        }
        #endregion
    }
}

