/* ****************************************************************************
 *
 * Copyright (c) Microsoft Corporation. 
 *
 * This source code is subject to terms and conditions of the Microsoft Permissive License. A 
 * copy of the license can be found in the License.html file at the root of this distribution. If 
 * you cannot locate the  Microsoft Permissive License, please send an email to 
 * ironruby@microsoft.com. By using this source code in any fashion, you are agreeing to be bound 
 * by the terms of the Microsoft Permissive License.
 *
 * You must not remove this notice, or any other, from this software.
 *
 *
 * ***************************************************************************/
using System;
using System.Collections.Generic;
using System.Text;
using Ruby.Runtime;
using SM = System.Math;

namespace Ruby.Builtins {

    [RubyModule]
    public static class Math {

        [RubyConstant]
        public const double E = System.Math.E;

        [RubyConstant]
        public const double PI = System.Math.PI;

        #region Private Implementation Details

        private static double DomainCheck(double result, string/*!*/ functionName) {
            if (double.IsNaN(result)) {
                throw new Errno.DomainError("Domain error - " + functionName);
            }
            return result;
        }

        #endregion

        #region Private Instance & Singleton Methods

        [RubyMethodAttribute("acos", RubyMethodAttributes.PrivateInstance)]
        [RubyMethodAttribute("acos", RubyMethodAttributes.PublicSingleton)]
        public static double Acos(object self, double x) {
            return DomainCheck(SM.Acos(x), "acos");
        }

        // acosh

        [RubyMethodAttribute("asin", RubyMethodAttributes.PrivateInstance)]
        [RubyMethodAttribute("asin", RubyMethodAttributes.PublicSingleton)]
        public static double Asin(object self, double x) {
            return DomainCheck(SM.Asin(x), "asin");
        }

        // asinh

        [RubyMethodAttribute("atan", RubyMethodAttributes.PrivateInstance)]
        [RubyMethodAttribute("atan", RubyMethodAttributes.PublicSingleton)]
        public static double Atan(object self, double x) {
            return SM.Atan(x);
        }

        [RubyMethodAttribute("atan2", RubyMethodAttributes.PrivateInstance)]
        [RubyMethodAttribute("atan2", RubyMethodAttributes.PublicSingleton)]
        public static double Atan2(object self, double y, double x) {
            return SM.Atan2(y, x);
        }
        
        //    atanh
        
        [RubyMethodAttribute("cos", RubyMethodAttributes.PrivateInstance)]
        [RubyMethodAttribute("cos", RubyMethodAttributes.PublicSingleton)]
        public static double Cos(object self, double x) {
            return SM.Cos(x);
        }

        [RubyMethodAttribute("cosh", RubyMethodAttributes.PrivateInstance)]
        [RubyMethodAttribute("cosh", RubyMethodAttributes.PublicSingleton)]
        public static double Cosh(object self, double x) {
            return SM.Cosh(x);
        }

        //    erf
        //    erfc

        [RubyMethodAttribute("exp", RubyMethodAttributes.PrivateInstance)]
        [RubyMethodAttribute("exp", RubyMethodAttributes.PublicSingleton)]
        public static double Exp(object self, double x) {
            return SM.Exp(x);
        }

        //    frexp
        //    hypot
        //    ldexp

        [RubyMethodAttribute("log", RubyMethodAttributes.PrivateInstance)]
        [RubyMethodAttribute("log", RubyMethodAttributes.PublicSingleton)]
        public static double Log(object self, double x) {
            return DomainCheck(SM.Log(x), "log");
        }

        [RubyMethodAttribute("log10", RubyMethodAttributes.PrivateInstance)]
        [RubyMethodAttribute("log10", RubyMethodAttributes.PublicSingleton)]
        public static double Log10(object self, double x) {
            return DomainCheck(SM.Log10(x), "log10");
        }

        [RubyMethodAttribute("sin", RubyMethodAttributes.PrivateInstance)]
        [RubyMethodAttribute("sin", RubyMethodAttributes.PublicSingleton)]
        public static double Sin(object self, double x) {
            return SM.Sin(x);
        }

        [RubyMethodAttribute("sinh", RubyMethodAttributes.PrivateInstance)]
        [RubyMethodAttribute("sinh", RubyMethodAttributes.PublicSingleton)]
        public static double Sinh(object self, double x) {
            return SM.Sinh(x);
        }

        [RubyMethodAttribute("sqrt", RubyMethodAttributes.PrivateInstance)]
        [RubyMethodAttribute("sqrt", RubyMethodAttributes.PublicSingleton)]
        public static double Sqrt(object self, double x) {
            return DomainCheck(SM.Sqrt(x), "sqrt");
        }

        [RubyMethodAttribute("tan", RubyMethodAttributes.PrivateInstance)]
        [RubyMethodAttribute("tan", RubyMethodAttributes.PublicSingleton)]
        public static double Tan(object self, double x) {
            return SM.Tan(x);
        }

        [RubyMethodAttribute("tanh", RubyMethodAttributes.PrivateInstance)]
        [RubyMethodAttribute("tanh", RubyMethodAttributes.PublicSingleton)]
        public static double Tanh(object self, double x) {
            return SM.Tanh(x);
        }

        #endregion
    }
}
