/* ****************************************************************************
 *
 * Copyright (c) Microsoft Corporation. 
 *
 * This source code is subject to terms and conditions of the Microsoft Permissive License. A 
 * copy of the license can be found in the License.html file at the root of this distribution. If 
 * you cannot locate the  Microsoft Permissive License, please send an email to 
 * ironruby@microsoft.com. By using this source code in any fashion, you are agreeing to be bound 
 * by the terms of the Microsoft Permissive License.
 *
 * You must not remove this notice, or any other, from this software.
 *
 *
 * ***************************************************************************/
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Scripting;
using Ruby.Runtime;

namespace Ruby.Builtins {

    [RubyClass, Includes(typeof(Enumerable))]
    public class Range : Object {
        private object _begin;
        private object _end;
        private bool _excludeEnd;

        public Range(object begin, object end, bool excludeEnd) {
            if (begin is bool || end is bool || begin == null || end == null)
                throw RubyExceptions.CreateArgumentError("bad value for range");

            _begin = begin;
            _end = end;
            _excludeEnd = excludeEnd;
        }

        [RubyMethodAttribute("begin", RubyMethodAttributes.PublicInstance)]
        public static object Begin(Range/*!*/ self) {
            return self._begin;
        }

        [RubyMethodAttribute("first", RubyMethodAttributes.PublicInstance)]
        public static object First(Range/*!*/ self) {
            return self._begin;
        }

        [RubyMethodAttribute("end", RubyMethodAttributes.PublicInstance)]
        public static object End(Range/*!*/ self) {
            return self._end;
        }

        [RubyMethodAttribute("last", RubyMethodAttributes.PublicInstance)]
        public static object Last(Range/*!*/ self) {
            return self._end;
        }

        [RubyMethodAttribute("exclude_end?", RubyMethodAttributes.PublicInstance)]
        public static bool ExcludeEnd(Range/*!*/ self) {
            return self._excludeEnd;
        }

        [RubyMethodAttribute("to_s", RubyMethodAttributes.PublicInstance)]
        [RubyMethodAttribute("inspect", RubyMethodAttributes.PublicInstance)]
        public static MutableString ToS(Range/*!*/ self) {
            return new MutableString(String.Format("{0}..{1}", RubySites.ToS(self._begin), RubySites.ToS(self._end)));
        }

        [RubyMethodAttribute("==", RubyMethodAttributes.PublicInstance)]
        public static bool Equals(Range/*!*/ self, object rhs) {
            return false;
        }
        
        [RubyMethodAttribute("==", RubyMethodAttributes.PublicInstance)]
        public static bool Equals(Range/*!*/ self, [NotNull]Range/*!*/ rhs) {
            return (self._begin == rhs._begin && self._end == rhs._end && self._excludeEnd == rhs._excludeEnd);
        }

        //declared private instance methods:
        //  initialize
        //declared protected instance methods:
        //declared public instance methods:
        //  ===
        //  each
        //  eql?
        //  hash
        //  include?
        //  member?

    }
}
