/* ****************************************************************************
 *
 * Copyright (c) Microsoft Corporation. 
 *
 * This source code is subject to terms and conditions of the Microsoft Permissive License. A 
 * copy of the license can be found in the License.html file at the root of this distribution. If 
 * you cannot locate the  Microsoft Permissive License, please send an email to 
 * ironruby@microsoft.com. By using this source code in any fashion, you are agreeing to be bound 
 * by the terms of the Microsoft Permissive License.
 *
 * You must not remove this notice, or any other, from this software.
 *
 *
 * ***************************************************************************/
using System;
using System.Collections.Generic;
using System.Text;
using Ruby.Runtime;
using Microsoft.Scripting;

namespace Ruby.Builtins {

    [RubyExtensionClass("Symbol", typeof(SymbolId))]
    public static class SymbolOps {

        #region Public Instance Methods

        [RubyMethod("id2name")]
        [RubyMethod("to_s")]
        public static MutableString/*!*/ ToString(SymbolId self) {
            return new MutableString(SymbolTable.IdToString(self));
        }

        [RubyMethod("inspect")]
        public static MutableString/*!*/ Inspect(SymbolId self) {
            return new MutableString(":" + SymbolTable.IdToString(self));
        }

        [RubyMethod("to_i")]
        [RubyMethod("to_int")]
        public static int ToInteger(SymbolId self) {
            return self.Id;
        }

        [RubyMethod("to_sym")]
        public static SymbolId ToSymbol(SymbolId self) {
            return self;
        }

        #endregion

        #region Public Singleton Methods

        [RubyMethod("all_symbols", RubyMethodAttributes.PublicSingleton)]
        public static List<object>/*!*/ GetAllSymbols(object self) {
            // TODO:
            throw new NotImplementedError();
        }

        #endregion
    }
}
