/* ****************************************************************************
 *
 * Copyright (c) Microsoft Corporation. 
 *
 * This source code is subject to terms and conditions of the Microsoft Permissive License. A 
 * copy of the license can be found in the License.html file at the root of this distribution. If 
 * you cannot locate the  Microsoft Permissive License, please send an email to 
 * ironruby@microsoft.com. By using this source code in any fashion, you are agreeing to be bound 
 * by the terms of the Microsoft Permissive License.
 *
 * You must not remove this notice, or any other, from this software.
 *
 *
 * ***************************************************************************/

using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;
using Microsoft.Scripting;
using MSA = Microsoft.Scripting.Ast;
using Ast = Microsoft.Scripting.Ast.Ast;
using Ruby.Runtime;

namespace Ruby.Compiler.AST {

    /// <summary>
    /// lhs = rhs
    /// lhs op= rhs
    /// </summary>
    public class ParallelAssignmentExpression : AssignmentExpression {
        private CompoundLeftValue/*!*/ _lhs;
        private RightValues/*!*/ _rhs;

        public override string NodeName { get { return "assignment"; } }

        public ParallelAssignmentExpression(CompoundLeftValue/*!*/ lhs, RightValues/*!*/ rhs, SymbolId operation, SourceSpan location)
            : base(operation, location) {
            Utils.Assert.NotNull(lhs, rhs);

            _lhs = lhs;
            _rhs = rhs;
        }

        private MSA.Expression CreateTempArrayOfRightValues(AstGenerator/*!*/ gen) {
            List<MSA.Expression> expressions = new List<MSA.Expression>();
            
            foreach (Expression expression in _rhs.Args) {
                expressions.Add(expression.TransformRead(gen));
            }
            
            return Ast.NewArray(typeof(object[]), expressions);
        }

        private MSA.Expression/*!*/ CreateExpression(AstGenerator/*!*/ gen) {
            MSA.Variable tempArrayVariable = gen.CurrentCodeBlock.CreateTemporaryVariable(SymbolTable.StringToId("#temp-array"), typeof(object[]));

            List<MSA.Expression> expressions = new List<MSA.Expression>();
            expressions.Add(Ast.Assign(tempArrayVariable, CreateTempArrayOfRightValues(gen)));

            for (int i = 0; i < _lhs.LeftValues.Count; ++i) {
                if (i >= _rhs.Args.Count) {
                    expressions.Add(_lhs.LeftValues[i].TransformWrite(gen, Ast.Null()));
                } else {
                    expressions.Add(_lhs.LeftValues[i].TransformWrite(gen, Ast.ReadItem(Ast.Read(tempArrayVariable), Ast.Constant(i))));
                }
            }

            expressions.Add(gen.OpCall("MakeArray", tempArrayVariable));
            return Ast.Comma(expressions.Count - 1, expressions);
        }

        public override MSA.Expression /*!*/ TransformRead(AstGenerator/*!*/ gen) {
            Utils.Assert.NotNull(gen);

            // TODO

            // Consider a, b = b, a
            // We need to generate a Comma(Compound)Expression:

            // temp = [b, a]
            // a = temp[0]
            // b = temp[1]
            // Convert.ToRubyArray(temp)

            return CreateExpression(gen);
        }
    }
}
