/* ****************************************************************************
 *
 * Copyright (c) Microsoft Corporation. 
 *
 * This source code is subject to terms and conditions of the Microsoft Permissive License. A 
 * copy of the license can be found in the License.html file at the root of this distribution. If 
 * you cannot locate the  Microsoft Permissive License, please send an email to 
 * ironruby@microsoft.com. By using this source code in any fashion, you are agreeing to be bound 
 * by the terms of the Microsoft Permissive License.
 *
 * You must not remove this notice, or any other, from this software.
 *
 *
 * ***************************************************************************/

using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;
using Microsoft.Scripting;
using MSA = Microsoft.Scripting.Ast;
using Ast = Microsoft.Scripting.Ast.Ast;

namespace Ruby.Compiler.AST {

    /// <summary>
    /// lhs = rhs
    /// lhs op= rhs
    /// </summary>
    public class SimpleAssignmentExpression : AssignmentExpression {
        private readonly LeftValue/*!*/ _lhs;
        private readonly Expression/*!*/ _rhs;
        private readonly SymbolId _operation;

        public LeftValue/*!*/ Left {
            get { return _lhs; }
        }

        public Expression/*!*/ Right {
            get { return _rhs; }
        }
        
        public override string NodeName { get { return "assignment"; } }

        public SimpleAssignmentExpression(LeftValue/*!*/ lhs, Expression/*!*/ rhs, SymbolId operation, SourceSpan location)
            : base(operation, location) {
            Utils.Assert.NotNull(lhs, rhs);

            _lhs = lhs;
            _rhs = rhs;
            _operation = operation;
        }

        public override MSA.Expression/*!*/ TransformRead(AstGenerator/*!*/ gen) {

            MSA.Expression transformedRhs = _rhs.TransformRead(gen);

            if (_operation != SymbolId.Empty) {
                transformedRhs = MethodCall.TransformRead(gen, _operation, _lhs, null, transformedRhs);
            } 

            return _lhs.TransformWrite(gen, transformedRhs);            
        }
    }
}
