/* ****************************************************************************
 *
 * Copyright (c) Microsoft Corporation. 
 *
 * This source code is subject to terms and conditions of the Microsoft Permissive License. A 
 * copy of the license can be found in the License.html file at the root of this distribution. If 
 * you cannot locate the  Microsoft Permissive License, please send an email to 
 * ironruby@microsoft.com. By using this source code in any fashion, you are agreeing to be bound 
 * by the terms of the Microsoft Permissive License.
 *
 * You must not remove this notice, or any other, from this software.
 *
 *
 * ***************************************************************************/

using System;
using System.Collections.Generic;
using System.Text;

using Microsoft.Scripting;
using MSA = Microsoft.Scripting.Ast;
using Ast = Microsoft.Scripting.Ast.Ast;
using Ruby.Runtime;

namespace Ruby.Compiler.AST {
    public sealed class ClassDeclaration : ModuleDeclaration {

        // class Name
        //   <statements>
        // end

        // RubyOps.DefineClass(<CodeContext>, <Name>, delegate => <statements>)
        
        private Expression _superClass;

        public ClassDeclaration(LexicalScope/*!*/ definedScope, ConstantVariable/*!*/ name, Expression superClass, Body/*!*/ body, SourceSpan location)
            : base(definedScope, name, body, location) {
            Utils.Assert.NotNull(name);
            
            _superClass = superClass;
        }

        protected override MSA.Expression/*!*/ MakeDefinitionExpression(AstGenerator/*!*/ gen) {
            return gen.OpCall("DefineClass", new MSA.Expression[] {
                Ast.CodeContext(),
                QualifiedName.TransformQualifier(gen),
                QualifiedName.TransformSymbolName(gen),
                (_superClass != null) ? _superClass.TransformRead(gen) : Ast.Null()
            });
        }
    }
}
