/* ****************************************************************************
 *
 * Copyright (c) Microsoft Corporation. 
 *
 * This source code is subject to terms and conditions of the Microsoft Permissive License. A 
 * copy of the license can be found in the License.html file at the root of this distribution. If 
 * you cannot locate the  Microsoft Permissive License, please send an email to 
 * ironruby@microsoft.com. By using this source code in any fashion, you are agreeing to be bound 
 * by the terms of the Microsoft Permissive License.
 *
 * You must not remove this notice, or any other, from this software.
 *
 *
 * ***************************************************************************/

using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Scripting;
using MSA = Microsoft.Scripting.Ast;
using Ast = Microsoft.Scripting.Ast.Ast;
using Ruby.Runtime;

namespace Ruby.Compiler.AST {
    public class ModuleDeclaration : DeclarationExpression {
        
        // TODO:
        private readonly ConstantVariable _qualifiedName;
        
        public ConstantVariable QualifiedName {
            get { return _qualifiedName; }
        }

        public ModuleDeclaration(LexicalScope/*!*/ definedScope, ConstantVariable qualifiedName, Body/*!*/ body, SourceSpan location)
            : base(definedScope, body, location) {
            Utils.Assert.NotNull(body);

            _qualifiedName = qualifiedName;
        }

        protected virtual MSA.Expression/*!*/ MakeDefinitionExpression(AstGenerator/*!*/ gen) {
            return gen.OpCall("DefineModule", new MSA.Expression[] {
                Ast.CodeContext(),
                QualifiedName.TransformQualifier(gen),
                QualifiedName.TransformSymbolName(gen)
            });
        }

        public sealed override MSA.Expression/*!*/ TransformRead(AstGenerator/*!*/ gen) {
            
            // definition needs to take place outside the defined lexical scope:
            MSA.Expression definition = MakeDefinitionExpression(gen);
            
            MSA.Variable oldSelf = gen.CurrentSelfVariable;
            MSA.Variable localSelf = gen.CurrentCodeBlock.CreateVariable(SymbolTable.StringToId("#module-self"), MSA.Variable.VariableKind.Temporary, typeof(object));

            gen.CurrentSelfVariable = localSelf;
            gen.CurrentScope = DefinedScope;

            // TODO: we need scoped variables in DLR here:
            // DefinedScope.TransformLocals(methodBlock, gen);
            
            // begin with new scope
            //   self = DefineModule/Class(... parent scope here ...)
            //   <body>
            // end
            MSA.Expression result = Ast.Comma(
                1,
                Ast.Assign(localSelf, definition),
                Body.TransformToExpression(gen)
            );

            gen.CurrentSelfVariable = oldSelf;
            gen.CurrentScope = DefinedScope.ParentScope;

            return result;
        }
    }
}
