/* ****************************************************************************
 *
 * Copyright (c) Microsoft Corporation. 
 *
 * This source code is subject to terms and conditions of the Microsoft Permissive License. A 
 * copy of the license can be found in the License.html file at the root of this distribution. If 
 * you cannot locate the  Microsoft Permissive License, please send an email to 
 * ironruby@microsoft.com. By using this source code in any fashion, you are agreeing to be bound 
 * by the terms of the Microsoft Permissive License.
 *
 * You must not remove this notice, or any other, from this software.
 *
 *
 * ***************************************************************************/

using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Scripting;
using MSA = Microsoft.Scripting.Ast;

namespace Ruby.Compiler.AST {

    /// <summary>
    /// NOTE: Block expression is a like comma expression. It doesn't define a scope.
    /// </summary>
    public class BlockExpression : Expression
    {
        //	begin 
        //		body
        //	end

        private readonly Body/*!*/ _body;

        public BlockExpression(Body/*!*/ body, SourceSpan location)
            : base(location) {
            Utils.Assert.NotNull(body);

            _body = body;
        }

        public override MSA.Expression/*!*/ TransformRead(AstGenerator/*!*/ gen) {
            return _body.TransformToExpression(gen);
        }
    }
}
