/* ****************************************************************************
 *
 * Copyright (c) Microsoft Corporation. 
 *
 * This source code is subject to terms and conditions of the Microsoft Permissive License. A 
 * copy of the license can be found in the License.html file at the root of this distribution. If 
 * you cannot locate the  Microsoft Permissive License, please send an email to 
 * ironruby@microsoft.com. By using this source code in any fashion, you are agreeing to be bound 
 * by the terms of the Microsoft Permissive License.
 *
 * You must not remove this notice, or any other, from this software.
 *
 *
 * ***************************************************************************/

using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Scripting;
using MSA = Microsoft.Scripting.Ast;
using Ast = Microsoft.Scripting.Ast.Ast;

namespace Ruby.Compiler.AST {

    /// <summary>
    /// Lightweight form of if-then-else-end expression. 
    /// Used for ternary ?: operator.
    /// </summary>
    public sealed class ConditionalExpression : Expression {
        private Expression/*!*/ _condition;
        private Expression/*!*/ _trueExpression;
        private Expression/*!*/ _falseExpression;

        public ConditionalExpression(Expression/*!*/ condition, Expression/*!*/ trueExpression, Expression/*!*/ falseExpression, SourceSpan location) 
            : base(location) {
            Utils.Assert.NotNull(condition, trueExpression, falseExpression);
            _condition = condition;
            _trueExpression = trueExpression;
            _falseExpression = falseExpression;
        }

        public override MSA.Expression/*!*/ TransformRead(AstGenerator/*!*/ gen) {
            return Ast.Condition(
                _condition.TransformRead(gen),
                _trueExpression.TransformRead(gen),
                _falseExpression.TransformRead(gen)
            );
        }
    }
}
