/* ****************************************************************************
 *
 * Copyright (c) Microsoft Corporation. 
 *
 * This source code is subject to terms and conditions of the Microsoft Permissive License. A 
 * copy of the license can be found in the License.html file at the root of this distribution. If 
 * you cannot locate the  Microsoft Permissive License, please send an email to 
 * ironruby@microsoft.com. By using this source code in any fashion, you are agreeing to be bound 
 * by the terms of the Microsoft Permissive License.
 *
 * You must not remove this notice, or any other, from this software.
 *
 *
 * ***************************************************************************/

using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Scripting;
using MSA = Microsoft.Scripting.Ast;
using Ast = Microsoft.Scripting.Ast.Ast;
using Ruby.Runtime;
using Microsoft.Scripting.Actions;

namespace Ruby.Compiler.AST {
    public class ForLoopExpression : Expression {
        //	for variables in list
        //		body
        //	end

        private readonly Block/*!*/ _block;
        private readonly Expression/*!*/ _list;

        public ForLoopExpression(CompoundLeftValue/*!*/ variables, Expression/*!*/ list, List<Statement> body, SourceSpan location)
            : base(location) {
            Utils.Assert.NotNull(variables, list);
            
            _block = new BlockDefinition(null, variables, body, location);
            _list = list;
        }

        public override MSA.Expression/*!*/ TransformRead(AstGenerator/*!*/ gen) {
            Utils.Assert.NotNull(gen);

            return Ast.Action.InvokeMember(Symbols.Each, typeof(object),
                InvokeMemberActionFlags.IsCallWithThis | InvokeMemberActionFlags.ReturnNonCallable,
                new ArgumentInfo[] { 
                    new ArgumentInfo(MSA.ArgumentKind.Instance), 
                    new ArgumentInfo(MSA.ArgumentKind.Block) 
                }, 
                _list.TransformRead(gen),
                _block.Transform(gen)
            );
        }
    }
}
