/* ****************************************************************************
 *
 * Copyright (c) Microsoft Corporation. 
 *
 * This source code is subject to terms and conditions of the Microsoft Permissive License. A 
 * copy of the license can be found in the License.html file at the root of this distribution. If 
 * you cannot locate the  Microsoft Permissive License, please send an email to 
 * ironruby@microsoft.com. By using this source code in any fashion, you are agreeing to be bound 
 * by the terms of the Microsoft Permissive License.
 *
 * You must not remove this notice, or any other, from this software.
 *
 *
 * ***************************************************************************/

using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;

using Microsoft.Scripting;
using MSA = Microsoft.Scripting.Ast;
using Ast = Microsoft.Scripting.Ast.Ast;

using Ruby.Runtime;

namespace Ruby.Compiler.AST {
    public class HashConstructor : Expression {
        // { key1 => value1, key2 => value2, ... }
        // or
        // { key1, value1, key2, value2, ... }

        public readonly List<Maplet> _maplets;
        public readonly List<Expression> _expressions;

        public HashConstructor(List<Maplet> maplets, List<Expression> expressions, SourceSpan location)
            : base(location) {
            Debug.Assert(maplets == null || expressions == null);
            Debug.Assert(expressions == null || expressions.Count % 2 == 0);

            _maplets = maplets;
            _expressions = expressions;
        }

        private List<MSA.Expression>/*!*/ MakeExpressionList(AstGenerator/*!*/ gen) {
            Utils.Assert.NotNull(gen);

            if (_maplets != null) {                
                return gen.TransformMapletsToExpressions(_maplets);
            } else if (_expressions != null) {
                return gen.TranformExpressions(_expressions);
            } else {
                return new List<MSA.Expression>(0);
            }
        }

        public override MSA.Expression/*!*/ TransformRead(AstGenerator/*!*/ gen) {
            return gen.OpCall("MakeHash", new MSA.Expression[] {
                Ast.NewArray(typeof(object[]), MakeExpressionList(gen))
            });
        }
    }
}
