/* ****************************************************************************
 *
 * Copyright (c) Microsoft Corporation. 
 *
 * This source code is subject to terms and conditions of the Microsoft Permissive License. A 
 * copy of the license can be found in the License.html file at the root of this distribution. If 
 * you cannot locate the  Microsoft Permissive License, please send an email to 
 * ironruby@microsoft.com. By using this source code in any fashion, you are agreeing to be bound 
 * by the terms of the Microsoft Permissive License.
 *
 * You must not remove this notice, or any other, from this software.
 *
 *
 * ***************************************************************************/

using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Scripting;
using System.Diagnostics;
using MSA = Microsoft.Scripting.Ast;
using Ast = Microsoft.Scripting.Ast.Ast;

namespace Ruby.Compiler.AST {

    public sealed class IfExpression : Expression {
        private Expression/*!*/ _condition;
        private bool _isUnless;
        private List<Statement> _body;
        private List<ElseIfClause> _elseIfClauses;

        public IfExpression(Expression/*!*/ condition, bool isUnless, List<Statement> body, List<ElseIfClause> elseIfClauses, SourceSpan location)
            : base(location) {
            Utils.Assert.NotNull(condition);
            Debug.Assert(body == null || body.Count > 0);
            Debug.Assert(elseIfClauses == null || elseIfClauses.Count > 0);

            _condition = condition;
            _isUnless = isUnless;
            _body = body;
            _elseIfClauses = elseIfClauses;
        }

        public override MSA.Expression/*!*/ TransformRead(AstGenerator/*!*/ gen) {

            MSA.Expression result;

            if (_elseIfClauses != null) {
                int i = _elseIfClauses.Count - 1;

                if (_elseIfClauses[i].Condition == null) {
                    // ... else body end
                    result = gen.TransformStatementsToExpression(_elseIfClauses[i].Statements);
                    i--;
                } else {
                    // ... elseif (condition) body end
                    // treat like if there was an empty else:
                    result = gen.TransformStatementsToExpression(null);
                }

                while (i > 0) {
                    // TODO: conversion

                    // ... else (if (condition) body else result)
                    result = Ast.Condition(
                        Ast.Convert(_elseIfClauses[i].Condition.TransformRead(gen), typeof(object)),
                        Ast.Convert(gen.TransformStatementsToExpression(_elseIfClauses[i].Statements), typeof(object)),
                        result
                    );
                    i--;
                }
            } else {
                // empty else:
                result = gen.TransformStatementsToExpression(null);
            }

            // TODO: conversion

            // if (condition) body else result
            return Ast.Condition(
                _condition.TransformRead(gen),
                Ast.Convert(gen.TransformStatementsToExpression(_body), typeof(object)),
                Ast.Convert(result, typeof(object))
            );
        }
    }
}
