/* ****************************************************************************
 *
 * Copyright (c) Microsoft Corporation. 
 *
 * This source code is subject to terms and conditions of the Microsoft Permissive License. A 
 * copy of the license can be found in the License.html file at the root of this distribution. If 
 * you cannot locate the  Microsoft Permissive License, please send an email to 
 * ironruby@microsoft.com. By using this source code in any fashion, you are agreeing to be bound 
 * by the terms of the Microsoft Permissive License.
 *
 * You must not remove this notice, or any other, from this software.
 *
 *
 * ***************************************************************************/

using System;
using System.Collections.Generic;
using System.Text;

using Microsoft.Scripting;
using MSA = Microsoft.Scripting.Ast;
using Ast = Microsoft.Scripting.Ast.Ast;
using System.Diagnostics;

namespace Ruby.Compiler.AST {

    public class Literal : Expression {
        private object _value;
        
        public object Value {
            get { return _value; }
            set { _value = value; }
        }

        public override string/*!*/ NodeName {
            get {
                if (_value == null) return "nil";
                if (ReferenceEquals(_value, RuntimeHelpers.True)) return "true";
                if (ReferenceEquals(_value, RuntimeHelpers.False)) return "false";
                return base.NodeName;
            }
        }

        public Literal(object value, SourceSpan location)
            : base(location) {
            _value = value;
        }

        public static Literal/*!*/ Nil(SourceSpan location) {
            return new Literal(null, location);
        }

        public static Literal/*!*/ True(SourceSpan location) {
            return new Literal(RuntimeHelpers.True, location);
        }

        public static Literal/*!*/ False(SourceSpan location) {
            return new Literal(RuntimeHelpers.False, location);
        }

        public override MSA.Expression TransformRead(AstGenerator gen) {
            // TODO: mutable string construction hook here as well -- are there other places this must be done?
            string str = _value as string;
            if (str != null) {
                return gen.OpCall("CreateMutableString", new MSA.Expression[] { Ast.Constant(str) });
            }
            return Ast.Constant(_value);
        }
    }
}
