/* ****************************************************************************
 *
 * Copyright (c) Microsoft Corporation. 
 *
 * This source code is subject to terms and conditions of the Microsoft Permissive License. A 
 * copy of the license can be found in the License.html file at the root of this distribution. If 
 * you cannot locate the  Microsoft Permissive License, please send an email to 
 * ironruby@microsoft.com. By using this source code in any fashion, you are agreeing to be bound 
 * by the terms of the Microsoft Permissive License.
 *
 * You must not remove this notice, or any other, from this software.
 *
 *
 * ***************************************************************************/

using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;

using System.Reflection;

using Microsoft.Scripting;
using MSA = Microsoft.Scripting.Ast;
using Ruby.Runtime;

namespace Ruby.Compiler.AST {
    using Ast = Microsoft.Scripting.Ast.Ast;
    using Microsoft.Scripting.Actions;

    /// <summary>
    /// target.method_id(args)
    /// </summary>
    public class MethodCall : CallExpression {

        protected SymbolId _methodName;
        private Expression _target;

        public override string NodeName { get { return "method"; } }

        public MethodCall(Expression target, SymbolId methodName, Arguments/*!*/ args, SourceSpan location)
            : base(args, location) {

            Utils.Assert.NotNull(methodName, args);

            _methodName = methodName;
            _target = target;
        }

        // TODO: unused?
        //public static Expression/*!*/ Create(Expression/*!*/ target, SymbolId methodName, Arguments/*!*/ args, SourceSpan location) {

        //    if (methodName == Symbols.Or)
        //        return new OrExpression(target, args, location);

        //    if (methodName == Symbols.And)
        //        return new AndExpression(target, args, location);

        //    return new MethodCall(target, methodName, args, location);
        //}

        public override MSA.Expression/*!*/ TransformRead(AstGenerator/*!*/ gen) {
            return TransformRead(gen, _methodName, _target, Arguments, null);
        }

        public static MSA.Expression/*!*/ TransformRead(AstGenerator/*!*/ gen, SymbolId methodName, Expression target,
            Arguments arguments, MSA.Expression additionalArg) {

            MSA.Expression result = Arguments.TransformToMethodInvocation(gen, methodName, target, arguments, additionalArg);

            if (arguments != null && arguments.Block != null) {
                result = MakeCallWithBlockRetryable(gen, result);
            }

            return result;
        }

        private static MSA.Expression/*!*/ MakeCallWithBlockRetryable(AstGenerator/*!*/ gen, MSA.Expression/*!*/ invoke) {
            Utils.Assert.NotNull(invoke);

            // see Ruby Language.doc/Control Flow Implementation/MethodCallWithBlock
            MSA.Variable resultVariable = gen.CurrentCodeBlock.CreateTemporaryVariable(SymbolTable.StringToId("#method-result"), typeof(object));

            return AstFactory.Result(resultVariable,
                Ast.Do(
                    Ast.Try(
                        AstFactory.Assign(resultVariable, invoke)
                    ).Catch(typeof(StackUnwinder.Retry),
                        AstFactory.Assign(resultVariable, Ast.ReadField(null, gen.RfcRetrySingleton))
                    )
                ).While(AstFactory.Equal(resultVariable, null, null, gen.RfcRetrySingleton))
            );
        }
    }
}
