/* ****************************************************************************
 *
 * Copyright (c) Microsoft Corporation. 
 *
 * This source code is subject to terms and conditions of the Microsoft Permissive License. A 
 * copy of the license can be found in the License.html file at the root of this distribution. If 
 * you cannot locate the  Microsoft Permissive License, please send an email to 
 * ironruby@microsoft.com. By using this source code in any fashion, you are agreeing to be bound 
 * by the terms of the Microsoft Permissive License.
 *
 * You must not remove this notice, or any other, from this software.
 *
 *
 * ***************************************************************************/

using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Scripting;
using MSA = Microsoft.Scripting.Ast;

namespace Ruby.Compiler.AST {
    public class RegularExpression : Expression {
        // /pattern/options
        private int _options;
        private List<Expression> _pattern;

        public RegularExpression(List<Expression> pattern, int options, SourceSpan location)
            : base(location) {
            this._pattern = pattern;
            this._options = options;
        }

        public override Microsoft.Scripting.Ast.Expression TransformRead(AstGenerator gen) {
            if (_pattern.Count == 1)
                return gen.OpCall("CreateRegex", new MSA.Expression[] { _pattern[0].TransformRead(gen) });
            else
                return gen.OpCall("CreateRegex2", new MSA.Expression[] {
                    MSA.Ast.NewArray(typeof(object[]), gen.TranformExpressions(_pattern))
                });
        }
    }
}
