/* ****************************************************************************
 *
 * Copyright (c) Microsoft Corporation. 
 *
 * This source code is subject to terms and conditions of the Microsoft Permissive License. A 
 * copy of the license can be found in the License.html file at the root of this distribution. If 
 * you cannot locate the  Microsoft Permissive License, please send an email to 
 * ironruby@microsoft.com. By using this source code in any fashion, you are agreeing to be bound 
 * by the terms of the Microsoft Permissive License.
 *
 * You must not remove this notice, or any other, from this software.
 *
 *
 * ***************************************************************************/

using System.Collections.Generic;

using Ruby.Runtime;
using Microsoft.Scripting;
using MSA = Microsoft.Scripting.Ast;
using Ast = Microsoft.Scripting.Ast.Ast;

namespace Ruby.Compiler.AST {

    /// <summary>
    /// Sequence of string literals and/or string embedded expressions.
    /// </summary>
    public class StringConstructor : Expression {
        private readonly List<Expression>/*!*/ _parts;

        public StringConstructor(List<Expression>/*!*/ parts, SourceSpan location) 
            : base(location) {
            //Debug.Assert(parts != null && parts.Count > 0);

            _parts = parts;
        }

        public override MSA.Expression/*!*/ TransformRead(AstGenerator/*!*/ gen) {
            // TODO: optimize -- this is being called too many times in the tree - it seems that the tree has too many nodes

            if (_parts.Count == 1) {
                return gen.OpCall("CreateMutableString", new MSA.Expression[] { _parts[0].TransformRead(gen) });
            } else {
                return gen.OpCall("ConcatMutableString", new MSA.Expression[] {
                    Ast.NewArray(typeof(object[]), gen.TranformExpressions(_parts))
                });
            }
        }
    }
}
