/* ****************************************************************************
 *
 * Copyright (c) Microsoft Corporation. 
 *
 * This source code is subject to terms and conditions of the Microsoft Permissive License. A 
 * copy of the license can be found in the License.html file at the root of this distribution. If 
 * you cannot locate the  Microsoft Permissive License, please send an email to 
 * ironruby@microsoft.com. By using this source code in any fashion, you are agreeing to be bound 
 * by the terms of the Microsoft Permissive License.
 *
 * You must not remove this notice, or any other, from this software.
 *
 *
 * ***************************************************************************/
using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;
using Microsoft.Scripting;
using MSA = Microsoft.Scripting.Ast;
using Ruby.Runtime;
using Ast = Microsoft.Scripting.Ast.Ast;
using Microsoft.Scripting.Actions;

namespace Ruby.Compiler.AST {
    /// <summary>
    /// yield(args)
    /// </summary>
    public class YieldCall : CallExpression {

        public override string NodeName { get { return "yield"; } }
        
        public YieldCall(Arguments/*!*/ args, SourceSpan location)
            : base(args, location) {

            Debug.Assert(args != null);
        }

        // see Ruby Language.doc/Runtime/Control Flow Implementation/Yield
        public override MSA.Expression/*!*/ TransformRead(AstGenerator/*!*/ gen) {
            MSA.Variable returnReasonVariable = gen.CurrentCodeBlock.CreateTemporaryVariable(SymbolTable.StringToId("#return-reason"), typeof(BlockReturnReasonHolder));
            MSA.Variable resultVariable = gen.CurrentCodeBlock.CreateTemporaryVariable(SymbolTable.StringToId("#result"), typeof(object));
            MSA.Variable unwinderVariable = gen.CurrentCodeBlock.CreateTemporaryVariable(SymbolTable.StringToId("#unwinder"), typeof(StackUnwinder.BlockReturn));

            MSA.Statement returnStatement;

            if (gen.CurrentRubyBlock != null) {
                // in a block:
                returnStatement = Ast.Statement(gen.RfcCall("DynamicReturn", resultVariable));
            } else {
                // otherwise:
                returnStatement = Ast.Return(Ast.Read(resultVariable));
            }

            return Ast.Comma(2,
                Ast.Assign(returnReasonVariable, Ast.New(BlockReturnReasonHolder.Constructor)),
                Ast.Void(Ast.Loop(
                    Ast.Try(
                        Ast.Write(resultVariable, Arguments.TransformToBlockCall(gen, Arguments, gen.CurrentBlockParameter, returnReasonVariable))
                    ).Catch(typeof(StackUnwinder.BlockReturn), unwinderVariable,  
                        Ast.Write(resultVariable, unwinderVariable, StackUnwinder.BlockReturn.ReturnValueField),
                        Ast.Write(returnReasonVariable, BlockReturnReasonHolder.ValueField, unwinderVariable, StackUnwinder.BlockReturn.ReasonField)
                    ),

                    Ast.IfThen(Ast.Equal(Ast.ReadField(Ast.Read(returnReasonVariable), BlockReturnReasonHolder.ValueField), Ast.Constant(BlockReturnReason.BreakOrRetry)),
                        returnStatement
                    ),

                    Ast.IfThen(Ast.NotEqual(Ast.ReadField(Ast.Read(returnReasonVariable), BlockReturnReasonHolder.ValueField), Ast.Constant(BlockReturnReason.Redo)),
                        Ast.Break()
                    ),
                    
                    Ast.Write(returnReasonVariable, BlockReturnReasonHolder.ValueField, Ast.Constant(BlockReturnReason.Undefined))
                )),
                Ast.Read(resultVariable)
            );
        }
    }
}
