/* ****************************************************************************
 *
 * Copyright (c) Microsoft Corporation. 
 *
 * This source code is subject to terms and conditions of the Microsoft Permissive License. A 
 * copy of the license can be found in the License.html file at the root of this distribution. If 
 * you cannot locate the  Microsoft Permissive License, please send an email to 
 * ironruby@microsoft.com. By using this source code in any fashion, you are agreeing to be bound 
 * by the terms of the Microsoft Permissive License.
 *
 * You must not remove this notice, or any other, from this software.
 *
 *
 * ***************************************************************************/

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Text;
using Microsoft.Scripting;
using MSA = Microsoft.Scripting.Ast;
using Ast = Microsoft.Scripting.Ast.Ast;
using Ruby.Runtime;

namespace Ruby.Compiler.AST {
    public class BreakStatement : JumpStatement {
        public BreakStatement(LexicalScope/*!*/ scope, Arguments arguments, SourceSpan location)
            : base(scope, arguments, location) {
        }

        // see Ruby Language.doc/Runtime/Control Flow Implementation/Break
        public override MSA.Statement/*!*/ Transform(AstGenerator/*!*/ gen) {

            MSA.Expression transformedReturnValue = TransformReturnValue(gen);

            // in a loop:

            if (gen.CurrentLoop != null) {
                return Ast.Block(
                    Ast.Statement(Ast.Assign(gen.CurrentLoop.ResultVariable, transformedReturnValue)),
                    Ast.Break()
                );
            }

            // in a block:

            if (gen.CurrentRubyBlock != null) {
                return Ast.Block(
                    AstFactory.AssignValueWithCast(gen.CurrentRubyBlock.ReturnReasonVariable, typeof(BlockReturnReasonHolder), BlockReturnReasonHolder.ValueField, BlockReturnReason.BreakOrRetry),
                    Ast.Return(transformedReturnValue)
                );
            }

            // TODO: this actually doesn't work in Ruby (we should test it more however)
            //// in a method (must get a block):

            //if (gen.CurrentMethod != null) {
            //    Utils.Assert.NotNull(gen.CurrentBlockParameter);

            //    return new MSA.BlockStatement(new MSA.Statement[] {
            //        new MSA.ExpressionStatement(new MSA.MethodCallExpression(RuntimeFlowControl.GetMethod("RequireBlockForBreak"), 
            //            new MSA.BoundExpression(gen.CurrentRuntimeFlowControlVariable), Ast.ToArgs(gen.CurrentBlockParameter))),
            //        new MSA.ReturnStatement(transformedReturnValue)
            //    });
            //}

            // otherwise:

            return Ast.Statement(gen.RfcCall("DynamicBreak", new MSA.Expression[] { transformedReturnValue }));
        }
    }
}
