/* ****************************************************************************
 *
 * Copyright (c) Microsoft Corporation. 
 *
 * This source code is subject to terms and conditions of the Microsoft Permissive License. A 
 * copy of the license can be found in the License.html file at the root of this distribution. If 
 * you cannot locate the  Microsoft Permissive License, please send an email to 
 * ironruby@microsoft.com. By using this source code in any fashion, you are agreeing to be bound 
 * by the terms of the Microsoft Permissive License.
 *
 * You must not remove this notice, or any other, from this software.
 *
 *
 * ***************************************************************************/

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Text;
using Microsoft.Scripting;
using MSA = Microsoft.Scripting.Ast;
using Ast = Microsoft.Scripting.Ast.Ast;
using Ruby.Runtime;

namespace Ruby.Compiler.AST {
    public class NextStatement : JumpStatement {
        public NextStatement(LexicalScope/*!*/ scope, Arguments arguments, SourceSpan location)
            : base(scope, arguments, location) {
        }

        // see Ruby Language.doc/Runtime/Control Flow Implementation/Next
        public override MSA.Statement/*!*/ Transform(AstGenerator/*!*/ gen) {

            // in a loop:

            MSA.Expression transformedReturnValue = TransformReturnValue(gen);

            if (gen.CurrentLoop != null) {
                return Ast.Block(
                    Ast.Statement(transformedReturnValue), // evaluate for side-effects
                    Ast.Continue()
                );
            }

            // in a block:

            if (gen.CurrentRubyBlock != null) {
                return Ast.Block(
                    AstFactory.AssignValueWithCast(gen.CurrentRubyBlock.ReturnReasonVariable, typeof(BlockReturnReasonHolder), BlockReturnReasonHolder.ValueField, BlockReturnReason.Next),
                    Ast.Return(transformedReturnValue)
                );
            }

            // otherwise:

            return Ast.Statement(gen.RfcCall("DynamicNext", new MSA.Expression[] { transformedReturnValue }));
        }
    }
}