/* ****************************************************************************
 *
 * Copyright (c) Microsoft Corporation. 
 *
 * This source code is subject to terms and conditions of the Microsoft Permissive License. A 
 * copy of the license can be found in the License.html file at the root of this distribution. If 
 * you cannot locate the  Microsoft Permissive License, please send an email to 
 * ironruby@microsoft.com. By using this source code in any fashion, you are agreeing to be bound 
 * by the terms of the Microsoft Permissive License.
 *
 * You must not remove this notice, or any other, from this software.
 *
 *
 * ***************************************************************************/

using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Scripting;
using MSA = Microsoft.Scripting.Ast;
using Ast = Microsoft.Scripting.Ast.Ast;
using Ruby.Runtime;
using System.Diagnostics;

namespace Ruby.Compiler.AST {
    public class RetryStatement : JumpStatement {
        public RetryStatement(LexicalScope/*!*/ scope, SourceSpan location)
            : base(scope, null, location) {
        }

        // see Ruby Language.doc/Runtime/Control Flow Implementation/Retry
        public override MSA.Statement/*!*/ Transform(AstGenerator/*!*/ gen) {

            // in eval:
            // TODO:

            // in try-rescue:
            if (gen.CurrentTryRescueBlock != null) {
                return Ast.Block(
                    AstFactory.AssignValue(gen.CurrentTryRescueBlock.RetryingVariable, true),
                    Ast.Continue()
                );
            }
            
            // in a block:
            if (gen.CurrentRubyBlock != null) {
                return Ast.Block(
                    AstFactory.AssignValueWithCast(gen.CurrentRubyBlock.ReturnReasonVariable, typeof(BlockReturnReasonHolder), BlockReturnReasonHolder.ValueField, BlockReturnReason.BreakOrRetry),
                    Ast.Return(gen.RfcStaticCall("RetryBlock", AstFactory.EmptyExpressions))
                );
            }
            
            Utils.Assert.NotNull(gen.CurrentBlockParameter);

            // otherwise:
            return Ast.Return(gen.RfcStaticCall("RetryMethod", AstFactory.ToArgs(gen.CurrentBlockParameter)));
        }
    }
}