/* ****************************************************************************
 *
 * Copyright (c) Microsoft Corporation. 
 *
 * This source code is subject to terms and conditions of the Microsoft Permissive License. A 
 * copy of the license can be found in the License.html file at the root of this distribution. If 
 * you cannot locate the  Microsoft Permissive License, please send an email to 
 * ironruby@microsoft.com. By using this source code in any fashion, you are agreeing to be bound 
 * by the terms of the Microsoft Permissive License.
 *
 * You must not remove this notice, or any other, from this software.
 *
 *
 * ***************************************************************************/

using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Scripting;
using MSA = Microsoft.Scripting.Ast;

namespace Ruby.Compiler.AST {
    public class ArrayItemAccess : LeftValue {
        // array [ args ]

        protected Expression/*!*/ array;
        protected Arguments/*!*/ args;

        public ArrayItemAccess(Expression/*!*/ array, Arguments/*!*/ args, SourceSpan location)
            : base(location) {
            Utils.Assert.NotNull(array, args);

            this.array = array;
            this.args = args;
        }

        public override MSA.Expression/*!*/ TransformRead(AstGenerator/*!*/ gen) {
            return MethodCall.TransformRead(gen, Symbols.ArrayItemRead, array, args, null);
        }

        public override MSA.Expression/*!*/ TransformWrite(AstGenerator/*!*/ gen, MSA.Expression/*!*/ rightValue) {
            return MethodCall.TransformRead(gen, Symbols.ArrayItemWrite, array, args, rightValue);
        }
    }

}