/* ****************************************************************************
 *
 * Copyright (c) Microsoft Corporation. 
 *
 * This source code is subject to terms and conditions of the Microsoft Permissive License. A 
 * copy of the license can be found in the License.html file at the root of this distribution. If 
 * you cannot locate the  Microsoft Permissive License, please send an email to 
 * ironruby@microsoft.com. By using this source code in any fashion, you are agreeing to be bound 
 * by the terms of the Microsoft Permissive License.
 *
 * You must not remove this notice, or any other, from this software.
 *
 *
 * ***************************************************************************/

using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Scripting;
using MSA = Microsoft.Scripting.Ast;

namespace Ruby.Compiler.AST {
    public class CompoundLeftValue : LeftValue {
        internal static readonly CompoundLeftValue/*!*/ Null = new CompoundLeftValue(LeftValue.EmptyList, null, SourceSpan.None);

        /// <summary>
        /// List of l-values, possibly compound.
        /// </summary>
        private readonly List<LeftValue>/*!*/ _leftValues;

        /// <summary>
        /// Residual l-value (l-value following the * sentinel).
        /// </summary>
        private readonly LeftValue _unsplattedValue;
        
        public List<LeftValue>/*!*/ LeftValues {
            get { return _leftValues; }
        }

        public LeftValue UnsplattedValue {
            get { return _unsplattedValue; }
        }

        public CompoundLeftValue(List<LeftValue>/*!*/ leftValues, LeftValue unsplattedValue, SourceSpan location)
            : base(location) {
            Utils.Assert.NotNull(leftValues);

            _leftValues = leftValues;
            _unsplattedValue = unsplattedValue;
        }

        public override MSA.Expression/*!*/ TransformWrite(AstGenerator/*!*/ gen, MSA.Expression/*!*/ rightValue) {
            throw new NotImplementedException("TODO");
        }
    }
}
