/* ****************************************************************************
 *
 * Copyright (c) Microsoft Corporation. 
 *
 * This source code is subject to terms and conditions of the Microsoft Permissive License. A 
 * copy of the license can be found in the License.html file at the root of this distribution. If 
 * you cannot locate the  Microsoft Permissive License, please send an email to 
 * ironruby@microsoft.com. By using this source code in any fashion, you are agreeing to be bound 
 * by the terms of the Microsoft Permissive License.
 *
 * You must not remove this notice, or any other, from this software.
 *
 *
 * ***************************************************************************/

using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Scripting;
using MSA = Microsoft.Scripting.Ast;
using Ast = Microsoft.Scripting.Ast.Ast;

using Ruby.Runtime;

namespace Ruby.Compiler.AST {

    public class ConstantVariable : Variable {
        private Expression _qualifier;
        private LexicalScope _scope;
        
        /// <summary>
        /// A constant may be qualfied by emoty qualifier (::FOO)
        /// </summary>
        private bool _qualified;

        public override string NodeName { get { return "constant"; } }

        public ConstantVariable(LexicalScope/*!*/ scope, SymbolId name, SourceSpan location)
            : base(name, location) {
            Utils.Assert.NotNull(scope, name);

            _scope = scope;
            _qualifier = null;
            _qualified = false;
        }

        public ConstantVariable(LexicalScope/*!*/ scope, Expression qualifier, SymbolId name, SourceSpan location)
            : base(name, location) {
            Utils.Assert.NotNull(scope, name);

            _scope = scope;
            _qualifier = qualifier;
            _qualified = true;
        }

        internal MSA.Expression/*!*/ TransformQualifier(AstGenerator/*!*/ gen) {
            if (_qualified) {
                if (_qualifier != null) {
                    return _qualifier.TransformRead(gen);
                } else {
                    return Ast.Null();
                }
            } else {
                return gen.MakeSelfRead();
            } 
        }

        public override MSA.Expression/*!*/ TransformRead(AstGenerator/*!*/ gen) {
            return gen.OpCall("GetConstant", new MSA.Expression[] {
                Ast.CodeContext(),
                TransformQualifier(gen),
                TransformSymbolName(gen)
            });
        }

        public override MSA.Expression/*!*/ TransformWrite(AstGenerator/*!*/ gen, MSA.Expression/*!*/ rightValue) {
            Utils.Assert.NotNull(gen, rightValue);

            return gen.OpCall("SetConstant", new MSA.Expression[] {
                Ast.CodeContext(),
                TransformQualifier(gen),
                TransformSymbolName(gen),
                rightValue
            });
        }
    }
}
