/* ****************************************************************************
 *
 * Copyright (c) Microsoft Corporation. 
 *
 * This source code is subject to terms and conditions of the Microsoft Permissive License. A 
 * copy of the license can be found in the License.html file at the root of this distribution. If 
 * you cannot locate the  Microsoft Permissive License, please send an email to 
 * ironruby@microsoft.com. By using this source code in any fashion, you are agreeing to be bound 
 * by the terms of the Microsoft Permissive License.
 *
 * You must not remove this notice, or any other, from this software.
 *
 *
 * ***************************************************************************/

using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;

using Microsoft.Scripting;
using MSA = Microsoft.Scripting.Ast;
using Ast = Microsoft.Scripting.Ast.Ast;

using Ruby.Runtime;

namespace Ruby.Compiler.AST {
    public class GlobalVariable : Variable {
        public override string NodeName { get { return "global-variable"; } }

        // TODO: remove $ from name before
        public GlobalVariable(SymbolId name, SourceSpan location)
            : base(SymbolTable.StringToId(SymbolTable.IdToString(name).Substring(1)), location) {
            Debug.Assert(SymbolTable.IdToString(name).StartsWith("$"));
        }

        public override MSA.Expression/*!*/ TransformRead(AstGenerator/*!*/ gen) {
            if (Name == Symbols.Bang) {
                return gen.OpCall("GetCurrentException", Ast.CodeContext());
            }

            return gen.OpCall("GetGlobalVariable", new MSA.Expression[] { Ast.CodeContext(), TransformSymbolName(gen) });
        }

        public override MSA.Expression/*!*/ TransformWrite(AstGenerator/*!*/ gen, MSA.Expression/*!*/ rightValue) {
            if (Name == Symbols.Bang) {
                return gen.OpCall("SetCurrentException", Ast.CodeContext(), rightValue);
            }

            return gen.OpCall("SetGlobalVariable", new MSA.Expression[] { Ast.CodeContext(), TransformSymbolName(gen), rightValue });
        }
    }
}
