/* ****************************************************************************
 *
 * Copyright (c) Microsoft Corporation. 
 *
 * This source code is subject to terms and conditions of the Microsoft Permissive License. A 
 * copy of the license can be found in the License.html file at the root of this distribution. If 
 * you cannot locate the  Microsoft Permissive License, please send an email to 
 * ironruby@microsoft.com. By using this source code in any fashion, you are agreeing to be bound 
 * by the terms of the Microsoft Permissive License.
 *
 * You must not remove this notice, or any other, from this software.
 *
 *
 * ***************************************************************************/

using System;
using System.Collections.Generic;
using System.Text;

using Microsoft.Scripting;
using MSA = Microsoft.Scripting.Ast;
using Ast = Microsoft.Scripting.Ast.Ast;
using System.Diagnostics;

namespace Ruby.Compiler.AST {
    public class LocalVariable : Variable {
        private readonly LexicalScope/*!*/ _containingScope;

        public override string NodeName { get { return "local-variable"; } }

        public LocalVariable(LexicalScope/*!*/ containingScope, SymbolId name, SourceSpan location)
            : base(name, location) {
            Debug.Assert(containingScope != null);

            _containingScope = containingScope;
        }

        public override MSA.Expression/*!*/ TransformRead(AstGenerator/*!*/ gen) {
            return Ast.Read(BindVariable(gen.CurrentCodeBlock));
        }

        public override MSA.Expression/*!*/ TransformWrite(AstGenerator/*!*/ gen, MSA.Expression/*!*/ rightValue) {
            return Ast.Assign(BindVariable(gen.CurrentCodeBlock), rightValue);
        }

        private MSA.Variable/*!*/ BindVariable(MSA.CodeBlock/*!*/ declaringBlock) {
            VariableInfo var = _containingScope.ResolveVariable(Name);
            Debug.Assert(var.TransformedVariable != null);
            return var.TransformedVariable;
        }
    }
}
