/* ****************************************************************************
 *
 * Copyright (c) Microsoft Corporation. 
 *
 * This source code is subject to terms and conditions of the Microsoft Permissive License. A 
 * copy of the license can be found in the License.html file at the root of this distribution. If 
 * you cannot locate the  Microsoft Permissive License, please send an email to 
 * ironruby@microsoft.com. By using this source code in any fashion, you are agreeing to be bound 
 * by the terms of the Microsoft Permissive License.
 *
 * You must not remove this notice, or any other, from this software.
 *
 *
 * ***************************************************************************/

using System.Collections.Generic;
using System.Diagnostics;
using System.Reflection;
using System.Text;
using Microsoft.Scripting;
using MSAst = Microsoft.Scripting.Ast.Ast;

namespace Ruby.Compiler.AST {
    public class MATCH : Expression {
        // str =~ pattern

        private Node _str;
        private Node _pattern;

        public override string NodeName { get { return "method"; } }

        public MATCH(Node str, Node pattern, SourceSpan location)
            : base(location) {
            Debug.Assert(str != null && pattern != null);
            this._str = str;
            this._pattern = pattern;
        }
    }

    public class InclusiveRangeExpression : Expression {
        protected Expression _begin;
        protected Expression _end;

        public InclusiveRangeExpression(Expression/*!*/ begin, Expression/*!*/ end, SourceSpan location)
            : base(location) {
            Utils.Assert.NotNull(begin, end);
            _begin = begin;
            _end = end;
        }

        public override Microsoft.Scripting.Ast.Expression TransformRead(AstGenerator gen) {
            return gen.OpCall("CreateRange", new Microsoft.Scripting.Ast.Expression[] { _begin.TransformRead(gen), _end.TransformRead(gen) });
        }
    }

    public class ExclusiveRangeExpression : InclusiveRangeExpression {
        public ExclusiveRangeExpression(Expression/*!*/ begin, Expression/*!*/ end, SourceSpan location)
            : base(begin, end, location) { }

        public override Microsoft.Scripting.Ast.Expression TransformRead(AstGenerator gen) {
            return gen.OpCall("CreateExclusiveRange", new Microsoft.Scripting.Ast.Expression[] { _begin.TransformRead(gen), _end.TransformRead(gen) });
        }
    }
}