/* ****************************************************************************
 *
 * Copyright (c) Microsoft Corporation. 
 *
 * This source code is subject to terms and conditions of the Microsoft Permissive License. A 
 * copy of the license can be found in the License.html file at the root of this distribution. If 
 * you cannot locate the  Microsoft Permissive License, please send an email to 
 * ironruby@microsoft.com. By using this source code in any fashion, you are agreeing to be bound 
 * by the terms of the Microsoft Permissive License.
 *
 * You must not remove this notice, or any other, from this software.
 *
 *
 * ***************************************************************************/

using Microsoft.Scripting;
using System.Collections.Generic;
using MSA = Microsoft.Scripting.Ast;
using Ast = Microsoft.Scripting.Ast.Ast;
using Ruby.Runtime;

namespace Ruby.Compiler.AST
{
    public class Parameters : Node {
        internal static readonly Parameters/*!*/ Empty = new Parameters(null, null, null, null, SourceSpan.None);

        private readonly List<LocalVariable> _mandatory;
        private readonly List<SimpleAssignmentExpression> _optional;
        private readonly LocalVariable _array;
        private readonly LocalVariable _block;

        public List<LocalVariable> Mandatory {
            get { return _mandatory; }
        }

        public List<SimpleAssignmentExpression> Optional {
            get { return _optional; }
        }

        public LocalVariable Array {
            get { return _array; }
        }

        public LocalVariable Block {
            get { return _block; }
        }

        public int MandatoryCount {
            get {
                return (_mandatory != null) ? _mandatory.Count : 0;
            }
        }

        public int OptionalCount {
            get {
                return (_optional != null) ? _optional.Count : 0;
            }
        }

        public Parameters(List<LocalVariable> mandatory, List<SimpleAssignmentExpression> optional, 
            LocalVariable array, LocalVariable block, SourceSpan location)
            : base(location) {

            _mandatory = mandatory;
            _optional = optional;
            _array = array;
            _block = block;
        }

        internal MSA.Statement/*!*/ TransformOptionalsInitialization(AstGenerator/*!*/ gen) {
            Utils.Assert.NotNull(gen);

            if (_optional == null) return Ast.Empty();

            MSA.Variable singleton = gen.CurrentCodeBlock.CreateTemporaryVariable(SymbolTable.StringToId("#default"), typeof(object));

            MSA.Statement result = Ast.Empty();
            for (int i = 0; i < _optional.Count; i++) {
                result = Ast.IfThen(Ast.Equal(_optional[i].Left.TransformRead(gen), Ast.Read(singleton)),
                    result,
                    Ast.Statement(_optional[i].TransformRead(gen))
                );
            }
            
            return Ast.Block(
                Ast.Statement(Ast.Assign(singleton, Ast.ReadField(null, RubyOps.DefaultArgumentField))),
                result
            );
        }
    }
}