/* ****************************************************************************
 *
 * Copyright (c) Microsoft Corporation. 
 *
 * This source code is subject to terms and conditions of the Microsoft Permissive License. A 
 * copy of the license can be found in the License.html file at the root of this distribution. If 
 * you cannot locate the  Microsoft Permissive License, please send an email to 
 * ironruby@microsoft.com. By using this source code in any fashion, you are agreeing to be bound 
 * by the terms of the Microsoft Permissive License.
 *
 * You must not remove this notice, or any other, from this software.
 *
 *
 * ***************************************************************************/

using System.Collections.Generic;
using System.Text;
using System.Diagnostics;

using MSA = Microsoft.Scripting.Ast;
using Ast = Microsoft.Scripting.Ast.Ast;
using Microsoft.Scripting;
using Ruby.Runtime;

namespace Ruby.Compiler.AST {

    public sealed class SourceUnitTree : Node {

        private readonly List<Statement> _statements;
        private readonly LexicalScope/*!*/ _scope;
        private readonly List<Initializer> _initializers;

        public LexicalScope Scope {
            get { return _scope; }
        }

        public SourceUnitTree(LexicalScope/*!*/ scope, List<Statement> statements, List<Initializer> initializers, SourceSpan location)
            : base(location) {
            Utils.Assert.NotNull(scope);

            _scope = scope;
            _statements = statements;
            _initializers = initializers;
        }

        public MSA.CodeBlock/*!*/ Transform(AstGenerator/*!*/ gen) {
            Debug.Assert(gen != null);

            MSA.CodeBlock globalBlock = Ast.CodeBlock(Location, "main");
            
            // TODO: we should get RCF from LanguageContext similarly to self (eval should pass it thru LC)
            MSA.Variable oldRuntimeControlFlowVariable = gen.CurrentRuntimeFlowControlVariable;
            gen.CurrentCodeBlock = globalBlock;
            gen.CurrentRuntimeFlowControlVariable = globalBlock.CreateLocalVariable(SymbolTable.StringToId("#rfc"), typeof(RuntimeFlowControl));
            
            gen.CurrentScope = _scope;

            _scope.TransformLocals(globalBlock, gen);

            globalBlock.Body = gen.TransformBodyStatements(_statements, new MSA.Statement[] {
                AstFactory.Assign(gen.CurrentRuntimeFlowControlVariable, 
                    // TODO: this should read from LC (eval ...):
                    Ast.New(RuntimeFlowControl.Constructor, new MSA.Expression[] { AstFactory.False })) 
            }, null);

            gen.CurrentScope = null;
            gen.CurrentCodeBlock = null;

            return globalBlock;
        }
    }
}
