/* ****************************************************************************
 *
 * Copyright (c) Microsoft Corporation. 
 *
 * This source code is subject to terms and conditions of the Microsoft Permissive License. A 
 * copy of the license can be found in the License.html file at the root of this distribution. If 
 * you cannot locate the  Microsoft Permissive License, please send an email to 
 * ironruby@microsoft.com. By using this source code in any fashion, you are agreeing to be bound 
 * by the terms of the Microsoft Permissive License.
 *
 * You must not remove this notice, or any other, from this software.
 *
 *
 * ***************************************************************************/

using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Scripting;

namespace Ruby.Compiler {
    public enum NamespaceType { NONE, LOCAL, INSTANCE, GLOBAL, ATTRSET, CONST, CLASS, INTERNAL, JUNK };

    // IDs are currently represented as CLR strings - this may be changed in future optimized versions.
    internal class ID {
        public static SymbolId intern(string name) {
            return SymbolTable.StringToId(name);
        }

        public static SymbolId intern(char name) {
            return SymbolTable.StringToId(name.ToString());
        }

        public static SymbolId intern(Tokens token) {
            switch (token) {
                case Tokens.tDOT2: return Symbols.DoubleDot;
                case Tokens.tDOT3: return Symbols.TripleDot;
                case Tokens.tPOW: return Symbols.DoubleStar;
                case Tokens.tUPLUS: return Symbols.UnaryPlus;
                case Tokens.tUMINUS: return Symbols.UnaryMinus;
                case Tokens.tCMP: return Symbols.Comparison;
                case Tokens.tGEQ: return Symbols.GreaterEqual;
                case Tokens.tLEQ: return Symbols.LessEqual;
                case Tokens.tEQ: return Symbols.Equal;
                case Tokens.tEQQ: return Symbols.StrictEqual;
                case Tokens.tNEQ: return Symbols.NotEqual;
                case Tokens.tMATCH: return Symbols.Match;
                case Tokens.tNMATCH: return Symbols.NotMatch;
                case Tokens.tAREF: return Symbols.ArrayItemRead;
                case Tokens.tASET: return Symbols.ArrayItemWrite;
                case Tokens.tLSHFT: return Symbols.LeftShift;
                case Tokens.tRSHFT: return Symbols.RightShift;
                case Tokens.tSEPARATING_DOUBLE_COLON: return Symbols.DoubleColon;
                case Tokens.tOROP: return Symbols.Or;
                case Tokens.tANDOP: return Symbols.And;
                default: return SymbolTable.StringToId(token.ToString());
            }
        }

        public static NamespaceType GetNamespaceType(SymbolId identifier) {
            return GetNamespaceType(SymbolTable.IdToString(identifier));
        }
        
        public static NamespaceType GetNamespaceType(string identifier) {
            switch (identifier[0]) {
                case '$':
                    return NamespaceType.GLOBAL;
                case '@':
                    if (identifier[1] == '@')
                        return NamespaceType.CLASS;
                    else
                        return NamespaceType.INSTANCE;
                default:
                    if (identifier[identifier.Length - 1] == '=')
                        return NamespaceType.ATTRSET;
                    if (Char.IsUpper(identifier[0]))
                        return NamespaceType.CONST;
                    else
                        return NamespaceType.LOCAL;
            }
        }
    }
}
