/* ****************************************************************************
 *
 * Copyright (c) Microsoft Corporation. 
 *
 * This source code is subject to terms and conditions of the Microsoft Permissive License. A 
 * copy of the license can be found in the License.html file at the root of this distribution. If 
 * you cannot locate the  Microsoft Permissive License, please send an email to 
 * ironruby@microsoft.com. By using this source code in any fashion, you are agreeing to be bound 
 * by the terms of the Microsoft Permissive License.
 *
 * You must not remove this notice, or any other, from this software.
 *
 *
 * ***************************************************************************/

%namespace Ruby.Compiler

%using Ruby.Compiler.AST
%using Microsoft.Scripting
%using Microsoft.Scripting.Math
%using Microsoft.Scripting.Hosting
%using System.Diagnostics

%union { }

%YYLTYPE SourceSpan
%YYSTYPE TokenValue

%partial
%visibility public

%token kCLASS kMODULE kDEF kUNDEF kBEGIN kRESCUE kENSURE kEND kIF kUNLESS kTHEN kELSIF kELSE
%token kCASE kWHEN kWHILE kUNTIL kFOR kBREAK kNEXT kREDO kRETRY kIN kDO kDO_COND kDO_BLOCK
%token kRETURN kYIELD kSUPER kSELF kNIL kTRUE kFALSE kAND kOR kNOT kIF_MOD kUNLESS_MOD
%token kWHILE_MOD kUNTIL_MOD kRESCUE_MOD kALIAS kDEFINED klBEGIN klEND k__LINE__ k__FILE__
%token tUPLUS tUMINUS tPOW tCMP tEQ tEQQ tNEQ tGEQ tLEQ tANDOP tOROP tMATCH tNMATCH
%token tDOT2 tDOT3 tAREF tASET tLSHFT tRSHFT tSEPARATING_DOUBLE_COLON tLEADING_DOUBLE_COLON tASSOC tLPAREN tSTRING_END
%token tLPAREN_ARG tLBRACK tLBRACE tLBRACE_ARG tSTAR tAMPER tSTRING_EMBEDDED_VARIABLE_BEGIN

%token<SymbolId> tIDENTIFIER tFID tGVAR tIVAR tCONSTANT tCVAR tOP_ASGN 
%token<Integer> tINTEGER 
%token<BigInteger> tBIG_INTEGER 
%token<Double> tFLOAT 
%token<String> tSTRING_CONTENT 
%token<Integer> tNTH_REF 
%token<SymbolId> tBACK_REF
%token<Integer>  tREGEXP_END 
%token<term> tSTRING_BEG tREGEXP_BEG tSHELL_STRING_BEGIN tWORDS_BEG tVERBATIM_WORDS_BEGIN tSTRING_EMBEDDED_CODE_BEGIN tSYMBEG

%type<AbstractSyntaxTree> program
%type<Statement> stmt
%type<Statements> stmts compstmt ensure_opt
%type<Expression> primary expr expression_statement superclass var_ref singleton case_expression
%type<Expression> arg
%type<Expressions> args exc_list
%type<Expression> block_expression declaration_expression
%type<Body> body
%type<Expression> 

%type<CallExpression> method_call block_call command_call block_command command

%type<ElseIfClause> else_opt 
%type<ElseIfClauses> if_tail
%type<UndefinedItems> undef_list 
%type<BlockReference> block_arg opt_block_arg 
%type<BlockDefinition> cmd_brace_block brace_block do_block 
%type<Arguments> when_args array_key open_args closed_args command_args_content paren_args opt_paren_args command_args 
%type<RightValues> mrhs
%type<ConstantVariable> qualified_module_name 

%type<RescueClauses> rescue_clauses rescue_clauses_opt
%type<RescueClause> rescue_clause

%type<WhenClauses> when_clauses 
%type<WhenClause> when_clause                          
 
%type<Maplets> maplets 
%type<Maplet> maplet 

%type<Parameters> parameters_declaration parameters
%type<LocalVariables> parameter_list
%type<LocalVariable> parameter array_parameter block_parameter block_parameter_opt
%type<SimpleAssignmentExpressions> default_parameter_list 
%type<SimpleAssignmentExpression> default_parameter 

%type<Expression> string_embedded_variable              // string embedded variable
%type<Expression> string_content                        // string piece - literal, string embedded code/variable
%type<Expressions> string_contents                      // list of string pieces
%type<Expression> string                                // quoted string constructor taking list of string pieces "#{foo}bar#{baz}"
%type<Expressions> string_concatenation                 // string constructor taking a list of quoted strings "foo" 'bar' "baz"

%type<Expression> shell_string                          // shell string constructor taking list of string pieces `#{foo}bar#{baz}`

%type<Expressions> word                                 // concatenation of string pieces
%type<Expressions> word_list verbatim_word_list         // list of words separated by space
%type<Expression> words verbatim_words                  // array constructor taking a list of words

%type<Expression> regexp 
%type<Expression> numeric_literal 
%type<Expression> symbolic_expression 
%type<Expression> backref 

%type<SymbolId> operation 
%type<SymbolId> variable 
%type<SymbolId> sym 
%type<SymbolId> operation2 
%type<SymbolId> operation3 
%type<SymbolId> module_name 
%type<SymbolId> op 
%type<SymbolId> fname 
%type<SymbolId> symbol 
%type<SymbolId> fitem

%type<CompoundLeftValue> compound_lhs 
%type<LeftValues> compound_lhs_head 
%type<LeftValue> compound_lhs_item compound_lhs_tail
%type<LeftValue> compound_lhs_node 

%type<LeftValue> var_lhs 
%type<LeftValue> lhs 
%type<CompoundLeftValue> block_parameters block_parameters_opt 
%type<LeftValue> exc_var

%nonassoc tLOWEST
%nonassoc tLBRACE_ARG
%nonassoc  kIF_MOD kUNLESS_MOD kWHILE_MOD kUNTIL_MOD
%left  kOR kAND
%right kNOT
%nonassoc kDEFINED
%right '=' tOP_ASGN
%left kRESCUE_MOD
%right '?' ':'
%nonassoc tDOT2 tDOT3
%left  tOROP
%left  tANDOP
%nonassoc  tCMP tEQ tEQQ tNEQ tMATCH tNMATCH
%left  '>' tGEQ '<' tLEQ
%left  '|' '^'
%left  '&'
%left  tLSHFT tRSHFT
%left  '+' '-'
%left  '*' '/' '%'
%right tUMINUS_NUM tUMINUS
%right tPOW
%right '!' '~' tUPLUS

%token tLAST_TOKEN


%%


program : {
            _tokenizer.lex_state = LexicalState.EXPR_BEG;
          }
		compstmt
          {
            _ast = new SourceUnitTree(CurrentScope, $2, _initializers, @$);
          }
		;

compstmt: stmts	opt_terms
			{
			 	$$ = $1;
			}
		;

stmts	: /* empty */
            {
                $$ = new List<Statement>();
            }
		| stmt 
		    {
		        $$ = Utils.Collections.MakeList<Statement>($1);
		    }
		| stmts terms stmt
			{
				$1.Add($3);
				$$ = $1;
			}
		| error stmt
			{
				$$ = Utils.Collections.MakeList<Statement>($2);
			}
		;

stmt	: kALIAS fitem 
			{
				_tokenizer.lex_state =	LexicalState.EXPR_FNAME;
			} 
		  fitem
			{
				$$ = new AliasStatement(CurrentScope, $2, $4, @$);
			}
		| kALIAS tGVAR tGVAR
			{
				$$ = new AliasStatement(null, $2, $3, @$);
			}
		| kALIAS tGVAR tBACK_REF
			{
				$$ = new AliasStatement(null, $2, $3, @$);
			}
		| kALIAS tGVAR tNTH_REF
		    {
		        _tokenizer.SoftError("cant make alias for the number variable");
		        $$ = new ExpressionStatement(new ErrorExpression(@$));
		    }
		| kUNDEF undef_list
			{
				$$ = new UndefineStatement(CurrentScope, $2, @$);
			}
		| stmt kIF_MOD expr
			{
				$$ = new IfStatement($3, false, $1, null, @$);
			}
		| stmt kUNLESS_MOD expr
			{
				$$ = new IfStatement($3, true, $1, null, @$);
			}
		| stmt kWHILE_MOD expr
			{
			    $$ = MakeLoopStatement($1, $3, true, @$);
			}
		| stmt kUNTIL_MOD expr
			{
			    $$ = MakeLoopStatement($1, $3, false, @$);
			}
		| stmt kRESCUE_MOD stmt
			{
				$$ = new RescueStatement($1, $3, @$);
			}
		| klBEGIN
			{
			    if (in_def != 0 || in_single != 0)
			        _tokenizer.SoftError("BEGIN in method");
			    			
				EnterNewScope(ScopeKind.Initializer);
			}
		  '{' compstmt '}'
			{
				$$ = AddInitializer(new Initializer(CurrentScope, $4, @$));
				LeaveScope();
			}
		| klEND
		    {
		       if (in_def != 0 || in_single != 0)
			        _errorSink.Add(_sourceUnit, "END in method; use at_exit", @1, -1, Severity.Warning);
		        
		        EnterNewScope(ScopeKind.Block);
		    } 
		  '{' compstmt '}'
			{					
				$$ = new Finalizer(CurrentScope, $4, @$);
				LeaveScope();
			}		
		| backref tOP_ASGN command_call
		    {
		        backref_error($1);
				$$ = null;
		    } 
	    | kRETURN
			{
				$$ = new ReturnStatement(CurrentScope, null, @$);
			}
		| kRETURN open_args
		    {
			    $$ = new ReturnStatement(CurrentScope, RequireNoBlockArg($2), @$);
		    }
		| kBREAK
			{
				$$ = new BreakStatement(CurrentScope, null, @$);
			}
		| kBREAK open_args
			{
				$$ = new BreakStatement(CurrentScope, RequireNoBlockArg($2), @$);
			}
		| kNEXT
			{
				$$ = new NextStatement(CurrentScope, null, @$);
			}
		| kNEXT open_args
			{
				$$ = new NextStatement(CurrentScope, RequireNoBlockArg($2), @$);
			}
		| kREDO
			{
				$$ = new RedoStatement(CurrentScope, @$);
			}
		| kRETRY
			{
				$$ = new RetryStatement(CurrentScope, @$);
			}
	    | expression_statement
	        {
	            $$ = new ExpressionStatement($1);    
	        }
		;

expression_statement: 
      expr
        {
            $$ = $1;
        }
    | lhs '=' command_call
		{
			$$ = new SimpleAssignmentExpression($1, $3, SymbolId.Empty, @$);
		}
	| compound_lhs '=' command_call
		{
			$$ = new ParallelAssignmentExpression($1, new RightValues(Utils.Collections.MakeList<Expression>($3), null), SymbolId.Empty, @$);
		}
	| var_lhs tOP_ASGN command_call
		{
			$$ = new SimpleAssignmentExpression($1, $3, $2, @$);
		}
	| primary '[' array_key ']' tOP_ASGN command_call
		{				
			$$ = new SimpleAssignmentExpression(new ArrayItemAccess($1, $3, @2), $6, $5, @$);
		}
	| primary '.' tIDENTIFIER tOP_ASGN command_call
		{
			$$ = new MemberAssignmentExpression($1, $3, $4, $5, @$);
		}
	| primary '.' tCONSTANT tOP_ASGN command_call
		{
			$$ = new MemberAssignmentExpression($1, $3, $4, $5, @$);
		}
	| primary tSEPARATING_DOUBLE_COLON tIDENTIFIER tOP_ASGN command_call
		{
			$$ = new MemberAssignmentExpression($1, $3, $4, $5, @$);
		}
	| lhs '=' mrhs
		{
			$$ = new ParallelAssignmentExpression(new CompoundLeftValue(Utils.Collections.MakeList<LeftValue>($1), null, @1), $3, SymbolId.Empty, @$);
		}
	| compound_lhs '=' arg
		{
			$$ = new ParallelAssignmentExpression($1, new RightValues(Utils.Collections.MakeList<Expression>($3), null), SymbolId.Empty, @$);
		}
	| compound_lhs '=' mrhs
		{
			$$ = new ParallelAssignmentExpression($1, $3, SymbolId.Empty, @$);
		}
	;
	
mrhs: args ',' arg
		{
			$1.Add($3);
			$$ = new RightValues($1, null);
		}
	| args ',' tSTAR arg
		{
			$$ = new RightValues($1, $4);
		}
	| tSTAR arg
		{
			$$ = new RightValues(null, $2);
		}
	;
			
expr	: command_call
		| expr kAND expr
			{
				$$ = new AndExpression($1, $3, @$);
			}
		| expr kOR expr
			{
				$$ = new OrExpression($1, $3, @$);
			}
		| kNOT expr
			{
				// TODO: warning: string literal in condition
				$$ = new NotExpression($2, @$);
			}
		| '!' command_call
			{
				// TODO: warning: string literal in condition
				$$ = new NotExpression($2, @$);
			}
		| arg
		;

command_call: command
				{
					$$ = $1;
				}
			| block_command
				{
					$$ = $1;
				}
			;

block_command	: block_call
					{
						$$ = $1;
					}
				| block_call '.' operation2 command_args
					{
						$$ = new MethodCall($1, $3, $4, @$);
					}
				| block_call tSEPARATING_DOUBLE_COLON operation2 command_args
					{
						$$ = new MethodCall($1, $3, $4, @$);
					}
				;

cmd_brace_block	: tLBRACE_ARG
					{
						EnterNewScope(ScopeKind.Block);
					}
				  block_parameters_opt 
					{
					}
				  compstmt
				  '}'
					{
						$$ = new BlockDefinition(CurrentScope, $3, $5, @$);
						LeaveScope();
					}
		;

command	: operation	command_args							%prec tLOWEST
			{
				$$ = new MethodCall(null, $1, $2, @$);
			}
		| operation command_args cmd_brace_block
			{
			    $2.Block = $3;
				$$ = new MethodCall(null, $1, $2, @$);
			}
		| primary '.' operation2 command_args				%prec tLOWEST
			{
				$$ = new MethodCall($1, $3, $4, @$);
			}
		| primary '.' operation2 command_args cmd_brace_block
			{
			    $4.Block = $5;
				$$ = new MethodCall($1, $3, $4, @$);
			}
		| primary tSEPARATING_DOUBLE_COLON operation2 command_args			%prec tLOWEST
			{
				$$ = new MethodCall($1, $3, $4, @$);
			}
		| primary tSEPARATING_DOUBLE_COLON operation2 command_args cmd_brace_block
			{
			    $4.Block = $5;
				$$ = new MethodCall($1, $3, $4, @$);
			}
		| kSUPER command_args
			{
				$$ = new SuperCall(CurrentScope, $2, @$);
			}
		| kYIELD command_args
			{
				$$ = new YieldCall($2, @$);
			}
		;

compound_lhs: 
	  compound_lhs_head compound_lhs_item
  		{
  			$1.Add($2);
			$$ = new CompoundLeftValue($1, null, @$);
  		}
	| compound_lhs_head 
  		{
			$$ = new CompoundLeftValue($1, null, @$);
  		}
	| tLPAREN compound_lhs ')'
  		{
			$$ = new CompoundLeftValue(Utils.Collections.MakeList<LeftValue>($2), null, @$);
  		}
	| compound_lhs_head compound_lhs_tail
	    {
			$$ = new CompoundLeftValue($1, $2, @$);
  	    }
  	| compound_lhs_tail
		{
			$$ = new CompoundLeftValue(LeftValue.EmptyList, $1, @$);
		}
;

compound_lhs_tail:
	  tSTAR compound_lhs_node
		{
			$$ = $2;
		}
	| tSTAR
		{
			$$ = null;
		}
;

compound_lhs_head:
	  compound_lhs_head compound_lhs_item ','
		{
			$1.Add($2);
			$$ = $1;
		}
	| compound_lhs_item ','
		{
			$$ = Utils.Collections.MakeList($1);
		}
;

compound_lhs_item:
	  compound_lhs_node
		{
			$$ = $1;
		}
	| tLPAREN compound_lhs ')'
		{
			$$ = $2;
		}
;

compound_lhs_node: 
	  variable
		{
			$$ = assignable($1, @$);
		}
	| primary '[' array_key ']'
		{
			$$ = new ArrayItemAccess($1, $3, @$);
		}
	| primary '.' tIDENTIFIER
		{
			$$ = new AttributeAccess($1, $3, @$);
		}
	| primary tSEPARATING_DOUBLE_COLON tIDENTIFIER
		{
			$$ = new AttributeAccess($1, $3, @$);
		}
	| primary '.' tCONSTANT
		{
			$$ = new AttributeAccess($1, $3, @$);
		}
	| primary tSEPARATING_DOUBLE_COLON tCONSTANT
		{
			$$ = new ConstantVariable(CurrentScope, $1, $3, @$);
		}
	| tLEADING_DOUBLE_COLON tCONSTANT
		{
			$$ = new ConstantVariable(CurrentScope, null, $2, @$);
		}
	| backref
		{
			backref_error($1);
			$$ = null;
		}
;


lhs	: variable
		{
			$$ = assignable($1, @$);
		}
	| primary '[' array_key ']'
		{
			$$ = new ArrayItemAccess($1, $3, @$);
		}
	| primary '.' tIDENTIFIER
		{
			$$ = new AttributeAccess($1, $3, @$);
		}
	| primary tSEPARATING_DOUBLE_COLON tIDENTIFIER
		{
			$$ = new AttributeAccess($1, $3, @$);
		}
	| primary '.' tCONSTANT
		{
			$$ = new AttributeAccess($1, $3, @$);
		}
	| primary tSEPARATING_DOUBLE_COLON tCONSTANT
		{
			$$ = new ConstantVariable(CurrentScope, $1, $3, @$);
		}
	| tLEADING_DOUBLE_COLON tCONSTANT
		{
			$$ = new ConstantVariable(CurrentScope, null, $2, @$);
		}
	| backref
		{
			backref_error($1);
			$$ = null;
		}
	;

module_name: 
      tCONSTANT
      {
          $$ = $1;
      }
    | tIDENTIFIER
      {
          _tokenizer.SoftError("Class/module name must be a constant");
          $$ = $1;
      }
;

qualified_module_name: 
      tLEADING_DOUBLE_COLON module_name
      {
          $$ = new ConstantVariable(CurrentScope, null, $2, @$);
      }
    | module_name
      {
          $$ = new ConstantVariable(CurrentScope, $1, @$);
      }
    | primary tSEPARATING_DOUBLE_COLON module_name
      {
          $$ = new ConstantVariable(CurrentScope, $1, $3, @$);
      }
;

fname	: tIDENTIFIER
			{
				$$ = $1;	
			}
		| tCONSTANT
			{
				$$ = $1;	
			}		
		| tFID
			{
				$$ = $1;	
			}		
		| op
			{
				_tokenizer.lex_state = LexicalState.EXPR_END;
				$$ = $1;
			}
		| reswords
			{
				_tokenizer.lex_state = LexicalState.EXPR_END;
				$$ = $<SymbolId>1;
			}
		;

fitem	: fname
			{
				$$ = $1;
			}
		| symbol
			{
				$$ = $1;
			}
		;

undef_list	: fitem
				{
					$$ = Utils.Collections.MakeList<UndefinedItem>(new UndefinedItem($1, @1));
				}
			| undef_list ',' 
				{
					_tokenizer.lex_state = LexicalState.EXPR_FNAME;
				} 
			  fitem
				{
					$1.Add(new UndefinedItem($4, @4));
					$$ = $1;
				}
			;

op		: '|'		{ $$ = ID.intern('|');	}
		| '^'		{ $$ = ID.intern('^'); }
		| '&'		{ $$ = ID.intern('&'); }
		| tCMP		{ $$ = ID.intern(Tokens.tCMP); }
		| tEQ		{ $$ = ID.intern(Tokens.tEQ); }
		| tEQQ		{ $$ = ID.intern(Tokens.tEQQ); }
		| tMATCH	{ $$ = ID.intern(Tokens.tMATCH); }
		| '>'		{ $$ = ID.intern('>'); }
		| tGEQ		{ $$ = ID.intern(Tokens.tGEQ); }
		| '<'		{ $$ = ID.intern('<'); }
		| tLEQ		{ $$ = ID.intern(Tokens.tLEQ); }
		| tLSHFT	{ $$ = ID.intern(Tokens.tLSHFT); }
		| tRSHFT	{ $$ = ID.intern(Tokens.tRSHFT); }
		| '+'		{ $$ = ID.intern('+'); }
		| '-'		{ $$ = ID.intern('-'); }
		| '*'		{ $$ = ID.intern('*'); }
		| tSTAR		{ $$ = ID.intern('*'); }
		| '/'		{ $$ = ID.intern('/'); }
		| '%'		{ $$ = ID.intern('%'); }
		| tPOW		{ $$ = ID.intern(Tokens.tPOW); }
		| '~'		{ $$ = ID.intern('~'); }
		| tUPLUS	{ $$ = ID.intern(Tokens.tUPLUS); }
		| tUMINUS	{ $$ = ID.intern(Tokens.tUMINUS); }
		| tAREF		{ $$ = ID.intern(Tokens.tAREF); }
		| tASET		{ $$ = ID.intern(Tokens.tASET); }
		| '`'		{ $$ = ID.intern('`'); }
		;

reswords: k__LINE__	| k__FILE__	 | klBEGIN | klEND
		| kALIAS | kAND | kBEGIN | kBREAK | kCASE | kCLASS | kDEF
		| kDEFINED | kDO | kELSE | kELSIF | kEND | kENSURE | kFALSE
		| kFOR | kIN | kMODULE | kNEXT | kNIL | kNOT
		| kOR | kREDO | kRESCUE | kRETRY | kRETURN | kSELF | kSUPER
		| kTHEN | kTRUE | kUNDEF | kWHEN | kYIELD
		| kIF_MOD | kUNLESS_MOD | kWHILE_MOD | kUNTIL_MOD | kRESCUE_MOD
		;

arg		: lhs '=' arg
			{
				$$ = new SimpleAssignmentExpression($1, $3, SymbolId.Empty, @$);
			}
		| lhs '=' arg kRESCUE_MOD arg
			{
				$$ = new SimpleAssignmentExpression($1, new RescueExpression($3, $5, @4), SymbolId.Empty, @$);
			}
		| var_lhs tOP_ASGN arg
			{
				$$ = new SimpleAssignmentExpression($1, $3, $2, @$);
			}
		| primary '[' array_key ']' tOP_ASGN arg
			{
				$$ = new SimpleAssignmentExpression(new ArrayItemAccess($1, $3, @2), $6, $5, @$);
			}
		| primary '.' tIDENTIFIER tOP_ASGN arg
			{
				$$ = new MemberAssignmentExpression($1, $3, $4, $5, @$);
			}
		| primary '.' tCONSTANT tOP_ASGN arg
			{
				$$ = new MemberAssignmentExpression($1, $3, $4, $5, @$);
			}
		| primary tSEPARATING_DOUBLE_COLON tIDENTIFIER tOP_ASGN arg
			{
				$$ = new MemberAssignmentExpression($1, $3, $4, $5, @$);
			}
		| primary tSEPARATING_DOUBLE_COLON tCONSTANT tOP_ASGN arg
			{
				_tokenizer.SoftError("constant re-assignment");
				$$ = new ErrorExpression(@$);
			}
		| tLEADING_DOUBLE_COLON tCONSTANT tOP_ASGN arg
			{
				_tokenizer.SoftError("constant re-assignment");
				$$ = new ErrorExpression(@$);
			}
		| backref tOP_ASGN arg
			{
				backref_error($1);
				$$ = null;
			}
		| arg tDOT2 arg
			{
				$$ = new InclusiveRangeExpression($1, $3, @$);
			}
		| arg tDOT3 arg
			{
				$$ = new ExclusiveRangeExpression($1, $3, @$);
			}
		| arg '+' arg
			{
				$$ = new MethodCall($1, ID.intern('+'), new Arguments($3), @$);
			}
		| arg '-' arg
			{
				$$ = new MethodCall($1, ID.intern('-'), new Arguments($3), @$);
			}
		| arg '*' arg
			{
				$$ = new MethodCall($1, ID.intern('*'), new Arguments($3), @$);
			}
		| arg '/' arg
			{
				$$ = new MethodCall($1, ID.intern('/'), new Arguments($3), @$);
			}
		| arg '%' arg
			{
				$$ = new MethodCall($1, ID.intern('%'), new Arguments($3), @$);
			}
		| arg tPOW arg
			{
				$$ = new MethodCall($1, ID.intern(Tokens.tPOW), new Arguments($3), @$);
			}
		| tUMINUS_NUM tINTEGER tPOW arg
			{
				$$ = new MethodCall(new MethodCall(new Literal($2, @2), ID.intern(Tokens.tPOW), new Arguments($4), @3), ID.intern(Tokens.tUMINUS), new Arguments(@1), @$);
			}
		| tUMINUS_NUM tBIG_INTEGER tPOW arg
			{
				$$ = new MethodCall(new MethodCall(new Literal($2, @2), ID.intern(Tokens.tPOW), new Arguments($4), @3), ID.intern(Tokens.tUMINUS), new Arguments(@1), @$);
			}
		| tUMINUS_NUM tFLOAT tPOW arg
			{
				$$ = new MethodCall(new MethodCall(new Literal($2, @2), ID.intern(Tokens.tPOW), new Arguments($4), @3), ID.intern(Tokens.tUMINUS), new Arguments(@1), @$);
			}
		| tUPLUS arg
			{
				$$ = new MethodCall($2, ID.intern(Tokens.tUPLUS), new Arguments(@2), @$);
			}
		| tUMINUS arg
			{
				$$ = new MethodCall($2, ID.intern(Tokens.tUMINUS), new Arguments(@2), @$);
			}
		| arg '|' arg
			{
				$$ = new MethodCall($1, ID.intern('|'), new Arguments($3), @$);
			}
		| arg '^' arg
			{
				$$ = new MethodCall($1, ID.intern('^'), new Arguments($3), @$);
			}
		| arg '&' arg
			{
				$$ = new MethodCall($1, ID.intern('&'), new Arguments($3), @$);
			}
		| arg tCMP arg
			{
				$$ = new MethodCall($1, ID.intern(Tokens.tCMP), new Arguments($3), @$);
			}
		| arg '>' arg
			{
				$$ = new MethodCall($1, ID.intern('>'), new Arguments($3), @$);
			}
		| arg tGEQ arg
			{
				$$ = new MethodCall($1, ID.intern(Tokens.tGEQ), new Arguments($3), @$);
			}
		| arg '<' arg
			{
				$$ = new MethodCall($1, ID.intern('<'), new Arguments($3), @$);
			}
		| arg tLEQ arg
			{
				$$ = new MethodCall($1, ID.intern(Tokens.tLEQ), new Arguments($3), @$);
			}
		| arg tEQ arg
			{
				$$ = new MethodCall($1, ID.intern(Tokens.tEQ), new Arguments($3), @$);
			}
		| arg tEQQ arg
			{
				$$ = new MethodCall($1, ID.intern(Tokens.tEQQ), new Arguments($3), @$);
			}
		| arg tNEQ arg
			{
				$$ = new NotExpression(new MethodCall($1, ID.intern(Tokens.tEQ), new Arguments($3), @$), @$);
			}
		| arg tMATCH arg
			{
				//$$ = new MatchExpression($1, $3, @$);
			}
		| arg tNMATCH arg
			{
				//$$ = new NotExpression(new MatchExpression($1, $3, @$), @$);
			}
		| '!' arg
			{
				// TODO: warning: string literal in condition
				$$ = new NotExpression($2, @$);
			}
		| '~' arg
			{
				$$ = new MethodCall($2, ID.intern('~'), new Arguments(@2), @$);
			}
		| arg tLSHFT arg
			{
				$$ = new MethodCall($1, ID.intern(Tokens.tLSHFT), new Arguments($3), @$);
			}
		| arg tRSHFT arg
			{
				$$ = new MethodCall($1, ID.intern(Tokens.tRSHFT), new Arguments($3), @$);
			}
		| arg tANDOP arg
			{
				$$ = new AndExpression($1, $3, @$);
			}
		| arg tOROP arg
			{
				$$ = new OrExpression($1, $3, @$);
			}
		| kDEFINED opt_nl 
			{
				// Empty placeholder
			}
		  arg
			{
				$$ = new AstNodeDescriptionExpression($4, @$);
			}
		| arg '?' arg ':' arg
			{
				$$ = new ConditionalExpression($1, $3, $5, @$);
			}
		| primary
			{
				$$ = $1;
			}
		;
		
array_key	: /* empty */
				{
					$$ = new Arguments(@$);
				}
			| command opt_nl
				{
					_tokenizer.yywarn("parenthesize argument(s) for future version");
					$$ = new Arguments($1);
				}
			| args trailer
				{
					$$ = new Arguments($1, null, null, null, @1);
				}
			| args ',' tSTAR arg opt_nl
				{
					$$ = new Arguments($1, null, $4, null, MergeLocations(@1, @4));
				}
			| maplets trailer
				{
					$$ = new Arguments(null, $1, null, null, @1);
				}
			| tSTAR arg opt_nl
				{
					$$ = new Arguments(null, null, $2, null, MergeLocations(@1, @2));
				}
			;

paren_args	: '(' /* empty */ ')'
				{
					$$ = new Arguments(@$);
				}
			| '(' open_args opt_nl ')'
				{
					$$ = $2;
				}
			| '(' block_call opt_nl ')'
				{
					_tokenizer.yywarn("parenthesize argument for future version");
					$$ = new Arguments($2);
				}
			| '(' args ',' block_call opt_nl ')'
				{
					_tokenizer.yywarn("parenthesize argument for future version");	
					$2.Add($4);
					$$ = new Arguments($2, null, null, null, @$);
				}
			;

opt_paren_args	: /* empty */
                {
					$$ = new Arguments(@$);
                }

				| paren_args
				;

open_args	: args opt_block_arg
				{
					$$ = new Arguments($1, null, null, $2, @$);
				}
			| args ',' tSTAR arg opt_block_arg
				{
					$$ = new Arguments($1, null, $4, $5, @$);
				}
			| maplets opt_block_arg
				{
					$$ = new Arguments(null, $1, null, $2, @$);
				}
			| maplets ',' tSTAR arg opt_block_arg
				{
					$$ = new Arguments(null, $1, $4, $5, @$);
				}
			| args ',' maplets opt_block_arg
				{
					$$ = new Arguments($1, $3, null, $4, @$);
				}
			| args ',' maplets ',' tSTAR arg opt_block_arg
				{
					$$ = new Arguments($1, $3, $6, $7, @$);
				}
			| tSTAR arg opt_block_arg
				{
					$$ = new Arguments(null, null, $2, $3, @$);
				}
			| block_arg
				{
					$$ = new Arguments(null, null, null, $1, @$);
				}
			| command
				{
					_tokenizer.yywarn("parenthesize argument(s) for future version");				
					$$ = new Arguments($1);
				}
			;

closed_args	: arg ',' args opt_block_arg
				{
				    $3.Insert(0, $1);
					$$ = new Arguments($3, null, null, $4, @$);
				}
			| arg ',' block_arg
				{
					$$ = new Arguments(Utils.Collections.MakeList<Expression>($1), null, null, $3, @$);
				}
			| arg ',' tSTAR arg opt_block_arg
				{
					$$ = new Arguments(Utils.Collections.MakeList<Expression>($1), null, $4, $5, @$);
				}
			| arg ',' args ',' tSTAR arg opt_block_arg
				{
				    $3.Insert(0, $1);
					$$ = new Arguments($3, null, $6, $7, @$);
				}
			| maplets opt_block_arg
				{
					$$ = new Arguments(null, $1, null, $2, @$);
				}
			| maplets ',' tSTAR arg opt_block_arg
				{
					$$ = new Arguments(null, $1, $4, $5, @$);
				}
			| arg ',' maplets opt_block_arg
				{
					$$ = new Arguments(Utils.Collections.MakeList<Expression>($1), $3, null, $4, @$);
				}
			| arg ',' args ',' maplets opt_block_arg
				{
				    $3.Insert(0, $1);
					$$ = new Arguments($3, $5, null, $6, @$);
				}
			| arg ',' maplets ',' tSTAR arg opt_block_arg
				{
					$$ = new Arguments(Utils.Collections.MakeList<Expression>($1), $3, $6, $7, @$);
				}
			| arg ',' args ',' maplets ',' tSTAR arg opt_block_arg
				{
				    $3.Insert(0, $1);
					$$ = new Arguments($3, $5, $8, $9, @$);
				}
			| tSTAR arg opt_block_arg
				{
					$$ = new Arguments(null, null, $2, $3, @$);
				}
			| block_arg
				{
					$$ = new Arguments(null, null, null, $1, @$);
				}
			;

command_args:
        {
            _tokenizer.CMDARG_PUSH(1);
        }
      command_args_content
        {
            _tokenizer.CMDARG_POP();
            $$ = $2;
        }
;

command_args_content: 
      open_args
    | tLPAREN_ARG
        {
            _tokenizer.lex_state = LexicalState.EXPR_ENDARG;
        }
      ')'
        {
            _tokenizer.yywarn("don't put space before argument parentheses");	
            $$ = null;
        }
    | tLPAREN_ARG closed_args
        {
            _tokenizer.lex_state = LexicalState.EXPR_ENDARG;
        }
      ')'
        {
            _tokenizer.yywarn("don't put space before argument parentheses");	
            $$ = $2;
        }
;

block_arg	: tAMPER arg
				{
					$$ = new BlockReference($2, @$);
				}
			;

opt_block_arg	: ',' block_arg
					{
						$$ = $2;
					}
				| /* empty */
					{
						$$ = null;
					}
				;

args: arg
		{
			$$ = Utils.Collections.MakeList<Expression>($1);
		}
	| args ',' arg
		{
			$1.Add($3);
			$$ = $1;
		}
	;

primary	: numeric_literal
		| symbol
			{
				$$ = new Literal($1, @$);
			}
		| symbolic_expression
		| string_concatenation
		    {
		        $$ = new StringConstructor($1, @1);
		    }
		| shell_string
		| regexp
		| words
		| verbatim_words
		| var_ref
		| backref
		| tFID
			{
				$$ = new MethodCall(null, $1, new Arguments(@1), @$);
			}
		| primary tSEPARATING_DOUBLE_COLON tCONSTANT
			{
				$$ = new ConstantVariable(CurrentScope, $1, $3, @$);
			}
		| tLEADING_DOUBLE_COLON tCONSTANT
			{
				$$ = new ConstantVariable(CurrentScope, null, $2, @$);
			}
		| primary '[' array_key ']'
			{
				$$ = new ArrayItemAccess($1, $3, @$);
			}
		| tLBRACK array_key ']'
			{
				$$ = new ArrayConstructor($2, @$);
			}
		| tLBRACE '}'
			{
				$$ = new HashConstructor(null, null, @$);
			}
		| tLBRACE maplets trailer '}'
			{
				$$ = new HashConstructor($2, null, @$);
			}
		| tLBRACE args trailer '}'
			{
				$$ = new HashConstructor(null, CheckHashExpressions($2, @3), @$);
			}						
		| kYIELD '(' open_args ')'
			{
				$$ = new YieldCall($3, @$);
			}
		| kYIELD '(' ')'
			{
				$$ = new YieldCall(new Arguments(@1), @$);
			}
		| kYIELD
			{
				$$ = new YieldCall(new Arguments(@1), @1);
			}
		| kDEFINED opt_nl '(' 
			{
				// Empty placeholder
			}
		  expr ')'
			{
				$$ = new AstNodeDescriptionExpression($5, @$);
			}
		| operation brace_block
			{
				$$ = new MethodCall(null, $1, new Arguments(null, null, null, $2, @1), @$);
			}
		| method_call
		| method_call brace_block
			{	
				$$ = $1;	
				$1.Arguments.Block = $2;
			}
		| kIF expr then compstmt if_tail kEND
			{
				$$ = new IfExpression($2, false, $4, $5, @$);
			}
		| kUNLESS expr then compstmt else_opt kEND
			{
				$$ = new IfExpression($2, true, $4, MakeListOpt($5), @$);
			}
		| kWHILE
			{
				_tokenizer.COND_PUSH(1);
			}
		  expr do
			{
				_tokenizer.COND_POP();
			}
		  compstmt kEND
			{
				$$ = new WhileLoopExpression(CurrentScope, $3, true, $6, @$);
			}
		| kUNTIL
			{
				_tokenizer.COND_PUSH(1);
			}
		  expr do
			{
				_tokenizer.COND_POP();
			}
		  compstmt kEND
			{
				$$ = new WhileLoopExpression(CurrentScope, $3, false, $6, @$);
			}
		| case_expression
		| kFOR 
			{
				// the block_parameters should be evaluated in the scope of the block
				//_forLoopBodyScope = EnterNewScope(ScopeKind.Block);
			}
		  block_parameters kIN
			{
				//LeaveScope();
				_tokenizer.COND_PUSH(1);
			}
		  expr do
			{
				_tokenizer.COND_POP();
				//EnterScope(_forLoopBodyScope);
			}
		  compstmt kEND
			{
			    $$ = new ForLoopExpression($3, $6, $9, @$);
			    //LeaveScope();
			}
		| block_expression
		| declaration_expression
;
		
block_expression:		
      tLPAREN_ARG expr 
	    {
		    _tokenizer.lex_state = LexicalState.EXPR_ENDARG;
	    } 
      opt_nl ')'
	    {
            _tokenizer.yywarn("(...) interpreted as grouped expression");			
		    $$ = new BlockExpression(new Body(Utils.Collections.MakeList<Statement>(new ExpressionStatement($2)), null, null, null, @$), @$);
	    }
    | tLPAREN compstmt ')'
	    {
		    $$ = new BlockExpression(new Body($2, null, null, null, @$), @$);
	    }
    | kBEGIN body kEND
	    {
		    $$ = new BlockExpression($2, @$);
	    }
;

declaration_expression:
      kCLASS qualified_module_name superclass
        {                
            EnterNewScope(ScopeKind.Module);
        }
      body kEND
        {
            $$ = new ClassDeclaration(CurrentScope, $2, $3, $5, @$);
            LeaveScope();
        }
    | kCLASS tLSHFT expr
        {
            $<Integer>$ = in_def;
            in_def = 0;
        }
      term
        {
            $<Integer>$ = in_single;
            in_single = 0;
            EnterNewScope(ScopeKind.Module);
        }
      body kEND
        {
            in_def = $<Integer>4;
            in_single = $<Integer>6;

            LexicalScope classScope = LeaveScope();
            LexicalScope outerScope = CurrentScope;

            ConstantVariable name = new ConstantVariable(outerScope, ID.intern("singleton"), @3);
            $$ = new SingletonDeclaration(classScope, name, $3, $7, @$);
        }
    | kMODULE qualified_module_name
        {
            EnterNewScope(ScopeKind.Module);
        }
      body kEND
        {
            $$ = new ModuleDeclaration(CurrentScope, $2, $4, @$);
            LeaveScope();
        }
    | kDEF fname
        {
            in_def++;
            EnterNewScope(ScopeKind.Method);
        }
      parameters_declaration body kEND
        {
            in_def--;
            $$ = new MethodDeclaration(CurrentScope, null, $2, $4, $5, @$);
            LeaveScope();
        }
    | kDEF singleton dot_or_colon
        {
            _tokenizer.lex_state = LexicalState.EXPR_FNAME;
        }
      fname
        {
            in_single++;
            _tokenizer.lex_state = LexicalState.EXPR_END;
            EnterNewScope(ScopeKind.Method);
        }
      parameters_declaration body kEND
        {
            in_single--;
            $$ = new MethodDeclaration(CurrentScope, $2, $5, $7, $8, @$);
            LeaveScope();
        }
;

body: 
	  compstmt rescue_clauses_opt else_opt ensure_opt
		{
			ElseIfClause elseIf = $3;
			Debug.Assert(elseIf == null || elseIf.Condition == null);
			
			if (elseIf != null && $2 == null) { 
			  _errorSink.Add(_sourceUnit, "else without rescue is useless", @3, -1, Severity.Warning);
			}
			
			$$ = new Body($1, $2, (elseIf != null) ? elseIf.Statements : null, $4, @$);
		}
;

case_expression:
      kCASE expr opt_terms when_clauses else_opt kEND
		{
			$$ = new CaseExpression($2, $4, $5, @$);
		}
	| kCASE opt_terms when_clauses else_opt kEND
		{
			$$ = new CaseExpression(null, $3, $4, @$);
		}
	| kCASE expr opt_terms else_opt kEND
		{
			$$ = new CaseExpression($2, null, $4, @$);
		}
	| kCASE opt_terms else_opt kEND
		{
			$$ = new CaseExpression(null, null, $3, @$);
		}
;

then	: term
		| ':'
		| kTHEN
		| term kTHEN
		;

do		: term
		| ':'
		| kDO_COND
		;

if_tail	: else_opt
            {
                $$ = MakeListOpt($1);
            }
		| kELSIF expr then compstmt if_tail
			{
				$5.Add(new ElseIfClause($2, $4, @$));
				$$ = $5;
			}
		;

else_opt: /* empty */
            {
                $$ = null;
            }
		| kELSE compstmt
			{
				$$ = new ElseIfClause(null, $2, @$);
			}
		;

block_parameters:
	  lhs 
		{ 
			$$ = new CompoundLeftValue(Utils.Collections.MakeList<LeftValue>($1), null, @1); 
		}
	| compound_lhs 
		{ 
			$$ = $1; 
		}
;

block_parameters_opt:
	  /* empty */
		{
			$$ = CompoundLeftValue.Null;
		}
	| '|' /* empty */ '|' 
		{
			$$ = CompoundLeftValue.Null;
		}
	| tOROP
		{
			$$ = CompoundLeftValue.Null;
		}
	| '|' block_parameters '|'
		{
			$$ = $2;
		}
;

do_block: kDO_BLOCK
			{
				EnterNewScope(ScopeKind.Block);
			}
		  block_parameters_opt 
			{
				// Empty placeholder
			}
		  compstmt kEND
			{
				$$ = new BlockDefinition(CurrentScope, $3, $5, @$);
				LeaveScope();
			}
		;

block_call  : command do_block
				{		    				
					$1.Arguments.Block = $2;
					$$ = $1;
				}
			| block_call '.' operation2 opt_paren_args
				{
					$$ = new MethodCall($1, $3, $4, @$);
				}
			| block_call tSEPARATING_DOUBLE_COLON operation2 opt_paren_args
				{
					$$ = new MethodCall($1, $3, $4, @$);
				}
			;

method_call	: operation	paren_args
				{
					$$ = new MethodCall(null, $1, $2, @$);
				}
			| primary '.' operation2 opt_paren_args
				{
					$$ = new MethodCall($1, $3, $4, @$);
				}
			| primary tSEPARATING_DOUBLE_COLON operation2 paren_args
				{
					$$ = new MethodCall($1, $3, $4, @$);
				}
			| primary tSEPARATING_DOUBLE_COLON operation3
				{
					$$ = new MethodCall($1, $3, new Arguments(@3), @$);
				}
			| kSUPER paren_args
				{
					$$ = new SuperCall(CurrentScope, $2, @$);
				}
			| kSUPER
				{
					$$ = new SuperCall(CurrentScope, null, @$);
				}
			;

brace_block	: '{'
				{
					EnterNewScope(ScopeKind.Block);
				}
			  block_parameters_opt 
				{
					// Empty placeholder				
				}
			  compstmt '}'
				{
                    $$ = new BlockDefinition(CurrentScope, $3, $5, @$);
					LeaveScope();
				}
			| kDO
				{
					EnterNewScope(ScopeKind.Block);	
				}
			  block_parameters_opt 
				{
					// Empty placeholder
				}
			  compstmt kEND
				{
			        $$ = new BlockDefinition(CurrentScope, $3, $5, @$);
					LeaveScope();
				}
			;

when_clauses : when_clause 
                 {
                     $$ = Utils.Collections.MakeList<WhenClause>($1); 
                 }
             | when_clauses when_clause 
                 {
                     $1.Add($2);
                     $$ = $1;
                 }
             ;

when_clause : kWHEN when_args then compstmt
				{
				    $$ = new WhenClause($2, $4, @$);
				}
			;
			
when_args	: args
				{
					$$ = new Arguments($1, null, null, null, @$);
				}
			| args ',' tSTAR arg
				{
					$$ = new Arguments($1, null, $4, null, @$);
				}
			| tSTAR arg
				{
					$$ = new Arguments(null, null, $2, null, @$);
				}
			;

rescue_clauses_opt : /* empty */
                   | rescue_clauses
                   ;

rescue_clauses : rescue_clause 
                   {
                       $$ = Utils.Collections.MakeList<RescueClause>($1);
                   }
               | rescue_clauses rescue_clause 
                   {
                       $1.Add($2);
                       $$ = $1;
                   }
                 ;

rescue_clause: 
      kRESCUE exc_list exc_var then compstmt
        {
            $$ = new RescueClause($2, $3, $5, @$);		
        }
;

exc_list: arg
			{
				$$ = Utils.Collections.MakeList<Expression>($1);
			}
		| mrhs
		    {
		        $$ = $1.Args;
		    }
		| /* empty */
		;

exc_var	: tASSOC lhs
			{
				$$ = $2;
			}
		| /* empty */
		;

ensure_opt	: /* empty */
            | kENSURE compstmt
				{
					$$ = $2;
				}
			;
		
string_concatenation : string
                         {
                             $$ = Utils.Collections.MakeList<Expression>($1);
                         }
		             | string_concatenation string
			             {
				             $1.Add($2);
				             $$ = $1;
			             }
		             ;

string	: tSTRING_BEG string_contents tSTRING_END
			{
				$$ = new StringConstructor($2, @$);
			}
		;

shell_string : tSHELL_STRING_BEGIN string_contents tSTRING_END
			     {
				     $$ = new ShellExpression($2, @$);
		         }
		     ;

regexp	: tREGEXP_BEG string_contents tREGEXP_END
			{
				$$ = new RegularExpression($2, $3, @$);
			}
		;

symbolic_expression : tSYMBEG string_contents tSTRING_END
		                {
			                // TODO: SymbolicExpression $$ = new SYMBOL($2, @$);
		                }
	                ;


words	: tWORDS_BEG ' ' tSTRING_END
			{
				$$ = new ArrayConstructor(null, @$);
			}
		| tWORDS_BEG word_list tSTRING_END
			{
				$$ = new ArrayConstructor(new Arguments($2, null, null, null, @2), @$);
			}
		;

word_list	: /* empty */
				{
					$$ = new List<Expression>();
				}
			| word_list word ' '
				{
					$1.Add(new StringConstructor($2, @2));
					$$ = $1;
				}
			;

word	: string_content
            {
                $$ = Utils.Collections.MakeList<Expression>($1);
            }
		| word string_content
			{
				$1.Add($2);
				$$ = $1;
			}
		;

verbatim_words : tVERBATIM_WORDS_BEGIN ' ' tSTRING_END
			       {
				       $$ = new ArrayConstructor(null, @$);
			       }
		       | tVERBATIM_WORDS_BEGIN verbatim_word_list tSTRING_END
			       {
			           $$ = new ArrayConstructor(new Arguments($2, null, null, null, @2), @$);
			       }
		       ;

verbatim_word_list : /* empty */
			           {
					      $$ = new List<Expression>();
                       }
                   | verbatim_word_list tSTRING_CONTENT ' '
                       {
                          $1.Add(new Literal($2, @2));
                          $$ = $1;
                       }
                   ;

string_contents	: /* empty */
					{
						$$ = new List<Expression>();
					}
				| string_contents string_content
					{
						$1.Add($2);
						$$ = $1;
					}
				;


string_content	: tSTRING_CONTENT
					{
						$$ = new Literal($1, @$);
					}
				| tSTRING_EMBEDDED_VARIABLE_BEGIN
					{
						$<term>$ = _tokenizer.lex_strterm;
						_tokenizer.lex_strterm = null;
						_tokenizer.lex_state = LexicalState.EXPR_BEG;
					}
				  string_embedded_variable
					{
						_tokenizer.lex_strterm = $<term>2;
						$$ = $3;
					}
				| tSTRING_EMBEDDED_CODE_BEGIN
					{
						$<term>$ = _tokenizer.lex_strterm;
						_tokenizer.lex_strterm = null;
						_tokenizer.lex_state = LexicalState.EXPR_BEG;
						_tokenizer.COND_PUSH(0);
						_tokenizer.CMDARG_PUSH(0);
					}
				  compstmt '}'
					{
						_tokenizer.lex_strterm = $<term>2;
						_tokenizer.COND_LEXPOP();
						_tokenizer.CMDARG_LEXPOP();
						$$ = new BlockExpression(new Body($3, null, null, null, @3), @3);
					}
				;

string_embedded_variable : tGVAR { $$ = new GlobalVariable($1, @$); }
                         | tIVAR { $$ = new InstanceVariable($1, @$); }
                         | tCVAR { $$ = new ClassVariable(CurrentScope, $1, @$); }
                         | backref { $$ = $1; }
                         ;

symbol	: tSYMBEG sym
			{
				_tokenizer.lex_state = LexicalState.EXPR_END;
				$$ = $2;
			}
		;

sym	: fname
	| tIVAR
	| tGVAR
	| tCVAR
	;

numeric_literal	: tINTEGER
                    {
                        $$ = new Literal($1, @$);
                    }
		        | tBIG_INTEGER
                    {
                        $$ = new Literal($1, @$);
                    }
		        | tFLOAT
                    {
                        $$ = new Literal($1, @$);
                    }
		        | tUMINUS_NUM tINTEGER                      %prec tLOWEST
                    {
                        // TODO: check overflow
                        $$ = new Literal(-$2, @$);
                    }
		        | tUMINUS_NUM tBIG_INTEGER                  %prec tLOWEST
			        {
                        $$ = new Literal(-$2, @$);
			        }
		        | tUMINUS_NUM tFLOAT                        %prec tLOWEST
			        {
				        $$ = new Literal(-$2, @$);
			        }
		        ;

variable: tIDENTIFIER { $$ = $1; }
		| tIVAR
		| tGVAR
		| tCONSTANT
		| tCVAR
		| kNIL		{ $$ = Symbols.Nil; }
		| kSELF		{ $$ = Symbols.Self; }
		| kTRUE		{ $$ = Symbols.True; }
		| kFALSE	{ $$ = Symbols.False; }
		| k__FILE__	{ $$ = Symbols.File; }
		| k__LINE__	{ $$ = Symbols.Line; }
		;

var_ref	: variable
			{
				$$ = gettable($1, @$);
			}
		;
		
var_lhs	: variable
			{
				$$ = assignable($1, @$);
			}
		;

backref	: tNTH_REF { $$ = new NTH_REF($1, @1); } // TODO
		| tBACK_REF { $$ = new BACK_REF($1, @1); } // TODO
		;

superclass	: term
				{
					$$ = null;
				}
			| '<'
				{
					_tokenizer.lex_state = LexicalState.EXPR_BEG;
				}
			  expr term
				{
					$$ = $3;
				}
			| error term
				{
					yyerrok();
					 $$ = null;
				}
			;

parameters_declaration : '(' parameters opt_nl ')'
				           {
					           $$ = $2;
					           _tokenizer.lex_state = LexicalState.EXPR_BEG;
				           }
			           | parameters term
				           {
					           $$ = $1;
				           }
			           ;

parameters  : parameter_list ',' default_parameter_list ',' array_parameter block_parameter_opt
			{
				$$ = new Parameters($1, $3, $5, $6, @$);
			}
		    | parameter_list ',' default_parameter_list block_parameter_opt
			    {
				    $$ = new Parameters($1, $3, null, $4, @$);
			    }
		    | parameter_list ',' array_parameter block_parameter_opt
			    {
				    $$ = new Parameters($1, null, $3, $4, @$);
			    }
		    | parameter_list block_parameter_opt
			    {
				    $$ = new Parameters($1, null, null, $2, @$);
			    }
		    | default_parameter_list ',' array_parameter block_parameter_opt
			    {
				    $$ = new Parameters(null, $1, $3, $4, @$);
			    }
		    | default_parameter_list block_parameter_opt
			    {
				    $$ = new Parameters(null, $1, null, $2, @$);
			    }
		    | array_parameter block_parameter_opt
			    {
				    $$ = new Parameters(null, null, $1, $2, @$);
			    }
		    | block_parameter
			    {
				    $$ = new Parameters(null, null, null, $1, @$);
			    }
		    | /* empty */ 
			    {
				    $$ = new Parameters(null, null, null, null, @$);
			    }
		    ;

parameter	: tCONSTANT
				{	
					_tokenizer.SoftError("formal argument cannot be a constant");
					$$ = DefineParameter(GenerateErrorConstantName(), @$);
				}
			| tIVAR
				{
					_tokenizer.SoftError("formal argument cannot be an instance variable");				
					$$ = DefineParameter(GenerateErrorConstantName(), @$);
				}
            | tGVAR
				{
                    _tokenizer.SoftError("formal argument cannot be a global variable");
                    $$ = DefineParameter(GenerateErrorConstantName(), @$);
				}
            | tCVAR
				{
                    _tokenizer.SoftError("formal argument cannot be a class variable");
                    $$ = DefineParameter(GenerateErrorConstantName(), @$);
				}				
			| tIDENTIFIER
				{		   
					$$ = DefineParameter($1, @$);
				}
			;

parameter_list : parameter
                 {
				    $$ = Utils.Collections.MakeList<LocalVariable>($1);
			     }
		       | parameter_list ',' parameter
			     {
			        $1.Add($3);
				    $$ = $1;
			     }
		       ;

default_parameter : parameter '=' arg
			        {		
				        $$ = new SimpleAssignmentExpression($1, $3, SymbolId.Empty, @$);
			        }
		        ;

default_parameter_list : default_parameter
			              {
				              $$ = Utils.Collections.MakeList<SimpleAssignmentExpression>($1);
			              }
		               | default_parameter_list ',' default_parameter
			              {
			                  $1.Add($3);
				              $$ = $1;
			              }
		               ; 

array_parameter_mark : '*'
                     | tSTAR
                     ;

array_parameter	: array_parameter_mark parameter
				    {	
					    $$ = $2;
				    }
			    | array_parameter_mark
				    {
					    $$ = DefineParameter(Symbols.RestArgsLocal, @1);
				    }
			    ;

block_parameter_mark : '&'
			         | tAMPER
			         ;

block_parameter : block_parameter_mark parameter
				    {
					    $$ = $2;
				    }
			    ;

block_parameter_opt	: /* empty */
                    | ',' block_parameter
					    {
						    $$ = $2;
					    }
				    ;

singleton	: var_ref
			| '('
				{
					_tokenizer.lex_state = LexicalState.EXPR_BEG;
				}
			  expr opt_nl ')'
				{						
					$$ = $3;
				}
			;

maplets	: maplet 
            {
                $$ = Utils.Collections.MakeList<Maplet>($1);
            }
		| maplets ',' maplet
			{
				$1.Add($3);
				$$ = $1;
			}
		;

maplet	: arg tASSOC arg
			{
				$$ = new Maplet($1, $3, @$);
			}
		;

operation	: tIDENTIFIER
			| tCONSTANT
			| tFID
			;

operation2	: tIDENTIFIER
			| tCONSTANT
			| tFID
			| op
			;

operation3	: tIDENTIFIER
			| tFID
			| op
			;

dot_or_colon: '.'
			| tSEPARATING_DOUBLE_COLON
			;

opt_terms	: /* empty */
			| terms
			;

opt_nl	: /* empty */
		| '\n'
		;

trailer	: /* empty */
		| '\n'
		| ','
		;

term	: ';'		{ yyerrok(); }
		| '\n'
		;

terms	: term
		| terms ';'	{ yyerrok(); }
		;
	
%%