/* ****************************************************************************
 *
 * Copyright (c) Microsoft Corporation. 
 *
 * This source code is subject to terms and conditions of the Microsoft Permissive License. A 
 * copy of the license can be found in the License.html file at the root of this distribution. If 
 * you cannot locate the  Microsoft Permissive License, please send an email to 
 * ironruby@microsoft.com. By using this source code in any fashion, you are agreeing to be bound 
 * by the terms of the Microsoft Permissive License.
 *
 * You must not remove this notice, or any other, from this software.
 *
 *
 * ***************************************************************************/

using System;
using System.Collections.Generic;
using System.Text;


namespace Ruby.Compiler
{
    using AST;
    using Microsoft.Scripting;

    public abstract class Terminator : Node
    {
        public Terminator(SourceSpan location)
            : base(location)
        {
        }
    }


    public enum string_type
    {
        str_minusone = -1,
        str_squote = (0),
        str_dquote = (0x02),
        str_xquote = (0x02),
        str_regexp = (0x04 | 0x01 | 0x02),
        str_sword = (0x08),
        str_dword = (0x08 | 0x02),
        str_ssym = (0x10),
        str_dsym = (0x10 | 0x02),
    };

    public class StringTerminator : Terminator
    {
        public int func;
        public int term;
        public int paren;
        public int name;
        public object nest;

        public StringTerminator(int func, int term, int paren, int name, SourceSpan location): base(location)
        {
            this.func = func;
            this.term = term;
            this.paren = paren;
            this.name = name;
            this.nest = 0;
        }
    }

    public class HEREDOC : Terminator
    {
        public string label;
        public int resume_position;
        public String resume_line;

        public HEREDOC(string label, int resume_position, String resume_line, SourceSpan location)
            : base(location)
        {
            this.label = label;
            this.resume_position = resume_position;
            this.resume_line = resume_line;
        }
    }
}
