/* ****************************************************************************
 *
 * Copyright (c) Microsoft Corporation. 
 *
 * This source code is subject to terms and conditions of the Microsoft Permissive License. A 
 * copy of the license can be found in the License.html file at the root of this distribution. If 
 * you cannot locate the  Microsoft Permissive License, please send an email to 
 * ironruby@microsoft.com. By using this source code in any fashion, you are agreeing to be bound 
 * by the terms of the Microsoft Permissive License.
 *
 * You must not remove this notice, or any other, from this software.
 *
 *
 * ***************************************************************************/

using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;

using Ruby.Compiler.AST;
using Microsoft.Scripting;
using Microsoft.Scripting.Math;

namespace Ruby.Compiler {

    public partial struct TokenValue {
        public Node node;
        public object obj;
        public Terminator term;

        public int Integer;
        public double Double;

        public SymbolId SymbolId { get { return new SymbolId(Integer); } set { Integer = value.Id; } }
        public Statement Statement { get { return (Statement)node; } set { node = value; } }
        public Expression Expression { get { return (Expression)node; } set { node = value; } }
        public CallExpression CallExpression { get { return (CallExpression)node; } set { node = value; } }
        public ElseIfClause ElseIfClause { get { return (ElseIfClause)node; } set { node = value; } }
        public RescueClause RescueClause { get { return (RescueClause)node; } set { node = value; } }
        public WhenClause WhenClause { get { return (WhenClause)node; } set { node = value; } }
        public Arguments Arguments { get { return (Arguments)node; } set { node = value; } }
        public BlockReference BlockReference { get { return (BlockReference)node; } set { node = value; } }
        public BlockDefinition BlockDefinition { get { return (BlockDefinition)node; } set { node = value; } }
        public ConstantVariable ConstantVariable { get { return (ConstantVariable)node; } set { node = value; } }
        public Maplet Maplet { get { return (Maplet)node; } set { node = value; } }
        public BigInteger/*!*/ BigInteger { get { return (BigInteger)obj; } set { obj = value; } }
        public String/*!*/ String { get { return (String)obj; } set { obj = value; } }
        public Parameters Parameters { get { return (Parameters)node; } set { node = value; } }
        public LocalVariable LocalVariable { get { return (LocalVariable)node; } set { node = value; } }
        public SimpleAssignmentExpression SimpleAssignmentExpression { get { return (SimpleAssignmentExpression)node; } set { node = value; } }
        public LeftValue LeftValue { get { return (LeftValue)node; } set { node = value; } }
        public CompoundLeftValue CompoundLeftValue { get { return (CompoundLeftValue)node; } set { node = value; } }
        public Body Body { get { return (Body)node; } set { node = value; } }

        public RightValues RightValues { get { return (RightValues)obj; } set { obj = value; } }

        public List<LeftValue> LeftValues { get { return (List<LeftValue>)obj; } set { obj = value; } }
        public List<UndefinedItem>/*!*/ UndefinedItems { get { return (List<UndefinedItem>)obj; } set { obj = value; } }
        public List<ElseIfClause>/*!*/ ElseIfClauses { get { return (List<ElseIfClause>)obj; } set { obj = value; } }
        public List<WhenClause>/*!*/ WhenClauses { get { return (List<WhenClause>)obj; } set { obj = value; } }
        public List<LeftValue>/*!*/ LeftValueList { get { return (List<LeftValue>)obj; } set { obj = value; } }
        public List<Expression>/*!*/ Expressions { get { return (List<Expression>)obj; } set { obj = value; } }
        public List<Statement>/*!*/ Statements { get { return (List<Statement>)obj; } set { obj = value; } }
        public List<RescueClause> RescueClauses { get { return (List<RescueClause>)obj; } set { obj = value; } }
        public List<Maplet> Maplets { get { return (List<Maplet>)obj; } set { obj = value; } }
        public List<LocalVariable>/*!*/ LocalVariables { get { return (List<LocalVariable>)obj; } set { obj = value; } }
        public List<SimpleAssignmentExpression>/*!*/ SimpleAssignmentExpressions { get { return (List<SimpleAssignmentExpression>)obj; } set { obj = value; } }  
    }
}
