/* ****************************************************************************
 *
 * Copyright (c) Microsoft Corporation. 
 *
 * This source code is subject to terms and conditions of the Microsoft Permissive License. A 
 * copy of the license can be found in the License.html file at the root of this distribution. If 
 * you cannot locate the  Microsoft Permissive License, please send an email to 
 * ironruby@microsoft.com. By using this source code in any fashion, you are agreeing to be bound 
 * by the terms of the Microsoft Permissive License.
 *
 * You must not remove this notice, or any other, from this software.
 *
 *
 * ***************************************************************************/

using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Scripting;
using Microsoft.Scripting.Hosting;
using MSA = Microsoft.Scripting.Ast;
using Ruby.Compiler.AST;
using Ruby.Hosting;

namespace Ruby.Compiler {
    public sealed class RubyScriptCompiler : ScriptCompiler {

        public RubyScriptCompiler(RubyEngine/*!*/ engine) 
            : base(engine) {

        }

        public override MSA.CodeBlock ParseInteractiveCode(CompilerContext/*!*/ cc, bool allowIncomplete, out InteractiveCodeProperties props) {
            SourceUnitTree ast = new Parser().ParseInteractiveCode(cc, out props);
            return (ast != null) ? BindAndTransform(cc, ast) : null;
        }

        public override MSA.CodeBlock ParseFile(CompilerContext/*!*/ cc) {
            SourceUnitTree ast = new Parser().Parse(cc);

            return (ast != null) ? BindAndTransform(cc, ast) : null;
        }

        public override MSA.CodeBlock ParseStatementCode(CompilerContext/*!*/ cc) {
            throw new Exception("The method or operation is not implemented.");
        }

        public override MSA.CodeBlock ParseExpressionCode(CompilerContext/*!*/ cc) {
            SourceUnitTree ast = new Parser().Parse(cc);
            return (ast != null) ? BindAndTransform(cc, ast) : null;
        }

#if !SILVERLIGHT
        public override SourceUnit ParseCodeDom(System.CodeDom.CodeObject codeDom) {
            throw new Exception("The method or operation is not implemented.");
        }
#endif

        private MSA.CodeBlock BindAndTransform(CompilerContext/*!*/ cc, SourceUnitTree/*!*/ ast) {
            AstGenerator astGen = new AstGenerator();

            return (MSA.CodeBlock)ast.Transform(astGen);
        }
    }
}
