/* ****************************************************************************
 *
 * Copyright (c) Microsoft Corporation. 
 *
 * This source code is subject to terms and conditions of the Microsoft Permissive License. A 
 * copy of the license can be found in the License.html file at the root of this distribution. If 
 * you cannot locate the  Microsoft Permissive License, please send an email to 
 * ironruby@microsoft.com. By using this source code in any fashion, you are agreeing to be bound 
 * by the terms of the Microsoft Permissive License.
 *
 * You must not remove this notice, or any other, from this software.
 *
 *
 * ***************************************************************************/

using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

using Microsoft.Scripting;
using Microsoft.Scripting.Hosting;
using Microsoft.Scripting.Shell;
using Microsoft.Scripting.Generation;


namespace Ruby.Hosting {
   
    /// <summary>
    /// A simple Ruby command-line should mimic the standard irb.exe
    /// </summary>
    public class RubyCommandLine : CommandLine {

        private new RubyConsoleOptions Options { get { return (RubyConsoleOptions)base.Options; } }
        private new RubyEngine Engine { get { return (RubyEngine)base.Engine; } }
        
        public RubyCommandLine() {
        }

        protected override string Logo {
            get {
                return String.Format("{0}{1}{2}{1}",
                    Engine.VersionString, Environment.NewLine, Engine.Copyright);
            }
        }

        protected override int RunCommand(string command) {
            throw new Exception("The method or operation is not implemented.");
        }

#if !SILVERLIGHT
        protected override int RunFile(string fileName) {
            int result = 1;
            if (!File.Exists(fileName)) {
                Console.WriteLine(string.Format("File {0} does not exist", fileName), Style.Error);
                return 1;
            }
            try {
                ScriptDomainManager.Options.AssemblyGenAttributes |= AssemblyGenAttributes.EmitDebugInfo;

                SourceFileUnit file_unit = new SourceFileUnit(Engine, fileName, Path.GetFileNameWithoutExtension(fileName), Encoding.Default);
                ScriptModule script_module = file_unit.CompileToModule(new Ruby.Compiler.RubyCompilerOptions(), Engine.GetDefaultErrorSink());

                script_module.Execute(); 
                //this.Engine.CreateOptimizedModule(fileName, Path.GetFileNameWithoutExtension(fileName)).Execute();
                result = 0;
            } catch (Exception e) {
                Console.WriteLine(this.Engine.FormatException(e), Style.Error);
            }
            return result;
        }

#endif
    }
}
