/* ****************************************************************************
 *
 * Copyright (c) Microsoft Corporation. 
 *
 * This source code is subject to terms and conditions of the Microsoft Permissive License. A 
 * copy of the license can be found in the License.html file at the root of this distribution. If 
 * you cannot locate the  Microsoft Permissive License, please send an email to 
 * ironruby@microsoft.com. By using this source code in any fashion, you are agreeing to be bound 
 * by the terms of the Microsoft Permissive License.
 *
 * You must not remove this notice, or any other, from this software.
 *
 *
 * ***************************************************************************/

using System;
using System.Text;
using System.IO;
using System.Diagnostics;
using System.Collections.Generic;

using Microsoft.Scripting;
using Microsoft.Scripting.Generation;
using Microsoft.Scripting.Hosting;
using Microsoft.Scripting.Shell;

namespace Ruby.Hosting {

    public sealed class RubyOptionsParser : OptionsParser {

        private RubyConsoleOptions _consoleOptions;
        private RubyEngineOptions _engineOptions;

        public override ConsoleOptions ConsoleOptions { get { return _consoleOptions; } set { _consoleOptions = (RubyConsoleOptions)value; } } 
        public override EngineOptions EngineOptions { get { return _engineOptions; } set { _engineOptions = (RubyEngineOptions)value; } }
        
        public RubyOptionsParser() {
        }

        public override ConsoleOptions GetDefaultConsoleOptions() {
            return new RubyConsoleOptions();
        }

        public override EngineOptions GetDefaultEngineOptions() {
            return new RubyEngineOptions();
        }

        public override void Parse(string[] args) {
            if (_consoleOptions == null) _consoleOptions = new RubyConsoleOptions();
            if (_engineOptions == null) _engineOptions = new RubyEngineOptions();

            base.Parse(args);
        }

        /// <exception cref="Exception">On error.</exception>
        protected override void ParseArgument(string arg) {
            if (arg == null) throw new ArgumentNullException("arg");

            Debug.Assert(_consoleOptions != null && _engineOptions != null);

            switch (arg) {
                case "-D": GlobalOptions.EngineDebug = true; break;
                case "-X:ColorfulConsole": _consoleOptions.ColorfulConsole = true; break;
                case "-X:ILDebug": GlobalOptions.AssemblyGenAttributes |= AssemblyGenAttributes.ILDebug; break;
                case "-X:SaveAssemblies": GlobalOptions.AssemblyGenAttributes |= AssemblyGenAttributes.SaveAndReloadAssemblies; break;
                case "-X:StaticMethods": GlobalOptions.AssemblyGenAttributes |= AssemblyGenAttributes.GenerateStaticMethods; break;
                default:
                    _consoleOptions.FileName = arg;
                    break;
            }
        }

        public override void GetHelp(out string commandLine, out string[,] options, out string[,] environmentVariables, out string comments) {

            commandLine = "[options] [file|- [arguments]]";

            options = new string[,] {
                { "-opt", "dummy" } 
            };

            environmentVariables = new string[,] {
                { "-envvar", "dummy" }
            };

            comments = null;
        }
    }
}
