/* ****************************************************************************
 *
 * Copyright (c) Microsoft Corporation. 
 *
 * This source code is subject to terms and conditions of the Microsoft Permissive License. A 
 * copy of the license can be found in the License.html file at the root of this distribution. If 
 * you cannot locate the  Microsoft Permissive License, please send an email to 
 * ironruby@microsoft.com. By using this source code in any fashion, you are agreeing to be bound 
 * by the terms of the Microsoft Permissive License.
 *
 * You must not remove this notice, or any other, from this software.
 *
 *
 * ***************************************************************************/

using Ruby.Runtime;
using Microsoft.Scripting;
using System;
using System.Diagnostics;
using Ruby.Compiler;

namespace Ruby.Builtins {

    public abstract class LibraryInitializer {
        private RubyExecutionContext _context;

        protected RubyExecutionContext/*!*/ Context {
            get { return _context; }
        }

        protected LibraryInitializer() {
        }

        internal void LoadModules(RubyExecutionContext/*!*/ context) {
            Utils.Assert.NotNull(context);
            _context = context;
            LoadModules();
        }

        protected RubyClass/*!*/ RegisterClass(string/*!*/ name, Type/*!*/ type, Action<RubyModule>/*!*/ initializer, RubyClass/*!*/ super,
            RubyModule[]/*!*/ mixins, Delegate[] factories) {

            Utils.Assert.NotNull(name, type, initializer, super);
            Debug.Assert(_context.ObjectClass != null);
            
            RubyClass result = new RubyClass(_context, SymbolTable.StringToId(name), type, initializer, super, true, _context.ShowCls);
            result.SetMixins(mixins);
            result.Factories = factories;

            _context.AddClassToCache(result);
            _context.ObjectClass.SetConstant(result.Name, result);

            return result;
        }

        protected RubyModule/*!*/ RegisterModule(string/*!*/ name, Type/*!*/ type, Action<RubyModule>/*!*/ initializer, RubyModule[]/*!*/ mixins) {
            Utils.Assert.NotNull(name, initializer);
            Debug.Assert(_context.ObjectClass != null);

            RubyModule module = new RubyModule(_context, SymbolTable.StringToId(name), initializer);
            module.SetMixins(mixins);
            _context.AddModuleToCache(module, type);
            _context.ObjectClass.SetConstant(module.Name, module);
            return module;
        }

        // TODO: dynamic library loading?
        protected RubyClass/*!*/ GetClass(Type/*!*/ type) {
            return _context.GetClass(type);
        }

        // TODO: dynamic library loading?
        protected RubyModule/*!*/ GetModule(Type/*!*/ type) {
            return _context.GetModule(type);
        }

        protected virtual void LoadModules() { 
            throw new NotImplementedException(); 
        }
    }
}
