/* ****************************************************************************
 *
 * Copyright (c) Microsoft Corporation. 
 *
 * This source code is subject to terms and conditions of the Microsoft Permissive License. A 
 * copy of the license can be found in the License.html file at the root of this distribution. If 
 * you cannot locate the  Microsoft Permissive License, please send an email to 
 * ironruby@microsoft.com. By using this source code in any fashion, you are agreeing to be bound 
 * by the terms of the Microsoft Permissive License.
 *
 * You must not remove this notice, or any other, from this software.
 *
 *
 * ***************************************************************************/

using System;
using System.Collections.Generic;
using System.Text;

namespace Ruby.Runtime {
    [Flags]
    public enum RubyMethodAttributes {
        Public = 1,
        Private = 2,
        Protected = 4,
        DefaultVisibility = Public,
        VisibilityMask = Public | Private | Protected,

        /// <summary>
        /// Method is defined in the type's instance method table.
        /// </summary>
        Instance = 8,

        /// <summary>
        /// Method is defined in the type's static method table.
        /// </summary>
        Singleton = 16,

        /// <summary>
        /// Method creates instances of the class whose class object is passed as self ("new").
        /// </summary>
        IsInstanceConstructor = 32,
        IsInstanceAllocator = 64,

        KindsMask = IsInstanceConstructor | IsInstanceAllocator,
        
        PublicInstance = Public | Instance,
        PrivateInstance = Private | Instance,
        ProtectedInstance = Protected | Instance,

        PublicSingleton = Public | Singleton,
        PrivateSingleton = Private | Singleton,
        ProtectedSingleton = Protected | Singleton,
        
        Default = PublicInstance,
    }

    public enum RubyMethodVisibility {
        Public,
        Private,
        Protected
    }
}
