/* ****************************************************************************
 *
 * Copyright (c) Microsoft Corporation. 
 *
 * This source code is subject to terms and conditions of the Microsoft Permissive License. A 
 * copy of the license can be found in the License.html file at the root of this distribution. If 
 * you cannot locate the  Microsoft Permissive License, please send an email to 
 * ironruby@microsoft.com. By using this source code in any fashion, you are agreeing to be bound 
 * by the terms of the Microsoft Permissive License.
 *
 * You must not remove this notice, or any other, from this software.
 *
 *
 * ***************************************************************************/
using System;
using Ruby.Builtins;
using Microsoft.Scripting;

namespace Ruby.Runtime {
    /// <summary>
    /// Helper class for creating the corresponding .NET exceptions from the Ruby error names
    /// </summary>
    public static class RubyExceptions {
        public static Exception/*!*/ CreateTypeError(string/*!*/ message) {
            return new InvalidOperationException(message);
        }

        public static Exception/*!*/ CreateTypeError(string/*!*/ message, Exception innerException) {
            return new InvalidOperationException(message, innerException);
        }

        public static Exception CannotConvertTypeToTargetType(object param, string targetType) {
            if (param == null)
                return new InvalidOperationException("can't convert nil into " + targetType);
            else
                return new InvalidOperationException("can't convert " + param.GetType().Name + " into " + targetType);
        }

        public static Exception CreateArgumentError(string message) {
            return new ArgumentException(message);
        }

        public static Exception/*!*/ CreateArgumentError(string/*!*/ message, Exception innerException) {
            return new ArgumentException(message, innerException);
        }

        public static Exception CreateNotImplementedError(string message) {
            return new NotImplementedError(message);
        }

        public static Exception CreateNotImplementedError(string message, Exception innerException) {
            return new NotImplementedError(message, innerException);
        }

        public static Exception CreateIndexError(string message) {
            return new IndexOutOfRangeException(message);
        }

        public static Exception CreateIndexError(string message, Exception innerException) {
            return new IndexOutOfRangeException(message, innerException);
        }

        public static Exception CreateRangeError(string message) {
            return new ArgumentOutOfRangeException(String.Empty, message);
        }

        internal static Exception/*!*/ CreateNameError(string/*!*/ message) {
            return new MemberAccessException(message);
        }

        public static Exception CreateLocalJumpError(string message) {
            return new LocalJumpError(message);
        }
    }
}