/* ****************************************************************************
 *
 * Copyright (c) Microsoft Corporation. 
 *
 * This source code is subject to terms and conditions of the Microsoft Permissive License. A 
 * copy of the license can be found in the License.html file at the root of this distribution. If 
 * you cannot locate the  Microsoft Permissive License, please send an email to 
 * ironruby@microsoft.com. By using this source code in any fashion, you are agreeing to be bound 
 * by the terms of the Microsoft Permissive License.
 *
 * You must not remove this notice, or any other, from this software.
 *
 *
 * ***************************************************************************/
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Scripting.Actions;
using Microsoft.Scripting;
using MSA = Microsoft.Scripting.Ast;
using Ast = Microsoft.Scripting.Ast.Ast;
using System.Diagnostics;

namespace Ruby.Runtime {
    public enum MethodKind {
        Regular,
        InstanceConstructor,
        InstanceAllocator,
        Block
    }

    public abstract class RubyMethodBase {
        private readonly SymbolId _name;
        private RubyMethodVisibility _visibility;
        private MethodKind _kind;

        public SymbolId Name {
            get { return _name; }
        }

        public RubyMethodVisibility Visibility {
            get { return _visibility; }
            set { _visibility = value; }
        }

        public MethodKind Kind {
            get { return _kind; }
        }

        public string NameString {
            get { return SymbolTable.IdToString(_name); }
        }

        public bool IsBlock {
            get { return _kind == MethodKind.Block; }
        }

        protected RubyMethodBase(SymbolId name, RubyMethodVisibility visibility, MethodKind kind) {
            _name = name;
            _visibility = visibility;
            _kind = kind;
	    }

        public override string/*!*/ ToString() {
            return SymbolTable.IdToString(_name);
        }

        public abstract void SetInvocationRule<T>(ActionBinder/*!*/ binder, StandardRule<T>/*!*/ result, ArgumentInfo[] argInfos,
            object[]/*!*/ args, SymbolId? methodNameArgument);

        internal static void AddTestForListArg<T>(StandardRule<T>/*!*/ rule, object/*!*/ argument, MSA.Expression/*!*/ parameter) {
            int length;
            MSA.Variable variable;
            AddTestForListArg(rule, argument, parameter, out length, out variable);
        }

        internal static void AddTestForListArg<T>(StandardRule<T>/*!*/ rule, object/*!*/ argument, MSA.Expression/*!*/ parameter, 
            out int listLength, out MSA.Variable/*!*/ listVariable) {
            Debug.Assert(argument is IList<object>);

            listLength = ((IList<object>)argument).Count;
            listVariable = rule.GetTemporary(typeof(IList<object>), "#list");

            // adds condition on list length to the test expression:
            // expr ::= expr && (list = RubyOps.AsArrayOfLength(parameter[i], length) != null)
            rule.AddTest(Ast.NotEqual(Ast.Null(),
                Ast.Assign(listVariable, Ast.Call(null, RubyOps.GetMethod("AsArrayOfLength"), parameter, Ast.Constant(listLength)))
            ));
        }
    }
}
