/* ****************************************************************************
 *
 * Copyright (c) Microsoft Corporation. 
 *
 * This source code is subject to terms and conditions of the Microsoft Permissive License. A 
 * copy of the license can be found in the License.html file at the root of this distribution. If 
 * you cannot locate the  Microsoft Permissive License, please send an email to 
 * ironruby@microsoft.com. By using this source code in any fashion, you are agreeing to be bound 
 * by the terms of the Microsoft Permissive License.
 *
 * You must not remove this notice, or any other, from this software.
 *
 *
 * ***************************************************************************/
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Scripting;
using Ruby.Builtins;
using System.Reflection;
using Ruby.Hosting;
using Ruby.Compiler;

namespace Ruby.Runtime {
    public sealed class RuntimeFlowControl {
        internal static readonly RuntimeFlowControl/*!*/ ClrMethod = new RuntimeFlowControl(false);
        
        // emitted:
        public static readonly object/*!*/ RetrySingleton = new object();
        
        // emitted:
        public bool InMethod;
        public bool InLoop;
        public bool InTryRescue;
        public bool InBlock;

        public RuntimeFlowControl(bool inMethod) {
            InMethod = inMethod;
        }

        internal static MethodInfo/*!*/ GetMethod(string/*!*/ name) {
            MethodInfo result = typeof(RuntimeFlowControl).GetMethod(name);
            Utils.Assert.NotNull(result);
            return result;
        }

        internal static ConstructorInfo/*!*/ Constructor {
            // TODO: cache
            get { return typeof(RuntimeFlowControl).GetConstructor(new Type[] { typeof(bool) }); }
        }
        
        public static bool IsRetrySingleton(object value) {
            return ReferenceEquals(value, RetrySingleton);
        }

        public static object RetryMethod(object blockParameter) {
            if (blockParameter != null) {
                return RetrySingleton;
            } else {
                throw new LocalJumpError("retry used out of rescue");
            }
        }

        public static object RetryBlock() {
            return RetrySingleton;
        }

        // TODO: not used
        //public void RequireBlockForBreak(object blockParameter) {
        //    if (blockParameter == null) {
        //        throw new LocalJumpError("invalid use of break");
        //    }
        //}

        //public void RequireBlockForNext(object blockParameter) {
        //    if (blockParameter == null) {
        //        throw new LocalJumpError("invalid use of next");
        //    }
        //}

        //public void RequireBlockForRedo(object blockParameter) {
        //    if (blockParameter == null) {
        //        throw new LocalJumpError("invalid use of redo");
        //    }
        //}
        
        public void DynamicRetry() {
            if (InTryRescue || InBlock) {
                StackUnwinder.Unwind(new StackUnwinder.Retry());
            } else {
                throw new LocalJumpError("retry used out of rescue");
            }
        }

        public void DynamicBreak(object returnValue) {
            if (InLoop || InBlock) {
                throw new StackUnwinder.BlockReturn(returnValue, BlockReturnReason.BreakOrRetry);
            } else {
                throw new LocalJumpError("invalid use of break");
            }
        }

        public void DynamicRedo() {
            if (InLoop || InBlock) {
                throw new StackUnwinder.BlockReturn(null, BlockReturnReason.Redo);
            } else {
                throw new LocalJumpError("invalid use of redo");
            }
        }

        public void DynamicNext(object returnValue) {
            if (InLoop || InBlock) {
                throw new StackUnwinder.BlockReturn(returnValue, BlockReturnReason.Next);
            } else {
                throw new LocalJumpError("invalid use of next");
            }
        }

        public void DynamicReturn(object returnValue) {
            if (InMethod) {
                StackUnwinder.Unwind(new StackUnwinder.Return(this, returnValue));
            } else {
                throw new LocalJumpError("unexpected return");
            }
        }
    }
}
