/* ****************************************************************************
 *
 * Copyright (c) Microsoft Corporation. 
 *
 * This source code is subject to terms and conditions of the Microsoft Permissive License. A 
 * copy of the license can be found in the License.html file at the root of this distribution. If 
 * you cannot locate the  Microsoft Permissive License, please send an email to 
 * ironruby@microsoft.com. By using this source code in any fashion, you are agreeing to be bound 
 * by the terms of the Microsoft Permissive License.
 *
 * You must not remove this notice, or any other, from this software.
 *
 *
 * ***************************************************************************/

using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Scripting;
using System.Reflection.Emit;
using MSA = Microsoft.Scripting.Ast;
using System.Reflection;

namespace Ruby.Runtime {
    // TODO: make internal and use filters, move to DLR
    /// <summary>
    /// This class is deliberately internal and not inherited from Exception so that it won't be caught unless one is catching all objects.
    /// </summary>
    public class StackUnwinder : Exception {
        //private static CallTarget1 _throwStub;

        /// <summary>
        /// Used for unwind from nested blocks on return.
        /// </summary>
        public sealed class Return : StackUnwinder {
            public readonly object ReturnValue;
            public readonly RuntimeFlowControl/*!*/ FlowControl;

            // TODO: cache
            internal static FieldInfo ReturnValueField { get { return typeof(Return).GetField("ReturnValue"); } }
            internal static FieldInfo FlowControlField { get { return typeof(Return).GetField("FlowControl"); } }
            
            internal Return(RuntimeFlowControl/*!*/ flowControl, object returnValue) {
                Utils.Assert.NotNull(flowControl);

                FlowControl = flowControl;
                ReturnValue = returnValue;
            }
        }

        /// <summary>
        /// Used for dynamic undind from blocks on break/next/redo.
        /// </summary>
        public sealed class BlockReturn : StackUnwinder {
            public readonly object ReturnValue;
            public readonly BlockReturnReason Reason;

            // TODO: cache
            internal static FieldInfo ReturnValueField { get { return typeof(BlockReturn).GetField("ReturnValue"); } }
            internal static FieldInfo ReasonField { get { return typeof(BlockReturn).GetField("Reason"); } }

            internal BlockReturn(object returnValue, BlockReturnReason reason) {
                ReturnValue = returnValue;
                Reason = reason;
            }
        }

        /// <summary>
        /// Used for dynamic unwind on retry.
        /// </summary>
        public sealed class Retry : StackUnwinder {
            internal Retry() {
            }
        }

        internal static void Unwind(StackUnwinder/*!*/ unwinder) {
            Utils.Assert.NotNull(unwinder);

            throw unwinder;
        }
    }
}
