#####################################################################################
#
#  Copyright (c) Microsoft Corporation. All rights reserved.
#
# This source code is subject to terms and conditions of the Microsoft Permissive License. A 
# copy of the license can be found in the License.html file at the root of this distribution. If 
# you cannot locate the  Microsoft Permissive License, please send an email to 
# ironruby@microsoft.com. By using this source code in any fashion, you are agreeing to be bound 
# by the terms of the Microsoft Permissive License.
#
# You must not remove this notice, or any other, from this software.
#
#
#####################################################################################

require '../../util/assert.rb'

a, b = 4, 5
assert_equal(a, 4)
assert_equal(b, 5)

# swap
a, b = b, a
assert_equal(a, 5)
assert_equal(b, 4)

# the values on the right side are evaluated in the order in which they apprear before any assignment
# is made to variables or attributes on the left

x = 1
a, b = x, x+=2
assert_equal(a, 1)
assert_equal(b, 3)

a, b = x+=2, x
assert_equal(a, 5)
assert_equal(b, 5)

# same variable
a, a = 4, 7
assert_equal(a, 7)

a = 10
a, a = 6, a+9
assert_equal(a, 19)


# when an assignment has more than one lvalue, the assignment expression returns an array of the rvalues


# if an assignment contains more lvalues than rvalues, the excess lvalues are set to nil
a, b, c = 12
assert_equal(a, 12)
assert_nil(b)
assert_nil(c)

# if the assignment contains more rvalues than lvalues, the extra rvalues are ignored.

a, b = 3, 4, 5
assert_equal(a, 3)
assert_equal(b, 4)

# if an assignment has just one lvalue and multiple rvalues, the rvalues are converted to an array and
# assigned to the lvalue: see Tests\Runtime\Expression\test_assignment.rb.

#
# collapse/expand arrays
#
# if the last lvalue is preceded by *, all the remaining rvalues will be collected and assigned to that
# lvalue as an array.
x = [3, 5, 7, 9]
*b = x
assert_equal(b, [[3, 5, 7, 9]])

a, *b = x
assert_equal(a, 3)
assert_equal(b, [5, 7, 9])

*b = x, 20
assert_equal(b, [[3, 5, 7, 9], 20])

a, *b = x, 20
assert_equal(a, [3, 5, 7, 9])
assert_equal(b, [20])

*b = 30, x
assert_equal(b, [30, [3, 5, 7, 9]])

a, *b = 30, x
assert_equal(a, 30)
assert_equal(b, [[3, 5, 7, 9]])

# similarly, if the last rvalue is an array, you can prefix it with *, which effectively expands it into
# its constituent values in place.
b = *x
assert_equal(b, [3, 5, 7, 9])

a, b = *x
assert_equal(a, 3)
assert_equal(b, 5)

b = 40, *x
assert_equal(b, [40, 3, 5, 7, 9])

a, b = 40, *x
assert_equal(a, 40)
assert_equal(b, 3)

## a, b = *x, 40, syntax error

# both last rvalue/lvalue have *
*b = *x
assert_equal(b, [3, 5, 7, 9])

*b = 40, *x
assert_equal(b, [40, 3, 5, 7, 9])

a, *b = *x
assert_equal(a, 3)
assert_equal(b, [5, 7, 9])

a, *b = 50, *x
assert_equal(a, 50)
assert_equal(b, [3, 5, 7, 9])

# this (having * ahead of rvalue) is not necessary if the rvalue is the only thing in the right side.
a, b = x
assert_equal(a, 3)
assert_equal(b, 5)
