#####################################################################################
#
#  Copyright (c) Microsoft Corporation. All rights reserved.
#
# This source code is subject to terms and conditions of the Microsoft Permissive License. A 
# copy of the license can be found in the License.html file at the root of this distribution. If 
# you cannot locate the  Microsoft Permissive License, please send an email to 
# ironruby@microsoft.com. By using this source code in any fashion, you are agreeing to be bound 
# by the terms of the Microsoft Permissive License.
#
# You must not remove this notice, or any other, from this software.
#
#
#####################################################################################
def log_error(message)
    puts message
    raise
end 

def assert(boolean, message)
    if boolean != true
        log_error(message)
    end 
end

def assert_unreachable()
    assert(false, "should not be reachable")
end 

def assert_should_have_thrown()
    assert(false, "should have thrown")
end 

def assert_equal(left, right)
    assert(left == right, "left: #{left}, right: #{right}")
end 

def assert_equal_with_message(left, right, message)
    if message
        assert(left == right, nil)
    else 
        assert(left == right, "left: #{left}, right: #{right}")
    end 
end 

def assert_nil(obj)
    assert_equal(obj, nil)
end 

def assert_nil_with_message(obj, message)
    assert_equal_with_message(obj, nil, message)
end 

def assert_not_nil(obj, message)
    assert(obj != nil, "should be not nil")
end 

def assert_isinstanceof(obj, kclass)
    assert_equal(obj.class, kclass)
end 

def assert_isinstanceof_with_message(obj, kclass, message)
    assert_equal_with_message(obj.class, kclass, message)
end 

def assert_raise(expected_exception)
    begin
        yield 
    rescue Exception => actual_exception_obj
        assert_isinstanceof_with_message(
                 actual_exception_obj, expected_exception, 
                "expect #{expected_exception}, but get #{actual_exception_obj.class}"
                )
    else
        assert_should_have_thrown
    end 
end

def assert_return(expected) 
    actual = yield
    assert_equal(expected, actual)
end 

#def assert_true(&b)
#    assert_return(true, &b) 
#end 
#
#def assert_false(&b)
#    assert_return(false, &b) 
#end 

def assert_true
    actual = yield
    assert_equal(true, actual)
end 

def assert_false
    actual = yield
    assert_equal(false, actual)
end 

def divide_by_zero
    1/0
end 

def empty_func
end 

def runtest(m, skiplist=[])
    m.private_methods.each do |m|
        if m[0,5] == "test_" 
            puts "running #{m}... "
            if skiplist.include? m
                puts "skipped"
            else 
                m.method(m).call 
            end
        end 
    end
end 