#####################################################################################
#
#  Copyright (c) Microsoft Corporation. All rights reserved.
#
# This source code is subject to terms and conditions of the Microsoft Permissive License. A 
# copy of the license can be found in the License.html file at the root of this distribution. If 
# you cannot locate the  Microsoft Permissive License, please send an email to 
# ironruby@microsoft.com. By using this source code in any fashion, you are agreeing to be bound 
# by the terms of the Microsoft Permissive License.
#
# You must not remove this notice, or any other, from this software.
#
#
#####################################################################################
$error_count = 0

class PositiveExpectation
  def initialize(obj)
    $obj = obj
  end

  def ==(other)
    if $obj != other
      $error_count += 1
      raise Exception.new("Equality expected for #{$obj} and #{other}")
    else
      print '.'
    end
  end
end

class NegativeExpectation
  def initialize(obj)
    $obj = obj
  end

  def ==(other)
    if $obj == other
      $error_count += 1
      raise Exception.new("Inequality expected for #{$obj} and #{other}")
    else
      print '.'
    end
  end
end

class Object
  def should
    PositiveExpectation.new(self)
  end

  def should_not
    NegativeExpectation.new(self)
  end
end

def it(name)
  print "\n  it #{name}: "
  $name = name
  yield
end

def skip(name)
  print "\n  **skipping** #{name}"
end

def describe(message)
  puts "\n\n#{message}"
  yield
end

def should_raise(expected_exception)
  begin
    yield
  rescue Exception => actual_exception
    if expected_exception.name != actual_exception.class.name
      $error_count += 1
      puts "#{$name} failed! expected #{expected_exception} but got #{actual_exception}" 
    end
  end
end

def finished
  puts "\n\nTests failed == #{$error_count}"
end
