#####################################################################################
#
#  Copyright (c) Microsoft Corporation. All rights reserved.
#
# This source code is subject to terms and conditions of the Microsoft Permissive License. A 
# copy of the license can be found in the License.html file at the root of this distribution. If 
# you cannot locate the  Microsoft Permissive License, please send an email to 
# ironruby@microsoft.com. By using this source code in any fashion, you are agreeing to be bound 
# by the terms of the Microsoft Permissive License.
#
# You must not remove this notice, or any other, from this software.
#
#
#####################################################################################
module TestPath
    def TestPath.get_environment_variable(x)
        ENV[x.upcase] or ENV[x.downcase]
    end 
    
    def TestPath.get_directory_name(dir)
        File.basename dir
    end 
    
    def TestPath.get_directory(dir)
        File.dirname dir
    end 
    
    def TestPath.probe_cruby_exe()
        [ "C:/Programs/Ruby/bin/ruby.exe", 
          "D:/ruby/bin/ruby.exe", 
          "C:/ruby/bin/ruby.exe",
          MERLIN_ROOT + "/../External/Languages/Ruby/bin/ruby.exe"
        ].each do |f|
            return f if File.exist? f
        end
    end
    
    MERLIN_ROOT     = get_environment_variable('MERLIN_ROOT')
    TEST_DIR        = MERLIN_ROOT + "/Languages/Ruby/Tests"
    ROWAN_BIN       = get_environment_variable('ROWAN_BIN')
    
    if ROWAN_BIN
        # assume we are running inside snap
        IRUBY_EXE       = ROWAN_BIN + "/rbx.exe"
        IPYTHON_EXE     = ROWAN_BIN + "/ipy.exe"
        CRUBY_EXE       = MERLIN_ROOT + "/../External/Languages/Ruby/bin/ruby.exe"
    else
        IRUBY_EXE       = MERLIN_ROOT + "/bin/debug/rbx.exe"
        IPYTHON_EXE     = MERLIN_ROOT + "/bin/debug/ipy.exe"
        THOST_EXE       = MERLIN_ROOT + "/bin/debug/thost.exe"
        CRUBY_EXE       = probe_cruby_exe()
    end
    
    TEST_TOOL_DIR   = MERLIN_ROOT + "/languages/ruby/Tests/Tools"
    TEST_TOOL_BIN_DIR = MERLIN_ROOT + "/languages/ruby/Tests/Tools/bin"
    #PARSEONLY_EXE   = TEST_TOOL_BIN_DIR + "/ParseOnly.exe"
    PARSEONLY_EXE   = IPYTHON_EXE + " " + TEST_TOOL_DIR + "/parseonly.py " 
end 

module Test
    class CRubyDriver
        def CRubyDriver.run(f, log="#{f}.c.log")
            if log.nil?
                system("#{TestPath::CRUBY_EXE} #{f}")
            else
                system("#{TestPath::CRUBY_EXE} #{f} >> #{log} 2>&1")
            end
            $?.exitstatus
        end 
    end 
    
    class IronRubyDriver
        attr_reader :mode
        @@mode_mapping = {
            1 => "-D -X:SaveAssemblies -X:StaticMethods",
            2 => "",
        }
        
        def initialize(mode=1)
            @mode = mode
        end 
        
        def run(f, log="#{f}.i.log")
            system("#{TestPath::IRUBY_EXE} #{@@mode_mapping[@mode]} #{f} >> #{log} 2>&1")
            $?.exitstatus
        end
     end 
     
     IronRubyDriver_M1 = IronRubyDriver.new(1)
     IronRubyDriver_M2 = IronRubyDriver.new(2)
     
     class SilverlightDriver
        def SilverlightDriver.run(f, log="#{f}.s.log")
            system("#{TestPath::THOST_EXE} /lang:rb /run:#{f} >> #{log} 2>&1")
            $?.exitstatus
        end
     end
end 
