/* ****************************************************************************
 *
 * Copyright (c) Microsoft Corporation. 
 *
 * This source code is subject to terms and conditions of the Microsoft Permissive License. A 
 * copy of the license can be found in the License.html file at the root of this distribution. If 
 * you cannot locate the  Microsoft Permissive License, please send an email to 
 * ironruby@microsoft.com. By using this source code in any fashion, you are agreeing to be bound 
 * by the terms of the Microsoft Permissive License.
 *
 * You must not remove this notice, or any other, from this software.
 *
 *
 * ***************************************************************************/

namespace Ruby.Builtins {
    internal sealed partial class Initializer {
        internal static void LoadModules(Ruby.Runtime.RubyExecutionContext/*!*/ context) {
            context.RegisterModule("Comparable", new System.Action<Ruby.Builtins.RubyModule>(Initializer.LoadComparable));
            context.RegisterModule("Enumerable", new System.Action<Ruby.Builtins.RubyModule>(Initializer.LoadEnumerable));
            context.RegisterModule("Math", new System.Action<Ruby.Builtins.RubyModule>(Initializer.LoadMath));
            
            context.RegisterClass("EOFError", typeof(Ruby.Builtins.EOFError), new System.Action<Ruby.Builtins.RubyModule>(LoadEOFError), TODO);
            context.RegisterClass("FloatDomainError", typeof(Ruby.Builtins.FloatDomainError), new System.Action<Ruby.Builtins.RubyModule>(LoadFloatDomainError), TODO);
            context.RegisterClass("LocalJumpError", typeof(Ruby.Builtins.LocalJumpError), new System.Action<Ruby.Builtins.RubyModule>(LoadLocalJumpError), TODO);
            context.RegisterClass("RegexpError", typeof(Ruby.Builtins.RegexpError), new System.Action<Ruby.Builtins.RubyModule>(LoadRegexpError), TODO);
            context.RegisterClass("RuntimeError", typeof(Ruby.Builtins.RuntimeError), new System.Action<Ruby.Builtins.RubyModule>(LoadRuntimeError), TODO);
            context.RegisterClass("ThreadError", typeof(Ruby.Builtins.ThreadError), new System.Action<Ruby.Builtins.RubyModule>(LoadThreadError), TODO);
            context.RegisterClass("UnboundMethod", typeof(Ruby.Builtins.UnboundMethod), new System.Action<Ruby.Builtins.RubyModule>(LoadUnboundMethod), TODO);
            context.RegisterClass("Hash", typeof(System.Collections.Generic.Dictionary<System.Object, System.Object>), new System.Action<Ruby.Builtins.RubyModule>(LoadHash), TODO);
            context.RegisterClass("Symbol", typeof(Microsoft.Scripting.SymbolId), new System.Action<Ruby.Builtins.RubyModule>(LoadSymbol), TODO);
            context.RegisterClass("TrueClass", typeof(Ruby.Builtins.TrueClass), new System.Action<Ruby.Builtins.RubyModule>(LoadTrueClass), TODO);
            context.RegisterClass("String", typeof(System.String), new System.Action<Ruby.Builtins.RubyModule>(LoadString), TODO);
            context.RegisterClass("Regexp", typeof(Ruby.Builtins.Regexp), new System.Action<Ruby.Builtins.RubyModule>(LoadRegexp), TODO);
            context.RegisterClass("NilClass", typeof(Microsoft.Scripting.None), new System.Action<Ruby.Builtins.RubyModule>(LoadNilClass), TODO);
            context.RegisterClass("Range", typeof(Ruby.Builtins.Range), new System.Action<Ruby.Builtins.RubyModule>(LoadRange), TODO);
            context.RegisterClass("FalseClass", typeof(Ruby.Builtins.FalseClass), new System.Action<Ruby.Builtins.RubyModule>(LoadFalseClass), TODO);
            context.RegisterClass("Numeric", typeof(Ruby.Builtins.Numeric), new System.Action<Ruby.Builtins.RubyModule>(LoadNumeric), TODO);
            context.RegisterClass("Proc", typeof(Ruby.Builtins.Proc), new System.Action<Ruby.Builtins.RubyModule>(LoadProc), TODO);
            context.RegisterClass("Fixnum", typeof(System.Int32), new System.Action<Ruby.Builtins.RubyModule>(LoadFixnum), TODO);
            context.RegisterClass("Array", typeof(System.Collections.Generic.List<System.Object>), new System.Action<Ruby.Builtins.RubyModule>(LoadArray), TODO);
            context.RegisterClass("Module", typeof(Ruby.Builtins.RubyModule), new System.Action<Ruby.Builtins.RubyModule>(LoadModule), TODO);
            context.RegisterClass("Method", typeof(Ruby.Builtins.Method), new System.Action<Ruby.Builtins.RubyModule>(LoadMethod), TODO);
            context.RegisterClass("Exception", typeof(System.Exception), new System.Action<Ruby.Builtins.RubyModule>(LoadException), TODO);
            context.RegisterClass("Integer", typeof(Ruby.Builtins.Integer), new System.Action<Ruby.Builtins.RubyModule>(LoadInteger), TODO);
            context.RegisterClass("Class", typeof(Ruby.Builtins.RubyClass), new System.Action<Ruby.Builtins.RubyModule>(LoadClass), TODO);
            context.RegisterClass("ScriptError", typeof(Ruby.Builtins.ScriptError), new System.Action<Ruby.Builtins.RubyModule>(LoadScriptError), TODO);
            context.RegisterClass("NoMemoryError", typeof(Ruby.Builtins.NoMemoryError), new System.Action<Ruby.Builtins.RubyModule>(LoadNoMemoryError), TODO);
            context.RegisterClass("SystemStackError", typeof(Ruby.Builtins.SystemStackError), new System.Action<Ruby.Builtins.RubyModule>(LoadSystemStackError), TODO);
            context.RegisterClass("Bignum", typeof(Microsoft.Scripting.Math.BigInteger), new System.Action<Ruby.Builtins.RubyModule>(LoadBignum), TODO);
            context.RegisterClass("LoadError", typeof(Ruby.Builtins.LoadError), new System.Action<Ruby.Builtins.RubyModule>(LoadLoadError), TODO);
            context.RegisterClass("NotImplementedError", typeof(Ruby.Builtins.NotImplementedError), new System.Action<Ruby.Builtins.RubyModule>(LoadNotImplementedError), TODO);
            context.RegisterClass("SyntaxError", typeof(Ruby.Builtins.SyntaxError), new System.Action<Ruby.Builtins.RubyModule>(LoadSyntaxError), TODO);
        }
        
        private static void LoadUnboundMethod(Ruby.Builtins.RubyModule/*!*/ module) {
        }
        
        private static void LoadHash(Ruby.Builtins.RubyModule/*!*/ module) {
            module.DefineClassMethod("[]", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<System.Object[], System.Collections.Generic.Dictionary<System.Object, System.Object>>(Ruby.Builtins.HashOps.CreateHash),
            });
            
        }
        
        private static void LoadSymbol(Ruby.Builtins.RubyModule/*!*/ module) {
        }
        
        private static void LoadTrueClass(Ruby.Builtins.RubyModule/*!*/ module) {
            module.DefineInstanceMethod("to_s", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<System.Object, System.Object>(Ruby.Builtins.TrueClass.ToString),
            });
            
        }
        
        private static void LoadString(Ruby.Builtins.RubyModule/*!*/ module) {
            module.DefineInstanceMethod("[]", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<System.String, System.Int32, System.String>(Ruby.Builtins.StringOps.GetChar),
                new Microsoft.Scripting.Function<System.String, Ruby.Builtins.Range, System.String>(Ruby.Builtins.StringOps.GetChar),
                new Microsoft.Scripting.Function<System.String, System.Int32, System.Int32, System.String>(Ruby.Builtins.StringOps.GetChar),
            });
            
            module.DefineInstanceMethod("index", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<System.String, System.String, System.Int32>(Ruby.Builtins.StringOps.IndexOf),
            });
            
            module.DefineInstanceMethod("split", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<System.String, System.Collections.Generic.List<System.Object>>(Ruby.Builtins.StringOps.Split),
                new Microsoft.Scripting.Function<System.String, Ruby.Builtins.Regexp, System.Collections.Generic.List<System.Object>>(Ruby.Builtins.StringOps.Split),
                new Microsoft.Scripting.Function<System.String, Ruby.Builtins.Regexp, System.Int32, System.Collections.Generic.List<System.Object>>(Ruby.Builtins.StringOps.Split),
            });
            
        }
        
        private static void LoadComparable(Ruby.Builtins.RubyModule/*!*/ module) {
        }
        
        private static void LoadEnumerable(Ruby.Builtins.RubyModule/*!*/ module) {
        }
        
        private static void LoadRegexp(Ruby.Builtins.RubyModule/*!*/ module) {
        }
        
        private static void LoadNilClass(Ruby.Builtins.RubyModule/*!*/ module) {
            module.DefineInstanceMethod("to_s", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<System.Object, System.Object>(Ruby.Builtins.NilClassOps.ToString),
            });
            
        }
        
        private static void LoadRange(Ruby.Builtins.RubyModule/*!*/ module) {
        }
        
        private static void LoadFalseClass(Ruby.Builtins.RubyModule/*!*/ module) {
            module.DefineInstanceMethod("to_s", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<System.Object, System.Object>(Ruby.Builtins.FalseClass.ToString),
            });
            
        }
        
        private static void LoadMath(Ruby.Builtins.RubyModule/*!*/ module) {
        }
        
        private static void LoadNumeric(Ruby.Builtins.RubyModule/*!*/ module) {
            module.DefineInstanceMethod("+@", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<Ruby.Builtins.Numeric>(Ruby.Builtins.Numeric.UnaryPlus),
            });
            
            module.DefineInstanceMethod("-@", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<Ruby.Builtins.Numeric>(Ruby.Builtins.Numeric.UnaryMinus),
            });
            
            module.DefineInstanceMethod("<=>", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<System.Object>(Ruby.Builtins.Numeric.Compare),
            });
            
        }
        
        private static void LoadInteger(Ruby.Builtins.RubyModule/*!*/ module) {
            module.DefineInstanceMethod("times", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<Microsoft.Scripting.CodeContext, System.Int32, Ruby.Builtins.Proc, System.Object>(Ruby.Builtins.Integer.Times),
                new Microsoft.Scripting.Function<Microsoft.Scripting.CodeContext, Microsoft.Scripting.Math.BigInteger, Ruby.Builtins.Proc, System.Object>(Ruby.Builtins.Integer.Times),
            });
            
        }
        
        private static void LoadBignum(Ruby.Builtins.RubyModule/*!*/ module) {
        }
        
        private static void LoadProc(Ruby.Builtins.RubyModule/*!*/ module) {
            module.DefineInstanceMethod("call", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<Ruby.Builtins.Proc, System.Object[], System.Object>(Ruby.Builtins.Proc.Call),
            });
            
            module.DefineInstanceMethod("[]", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<Ruby.Builtins.Proc, System.Object[], System.Object>(Ruby.Builtins.Proc.Call),
            });
            
        }
        
        private static void LoadFixnum(Ruby.Builtins.RubyModule/*!*/ module) {
            module.DefineInstanceMethod("+", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<System.Int32, System.Int32, System.Object>(Ruby.Builtins.FixnumOps.Add),
                new Microsoft.Scripting.Function<System.Int32, Microsoft.Scripting.Math.BigInteger, Microsoft.Scripting.Math.BigInteger>(Ruby.Builtins.FixnumOps.Add),
            });
            
            module.DefineInstanceMethod("-", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<System.Int32, System.Int32, System.Object>(Ruby.Builtins.FixnumOps.Subtract),
            });
            
            module.DefineInstanceMethod("*", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<System.Int32, System.Int32, System.Object>(Ruby.Builtins.FixnumOps.Multiply),
            });
            
            module.DefineInstanceMethod("/", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<System.Int32, System.Int32, System.Object>(Ruby.Builtins.FixnumOps.Divide),
            });
            
            module.DefineInstanceMethod(">", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<System.Int32, System.Int32, System.Boolean>(Ruby.Builtins.FixnumOps.GraterThan),
            });
            
            module.DefineInstanceMethod(">=", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<System.Int32, System.Int32, System.Boolean>(Ruby.Builtins.FixnumOps.GreaterThanOrEqual),
            });
            
            module.DefineInstanceMethod("<", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<System.Int32, System.Int32, System.Boolean>(Ruby.Builtins.FixnumOps.LessThan),
            });
            
            module.DefineInstanceMethod("<=", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<System.Int32, System.Int32, System.Boolean>(Ruby.Builtins.FixnumOps.LessThanOrEqual),
            });
            
            module.DefineInstanceMethod("==", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<System.Int32, System.Int32, System.Boolean>(Ruby.Builtins.FixnumOps.Equal),
            });
            
        }
        
        private static void LoadArray(Ruby.Builtins.RubyModule/*!*/ module) {
            module.DefineInstanceMethod("[]", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<System.Object[], System.Object>(Ruby.Builtins.ArrayOps.GetElement),
            });
            
            module.DefineInstanceMethod("slice", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<System.Object[], System.Object>(Ruby.Builtins.ArrayOps.GetElement),
            });
            
            module.DefineInstanceMethod("[]=", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<System.Object, System.Object, System.Object, System.Object>(Ruby.Builtins.ArrayOps.SetElement),
            });
            
            module.DefineInstanceMethod("==", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<Microsoft.Scripting.CodeContext, System.Object, System.Object, System.Object>(Ruby.Builtins.ArrayOps.Equals),
            });
            
            module.DefineInstanceMethod("clear", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<System.Object, System.Object>(Ruby.Builtins.ArrayOps.Clear),
            });
            
            module.DefineInstanceMethod("first", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<System.Object, System.Object>(Ruby.Builtins.ArrayOps.First),
            });
            
            module.DefineInstanceMethod("last", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<System.Object, System.Object>(Ruby.Builtins.ArrayOps.Last),
            });
            
            module.DefineInstanceMethod("length", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<System.Object, System.Object>(Ruby.Builtins.ArrayOps.Length),
            });
            
            module.DefineInstanceMethod("size", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<System.Object, System.Object>(Ruby.Builtins.ArrayOps.Length),
            });
            
            module.DefineInstanceMethod("nitems", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<System.Object, System.Object>(Ruby.Builtins.ArrayOps.NumberOfNonNilItems),
            });
            
            module.DefineInstanceMethod("reverse", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<System.Object, System.Object>(Ruby.Builtins.ArrayOps.Reverse),
            });
            
            module.DefineInstanceMethod("reverse!", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<System.Object, System.Object>(Ruby.Builtins.ArrayOps.InPlaceReverse),
            });
            
            module.DefineInstanceMethod("<<", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<System.Object, System.Object, System.Object>(Ruby.Builtins.ArrayOps.Append),
            });
            
            module.DefineInstanceMethod("pop", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<System.Object, System.Object>(Ruby.Builtins.ArrayOps.Pop),
            });
            
            module.DefineInstanceMethod("push", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<System.Object[], System.Object>(Ruby.Builtins.ArrayOps.Push),
            });
            
            module.DefineInstanceMethod("+", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<System.Object, System.Object, System.Object>(Ruby.Builtins.ArrayOps.Concatenate),
            });
            
            module.DefineInstanceMethod("join", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<System.Object, System.Object, System.Object>(Ruby.Builtins.ArrayOps.Join),
            });
            
            module.DefineInstanceMethod("to_s", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<System.Object, System.Object>(Ruby.Builtins.ArrayOps.ToString),
            });
            
            module.DefineInstanceMethod("*", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<System.Object, System.Object, System.Object>(Ruby.Builtins.ArrayOps.Repetition),
            });
            
            module.DefineInstanceMethod("each", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<Microsoft.Scripting.CodeContext, System.Collections.Generic.List<System.Object>, Ruby.Builtins.Proc, System.Collections.Generic.List<System.Object>>(Ruby.Builtins.ArrayOps.Each),
            });
            
            module.DefineInstanceMethod("each_index", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<Microsoft.Scripting.CodeContext, System.Object, System.Object, System.Object>(Ruby.Builtins.ArrayOps.EachIndex),
            });
            
            module.DefineInstanceMethod("collect!", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<Microsoft.Scripting.CodeContext, System.Object, System.Object, System.Object>(Ruby.Builtins.ArrayOps.Collect),
            });
            
            module.DefineInstanceMethod("empty?", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<System.Object, System.Object>(Ruby.Builtins.ArrayOps.Empty),
            });
            
            module.DefineClassMethod("[]", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<System.Object, System.Object[], System.Collections.Generic.List<System.Object>>(Ruby.Builtins.ArrayOps.CreateArray),
            });
            
        }
        
        private static void LoadModule(Ruby.Builtins.RubyModule/*!*/ module) {
            module.DefineInstanceMethod("private", Ruby.Runtime.RubyMethodVisibility.Private, new System.Delegate[] {
                new Microsoft.Scripting.Function<Microsoft.Scripting.CodeContext, Ruby.Builtins.RubyModule, Microsoft.Scripting.SymbolId[], Ruby.Builtins.RubyModule>(Ruby.Builtins.RubyModule.SetPrivateVisibility),
            });
            
            module.DefineInstanceMethod("protected", Ruby.Runtime.RubyMethodVisibility.Private, new System.Delegate[] {
                new Microsoft.Scripting.Function<Microsoft.Scripting.CodeContext, Ruby.Builtins.RubyModule, Microsoft.Scripting.SymbolId[], Ruby.Builtins.RubyModule>(Ruby.Builtins.RubyModule.SetProtectedVisibility),
            });
            
            module.DefineInstanceMethod("public", Ruby.Runtime.RubyMethodVisibility.Private, new System.Delegate[] {
                new Microsoft.Scripting.Function<Microsoft.Scripting.CodeContext, Ruby.Builtins.RubyModule, Microsoft.Scripting.SymbolId[], Ruby.Builtins.RubyModule>(Ruby.Builtins.RubyModule.SetPublicVisibility),
            });
            
            module.DefineInstanceMethod("===", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<Ruby.Builtins.RubyModule, System.Object, System.Boolean>(Ruby.Builtins.RubyModule.CaseEquals),
            });
            
            module.DefineInstanceMethod("class_eval", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<System.Object, System.String, System.Object>(Ruby.Builtins.RubyModule.Evaluate),
                new Microsoft.Scripting.Function<System.Object, System.String, System.String, System.Object>(Ruby.Builtins.RubyModule.Evaluate),
                new Microsoft.Scripting.Function<System.Object, System.String, System.String, System.Int32, System.Object>(Ruby.Builtins.RubyModule.Evaluate),
                new Microsoft.Scripting.Function<System.Object, System.Object, System.Object>(Ruby.Builtins.RubyModule.Evaluate),
            });
            
            module.DefineInstanceMethod("constants", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<Ruby.Builtins.RubyModule, System.Collections.Generic.List<System.Object>>(Ruby.Builtins.RubyModule.GetDefinedConstants),
            });
            
            module.DefineInstanceMethod("instance_method", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<Ruby.Builtins.RubyModule, Microsoft.Scripting.SymbolId, Ruby.Builtins.UnboundMethod>(Ruby.Builtins.RubyModule.GetInstanceMethod),
            });
            
        }
        
        private static void LoadKernel(Ruby.Builtins.RubyModule/*!*/ module) {
            module.DefineInstanceMethod("exit", Ruby.Runtime.RubyMethodVisibility.Private, new System.Delegate[] {
                new System.Action<System.Object>(Ruby.Builtins.Kernel.Exit),
                new System.Action<System.Object, System.Boolean>(Ruby.Builtins.Kernel.Exit),
                new System.Action<System.Object, System.Int32>(Ruby.Builtins.Kernel.Exit),
            });
            
            module.DefineInstanceMethod("exit!", Ruby.Runtime.RubyMethodVisibility.Private, new System.Delegate[] {
                new System.Action<System.Object>(Ruby.Builtins.Kernel.TerminateExecution),
                new System.Action<System.Object, System.Boolean>(Ruby.Builtins.Kernel.TerminateExecution),
                new System.Action<System.Object, System.Int32>(Ruby.Builtins.Kernel.TerminateExecution),
            });
            
            module.DefineInstanceMethod("load", Ruby.Runtime.RubyMethodVisibility.Private, new System.Delegate[] {
                new Microsoft.Scripting.Function<Microsoft.Scripting.CodeContext, System.Object, System.Object, System.Object>(Ruby.Builtins.Kernel.Load),
                new Microsoft.Scripting.Function<Microsoft.Scripting.CodeContext, System.Object, System.Object, System.Object, System.Object>(Ruby.Builtins.Kernel.Load),
            });
            
            module.DefineInstanceMethod("method_missing", Ruby.Runtime.RubyMethodVisibility.Private, new System.Delegate[] {
                new Microsoft.Scripting.Function<Microsoft.Scripting.CodeContext, System.Object, Ruby.Builtins.Proc, System.String, System.Object[], System.Object>(Ruby.Builtins.Kernel.MethodMissing),
            });
            
            module.DefineInstanceMethod("print", Ruby.Runtime.RubyMethodVisibility.Private, new System.Delegate[] {
                new System.Action<System.Object, System.Object[]>(Ruby.Builtins.Kernel.Print),
            });
            
            module.DefineInstanceMethod("puts", Ruby.Runtime.RubyMethodVisibility.Private, new System.Delegate[] {
                new System.Action<System.Object, System.Object[]>(Ruby.Builtins.Kernel.PutString),
            });
            
            module.DefineInstanceMethod("raise", Ruby.Runtime.RubyMethodVisibility.Private, new System.Delegate[] {
                new System.Action<Microsoft.Scripting.CodeContext, System.Object>(Ruby.Builtins.Kernel.RaiseException),
                new System.Action<System.Object, System.String>(Ruby.Builtins.Kernel.RaiseException),
                new System.Action<Microsoft.Scripting.CodeContext, System.Object, Ruby.Builtins.RubyClass>(Ruby.Builtins.Kernel.RaiseException),
                new System.Action<Microsoft.Scripting.CodeContext, System.Object, Ruby.Builtins.RubyClass, System.String>(Ruby.Builtins.Kernel.RaiseException),
                new System.Action<Microsoft.Scripting.CodeContext, System.Object, Ruby.Builtins.RubyClass, System.String, System.Collections.Generic.List<System.Object>>(Ruby.Builtins.Kernel.RaiseException),
            });
            
            module.DefineInstanceMethod("fail", Ruby.Runtime.RubyMethodVisibility.Private, new System.Delegate[] {
                new System.Action<Microsoft.Scripting.CodeContext, System.Object>(Ruby.Builtins.Kernel.RaiseException),
                new System.Action<System.Object, System.String>(Ruby.Builtins.Kernel.RaiseException),
                new System.Action<Microsoft.Scripting.CodeContext, System.Object, Ruby.Builtins.RubyClass>(Ruby.Builtins.Kernel.RaiseException),
                new System.Action<Microsoft.Scripting.CodeContext, System.Object, Ruby.Builtins.RubyClass, System.String>(Ruby.Builtins.Kernel.RaiseException),
                new System.Action<Microsoft.Scripting.CodeContext, System.Object, Ruby.Builtins.RubyClass, System.String, System.Collections.Generic.List<System.Object>>(Ruby.Builtins.Kernel.RaiseException),
            });
            
            module.DefineInstanceMethod("require", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<Microsoft.Scripting.CodeContext, System.Object, System.String, System.Boolean>(Ruby.Builtins.Kernel.Require),
            });
            
            module.DefineClassMethod("ruby_case_set", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<System.Object, System.Object, System.Object>(Ruby.Builtins.Kernel.RubyCaseSet),
            });
            
            module.DefineClassMethod("exit", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new System.Action<System.Object>(Ruby.Builtins.Kernel.Exit),
                new System.Action<System.Object, System.Boolean>(Ruby.Builtins.Kernel.Exit),
                new System.Action<System.Object, System.Int32>(Ruby.Builtins.Kernel.Exit),
            });
            
            module.DefineClassMethod("exit!", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new System.Action<System.Object>(Ruby.Builtins.Kernel.TerminateExecution),
                new System.Action<System.Object, System.Boolean>(Ruby.Builtins.Kernel.TerminateExecution),
                new System.Action<System.Object, System.Int32>(Ruby.Builtins.Kernel.TerminateExecution),
            });
            
            module.DefineClassMethod("load", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<Microsoft.Scripting.CodeContext, System.Object, System.Object, System.Object>(Ruby.Builtins.Kernel.Load),
                new Microsoft.Scripting.Function<Microsoft.Scripting.CodeContext, System.Object, System.Object, System.Object, System.Object>(Ruby.Builtins.Kernel.Load),
            });
            
            module.DefineClassMethod("method_missing", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<Microsoft.Scripting.CodeContext, System.Object, Ruby.Builtins.Proc, System.String, System.Object[], System.Object>(Ruby.Builtins.Kernel.MethodMissing),
            });
            
            module.DefineClassMethod("print", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new System.Action<System.Object, System.Object[]>(Ruby.Builtins.Kernel.Print),
            });
            
            module.DefineClassMethod("puts", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new System.Action<System.Object, System.Object[]>(Ruby.Builtins.Kernel.PutString),
            });
            
            module.DefineClassMethod("fail", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new System.Action<Microsoft.Scripting.CodeContext, System.Object>(Ruby.Builtins.Kernel.RaiseException),
                new System.Action<System.Object, System.String>(Ruby.Builtins.Kernel.RaiseException),
                new System.Action<Microsoft.Scripting.CodeContext, System.Object, Ruby.Builtins.RubyClass>(Ruby.Builtins.Kernel.RaiseException),
                new System.Action<Microsoft.Scripting.CodeContext, System.Object, Ruby.Builtins.RubyClass, System.String>(Ruby.Builtins.Kernel.RaiseException),
                new System.Action<Microsoft.Scripting.CodeContext, System.Object, Ruby.Builtins.RubyClass, System.String, System.Collections.Generic.List<System.Object>>(Ruby.Builtins.Kernel.RaiseException),
            });
            
            module.DefineClassMethod("raise", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new System.Action<Microsoft.Scripting.CodeContext, System.Object>(Ruby.Builtins.Kernel.RaiseException),
                new System.Action<System.Object, System.String>(Ruby.Builtins.Kernel.RaiseException),
                new System.Action<Microsoft.Scripting.CodeContext, System.Object, Ruby.Builtins.RubyClass>(Ruby.Builtins.Kernel.RaiseException),
                new System.Action<Microsoft.Scripting.CodeContext, System.Object, Ruby.Builtins.RubyClass, System.String>(Ruby.Builtins.Kernel.RaiseException),
                new System.Action<Microsoft.Scripting.CodeContext, System.Object, Ruby.Builtins.RubyClass, System.String, System.Collections.Generic.List<System.Object>>(Ruby.Builtins.Kernel.RaiseException),
            });
            
            module.DefineClassMethod("require", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<Microsoft.Scripting.CodeContext, System.Object, System.String, System.Boolean>(Ruby.Builtins.Kernel.Require),
            });
            
        }
        
        private static void LoadMethod(Ruby.Builtins.RubyModule/*!*/ module) {
        }
        
        private static void LoadClass(Ruby.Builtins.RubyModule/*!*/ module) {
            module.DefineInstanceMethod("allocate", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<Ruby.Builtins.RubyClass, System.Object>(Ruby.Builtins.RubyClass.Allocate),
            });
            
            module.DefineInstanceMethod("new", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<Ruby.Builtins.RubyClass, System.Object[], System.Object>(Ruby.Builtins.RubyClass.New),
            });
            
            module.DefineInstanceMethod("name", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<Ruby.Builtins.RubyClass, System.String>(Ruby.Builtins.RubyClass.GetName),
            });
            
            module.DefineInstanceMethod("to_s", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<Ruby.Builtins.RubyClass, System.String>(Ruby.Builtins.RubyClass.GetName),
            });
            
            module.DefineInstanceMethod("superclass", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<Ruby.Builtins.RubyClass, Ruby.Builtins.RubyClass>(Ruby.Builtins.RubyClass.GetSuperclass),
            });
            
        }
        
        private static void LoadObject(Ruby.Builtins.RubyModule/*!*/ module) {
            module.DefineInstanceMethod("to_s", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<Microsoft.Scripting.CodeContext, System.Object, System.String>(Ruby.Builtins.ObjectOps.ToString),
            });
            
            module.DefineInstanceMethod("class", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<Microsoft.Scripting.CodeContext, System.Object, Ruby.Builtins.RubyClass>(Ruby.Builtins.ObjectOps.GetClass),
            });
            
            module.DefineInstanceMethod("id", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<System.Object, System.Int32>(Ruby.Builtins.ObjectOps.GetId),
            });
            
            module.DefineInstanceMethod("object_id", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<System.Object, System.Int32>(Ruby.Builtins.ObjectOps.GetObjectId),
            });
            
            module.DefineInstanceMethod("==", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<System.Object, System.Object, System.Boolean>(Ruby.Builtins.ObjectOps.Equals),
            });
            
            module.DefineInstanceMethod("nil?", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<System.Object, System.Boolean>(Ruby.Builtins.ObjectOps.IsNil),
            });
            
            module.DefineInstanceMethod("kind_of?", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<System.Object, Ruby.Builtins.RubyModule, System.Boolean>(Ruby.Builtins.ObjectOps.IsKindOf),
            });
            
            module.DefineInstanceMethod("is_a?", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<System.Object, Ruby.Builtins.RubyModule, System.Boolean>(Ruby.Builtins.ObjectOps.IsKindOf),
            });
            
            module.DefineInstanceMethod("instance_of?", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<System.Object, Ruby.Builtins.RubyModule, System.Boolean>(Ruby.Builtins.ObjectOps.IsOfClass),
            });
            
        }
        
        private static void LoadScriptError(Ruby.Builtins.RubyModule/*!*/ module) {
        }
        
        private static void LoadLoadError(Ruby.Builtins.RubyModule/*!*/ module) {
        }
        
        private static void LoadNotImplementedError(Ruby.Builtins.RubyModule/*!*/ module) {
        }
        
        private static void LoadSyntaxError(Ruby.Builtins.RubyModule/*!*/ module) {
        }
        
        private static void LoadNoMemoryError(Ruby.Builtins.RubyModule/*!*/ module) {
        }
        
        private static void LoadSystemStackError(Ruby.Builtins.RubyModule/*!*/ module) {
        }
        
        private static void LoadEOFError(Ruby.Builtins.RubyModule/*!*/ module) {
        }
        
        private static void LoadFloatDomainError(Ruby.Builtins.RubyModule/*!*/ module) {
        }
        
        private static void LoadLocalJumpError(Ruby.Builtins.RubyModule/*!*/ module) {
        }
        
        private static void LoadRegexpError(Ruby.Builtins.RubyModule/*!*/ module) {
        }
        
        private static void LoadRuntimeError(Ruby.Builtins.RubyModule/*!*/ module) {
        }
        
        private static void LoadThreadError(Ruby.Builtins.RubyModule/*!*/ module) {
        }
        
        private static void LoadException(Ruby.Builtins.RubyModule/*!*/ module) {
            module.DefineInstanceMethod("backtrace", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<System.Exception, System.Collections.Generic.List<System.Object>>(Ruby.Builtins.ExceptionOps.GetBacktrace),
            });
            
            module.DefineInstanceMethod("set_backtrace", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<System.Object, System.Object, System.Object>(Ruby.Builtins.ExceptionOps.SetBacktrace),
            });
            
            module.DefineInstanceMethod("exception", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<System.Object, System.Object>(Ruby.Builtins.ExceptionOps.GetException),
                new Microsoft.Scripting.Function<Microsoft.Scripting.CodeContext, System.Object, System.Object, System.Object>(Ruby.Builtins.ExceptionOps.GetException),
            });
            
            module.DefineInstanceMethod("to_s", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<System.Object, System.Object>(Ruby.Builtins.ExceptionOps.GetMessage),
            });
            
            module.DefineInstanceMethod("message", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<System.Object, System.Object>(Ruby.Builtins.ExceptionOps.GetMessage),
            });
            
            module.DefineClassMethod("exception", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<System.Object>(Ruby.Builtins.ExceptionOps.CreateException),
                new Microsoft.Scripting.Function<System.Object, System.Object>(Ruby.Builtins.ExceptionOps.CreateException),
            });
            
            module.DefineClassMethod("new", Ruby.Runtime.RubyMethodVisibility.Public, new System.Delegate[] {
                new Microsoft.Scripting.Function<System.Object>(Ruby.Builtins.ExceptionOps.CreateException),
                new Microsoft.Scripting.Function<System.Object, System.Object>(Ruby.Builtins.ExceptionOps.CreateException),
            });
            
        }
        
    }
}
